#!/usr/bin/python

import sqlite3
from pysqlcipher import dbapi2 as sqlcipher
import sys
import time
import pragma
from logger import Logger

#946684800 = 1/1/2000:00:00:00


def uTime2dbTime(utime):
    return time.strftime("%d-%m-%y %H:%M:%S", time.localtime(utime))


def dbTime2uTime(atime):
    dt = atime.split(' ')
    d = dt[0].split('-')
    t = dt[1].split(':')
    return time.mktime((int(d[2]) + 2000, int(d[1]), int(d[0]), int(t[0]), int(t[1]), int(t[2]), 0, 1, -1))
 

class DatabaseConnection():

  def __init__(self, fileName):
    self.fileName = fileName   
    self.db_open = False
    self.syncdMesgs = ""

  # make sure the database connection is closed when object is destroyed
  def __del__(self):
    #Logger.debug("Database", "Deleting Database")
    self.close()
  
  def connect(self):
    #Logger.debug("Database", "connect to database")
    if (self.db_open == True):
       return True
    try:
      self.con = sqlcipher.connect(self.fileName)
      c = self.con.cursor()
      c.execute('PRAGMA key=' + pragma.key)
      c.execute('pragma kdf_iter=64000;');
      c.execute('pragma busy_timeout=60000 ;');
      c.execute('pragma cache_size=4000;');
      c.execute('pragma synchronous=2;');
      self.db_open = True
      #Logger.debug("Database", 'database opened')
      return True   
    except:
      Logger.debug("Database", sys.exc_info() )
      Logger.debug("Database", "!!!!! unable to connect to db !!!!!")
      return False     

  def close(self):
    if self.con:
      self.con.close()
    self.db_open = False

  def query(self, queryString):
    #Logger.debug("Database", "query: " + queryString )
    queryResult = [None]
    if self.connect():
      try:
        queryResult = self.con.execute(queryString).fetchone()    
      except:
        queryResult = None
      self.close()
    return queryResult

  def queryGetColumn(self, queryString):
    #Logger.debug("Database", "queryGetColumn: " + queryString )
    queryResult = [None]
    if self.connect():
      try:
        queryResult = self.con.execute(queryString).fetchall()
      except:
        queryResult = None
      self.close()
    return queryResult

  def dictionaryGet(self, name):
    #Logger.debug("Database", "dictionaryGet: " + name )
    queryResult = ""
    if self.connect():
      try:
        queryResult = self.con.execute("SELECT value FROM dictionary WHERE name='" + name + "'").fetchone()      
        queryResult = queryResult[0]
      except:
        queryResult = ""
      self.close()
    return queryResult

  def dictionarySet(self, name, value):
    #Logger.debug("Database", "dictionarySet: " + name + " = " + value )
    queryResult = None
    update = self.dictionaryExists(name)

    if self.connect():
      if update:
	       queryResult = self.con.execute("UPDATE dictionary SET value='" + value + "' WHERE name='" + name + "'")
      else:
	       queryResult = self.con.execute("INSERT INTO dictionary (name, value) VALUES ('" + name + "','" + value + "')")
      self.con.commit()
      self.close()
    return
  
  def dictionaryExists(self, name):
    #Logger.debug("Database", "dictionaryExists: " + name )
    queryResult = None
    self.connect()
    if self.connect() and self.con.execute("SELECT * FROM dictionary WHERE name='" + name + "'").fetchone():
      return True
    return False;

  def manufacturerSetupGet(self, name):
    queryResult = ""
    if self.connect():
      try:
        queryResult = self.con.execute("SELECT value FROM manufacturerSetup WHERE name='" + name + "'").fetchone()      
        queryResult = queryResult[0]
      except:
        queryResult = ""
      self.close()
    return queryResult
      
  def saveVideoPath(self, path):
    return self.dictionarySet("videoPath", path)
 
  def UpdateStats(self, uri, media_source):
    #Logger.debug("Database", "UpdateStats: " + uri + ", " + media_source )
    # check if a record exits
    self.connect()
    queryResult = None
    self.connect()
    cur = self.con.cursor()
    queryString = "SELECT COUNT(*) from playListStatistics where uri='" + uri + "'";
    #Logger.debug("Database", queryString)
    queryResult = self.con.execute(queryString).fetchone()
    count = queryResult[0]
    if count == 0:
      queryString = "INSERT INTO playlistStatistics Values(null, '" + uri +"', '" + media_source +"', " + str(int(time.time())) + "," + str(int(time.time())) + ", 1, '')"
      #Logger.debug("Database", queryString)
      cur.execute(queryString)
      self.con.commit()
    else:
      queryString = "SELECT  play_count from playListStatistics where uri='" + uri + "'";

      #Logger.debug("Database", queryString)
      queryResult = self.con.execute(queryString).fetchone()
      count = queryResult[0]
      count += 1
      queryString = "Update playListStatistics set play_count=" + str(count) + ", last_play=" + str(int(time.time())) + " where uri='" + uri + "'"
      #Logger.debug("Database", queryString)
      cur.execute(queryString)
      self.con.commit()
      self.close()
      
    

  def insertSystemMessage(self,  service_name, mtype, msg):
      self.connect()
      cur = self.con.cursor()
      date =  time.strftime("%d-%m-%y %H:%M:%S", time.localtime(time.time()))
      queryString = "INSERT INTO systemMessages (name, type, date, msg) \
                     VALUES('" + service_name + "', '" + \
                     mtype + "','" + date + "','" + msg + "')"
      #Logger.debug("Database", queryString)
      cur.execute(queryString)
      self.con.commit()
      self.close()

  def getSystemErrorMessages(self, last_time):
      self.connect()
      cur = self.con.cursor()
      date =  time.strftime("%m-%d-%y %H:%M:%S", time.localtime(time.time()))
      queryString = "Select  date, name, msg from systemMessages where (type = 'Error' or type = 'Critical')"

      #Logger.debug("Database", queryString)
      log = cur.execute(queryString).fetchall()
      new_errors = []
      for l in log:
        etime = dbTime2uTime(str(l[0]))
        if (etime > last_time):
            new_errors.append((etime, l[1], l[2]))
      self.close()
      return new_errors


  def setIotSynced(self):
    self.connect()
    curr = self.con.cursor()
    update_string = "update pourStatisticsCounts set iot_synced=1 where iot_synced!=1"
    #Logger.debug("Database", "setIotSynced: " + update_string)
    try:
      curr.execute(update_string)
      self.con.commit()
    except:
      Logger.error("Database",  "Error Updating database")
    self.close()  

  def getPourStats(self):    
    self.connect()
    curr = self.con.cursor()
    query = "SELECT psc.id AS id, "\
            + "s.name AS syrup_name, "\
            + "ps.flavor_combination AS flavor_ids, "\
            + "psc.date AS date, "\
            + "psc.count AS count, "\
            + "psc.time AS time "\
            + "FROM pourStatisticsCounts psc, pourStatistics ps, syrups s "\
            + "WHERE psc.pour_statistics_id=ps.id AND ps.syrup_id=s.id AND psc.iot_synced!=1 " \
            + "ORDER by psc.id DESC "
    #Logger.debug("Database", "getPourStats: " + query)
    curr.execute(query)
    rows = curr.fetchall()
    self.close()
    return rows


  def getSyrupBrandConfig(self):    
      self.connect()
      curr = self.con.cursor()
      query = "Select id,type,name,imageName,ratio,leftValves,rightValves,waterValves From syrups where (leftValves > 0 or rightValves > 0) and type='brand';"     
      #Logger.debug("Database", "getSyrupBrandConfig: " + query)
      curr.execute(query)
      rows = curr.fetchall()
      self.close()
      return rows

  def getSyrupFlavorConfig(self):    
      self.connect()
      curr = self.con.cursor()
      query = "Select id,type,name,imageName,ratio,leftValves,rightValves,waterValves From syrups where (leftValves > 0 or rightValves > 0) and type='flavor';"     
      #Logger.debug("Database", "getSyrupFlavorConfig: " + query)
      curr.execute(query)
      rows = curr.fetchall()
      self.close()
      return rows
  
  def getValveTotals(self):
      self.connect()
      curr = self.con.cursor()
      query = "Select name, msec from valveTime;"     
      #Logger.debug("Database", "getValveTotals: " + query)
      curr.execute(query)
      rows = curr.fetchall()
      self.close()
      return rows

  def getWaterFlowRates(self):
      result = {}
      for water_type in ["CW", "MW", "PW"]:
          flow_rate = self.query("SELECT flowRate FROM waterMapping WHERE name='%s'" % water_type)
          if flow_rate is None:
              flow_rate = "2.5" # This is what people should be using in 99.9999% of cases.
          elif len(flow_rate) > 0:
              flow_rate = flow_rate[0]
          result[water_type] = flow_rate
      return result

  def getViperTotals(self):
      self.connect()
      curr = self.con.cursor()
      query = "Select total_name, value from viperTotals where iot_synced == 0;"     
      #Logger.debug("Database", "getViperTotals: " + query)
      curr.execute(query)
      rows = curr.fetchall()
      self.close()
      return rows

  def getViperConfig(self):
      self.connect()
      curr = self.con.cursor()
      query = "Select name, value from viperConfig where iot_synced == 0;"     
      #Logger.debug("Database", "getViperConfig: " + query)
      curr.execute(query)
      rows = curr.fetchall()
      self.close()
      return rows

  def getViperBrands( self ):
      self.connect()
      curr = self.con.cursor()
      query = "Select position, name, imageName from brands where position > 0;"     
      #Logger.debug("Database", "getViperBrands: " + query)
      curr.execute(query)
      rows = curr.fetchall()
      self.close()
      return rows

  def checkSysMesgsExt( self, severity, cutoff ):
      self.connect()
      curr = self.con.cursor()
      query = "Select count(*) from systemMessagesExt where iot_synced=0 and level >='%s' and time >= '%s';" % ( severity , cutoff )    
      count = self.query(query)
      # Logger.debug("Database", "checkSysMesgs: " + query + ", result: " + str(count[0]) )
        
      self.close()
      return count[0]

  # Result is 0=id, 1=time, 2=level, 3=module, 4=state, 5=msg_id, 6=text, 7=element
  def getSysMesgsExt( self, severity, cutoff ):
      self.connect()
      curr = self.con.cursor()
      # Get only 25 messages at a time to reduce connection hogging if
      # there are a lot of log entries.
      # Due to a bug in MIM (2/4/19), have to limit data to 1K, so reducing this to only 5 messages
      query = "Select id, time, level, module, state, msg_id, text, element from systemMessagesExt where iot_synced=0 and level >='%s' and time >= '%s' order by time desc limit 5;" % ( severity, cutoff )    
      #Logger.debug("Database", "getViperBrands: " + query)
      curr.execute(query)
      rows = curr.fetchall()
      # Create a string of all of the items that were extracted
      # so that they can be marked as sync'd later.
      # This is really convoluted single-line code, 
      # you have to read it from right to left...
      # Extract each row from all rows
      # Get the first item in the row, which is the id
      # Convert it to a string
      # Join (append) it to the string
      # Sperate each row id by a comma
      # So it takes this array: [[1,x,y,z,a,b,c],[2,x,y,z,a,b,c],[3,x,y,z,a,b,c]]
      # And turns it into this string: "1,2,3"
      self.syncdMesgs = ",".join( [str(row[0]) for row in rows] )
      self.close()
      return rows

  def setSysMesgsExtSyncd( self ):
      self.connect()
      curr = self.con.cursor()
      update_string = "update systemMessagesExt set iot_synced=1 where id in (" + self.syncdMesgs + ")"
      try:
        curr.execute( update_string )
        self.con.commit()
        self.syncdMesgs = ""
      except:
        Logger.error( "Database", "Error updating iot_synced in systemMessagesExt: " + update_string )

      self.close()
