#!/usr/bin/python

# ******************************************************************************
# *                                                                            
# *   Copyright 2014 Cornelius, Inc. All rights reserved.                      
# *                                                                            
# *   This file is confidential and a trade secret of Cornelius, Inc.          
# *   The receipt of or possession of this file does not convey any rights     
# *   to reproduce or disclose its contents or to manufacture, use, or sell    
# *   anything it may describe, in whole, or in part, without the specific     
# *   written consent of Cornelius, Inc.                                       
# *                                                                            
# **********************************************************x********************
# ------------------------------------------------------------------------------
#
#    DEVELOPED BY : Michal Augustyniak
#
# ------------------------------------------------------------------------------

softwareVersion="1.3.2"

import sys
import json
import time
import socket
import select
import os
import traceback
from urllib2 import urlparse

from ConfigParser import SafeConfigParser
from logger import Logger
from corcoMqttClient import CorcoMqttClient

from downloader import Downloader
from DBusWorker import DBusWorker
from spin import spin
import Queue
import netifaces
import DatabaseHelper

Logger.info("Gateway v" + softwareVersion, '*** LOADING ***')

database = DatabaseHelper.DatabaseConnection("/opt/Refresh/Database/refresh_database_enc.db")
systemType = database.dictionaryGet( "systemType" )
Logger.info( "Gateway", "System Type: " + systemType )

if systemType == "viper" or systemType == "vipergateway":
    from viperWrapper import RefreshWrapper
else:
    from refreshWrapper import RefreshWrapper

# save off our start time
boot_time =  time.time()
last_connect_time =  0

# Instantiate refresh wrapper
refreshWrapper = RefreshWrapper()

dBusWorker = DBusWorker()

download_complete_queue = Queue.Queue()
Downloader = Downloader(download_complete_queue)
Downloader.start()

autoConnect = False

# True/False
unitCommissioned = False

initialUpdate = True
updateConfigurationChanges = False
updateTotals = False
updateELog = False
updateErrors = False
updateMarketing = False
updateConnection = False
updateSysMsgsExt = False

# Minimum severity level of messages to report
reportSysMsgsExtSev = database.dictionaryGet( "sysMesgExtReportLevel" )

if reportSysMsgsExtSev == "":
    # 4 = errors & critical
    reportSysMsgsExtSev = 2
    database.dictionarySet( "sysMesgExtReportLevel", "2" )

# Cutoff time for messages to be reported
reportSysMsgsExtCutoff = database.dictionaryGet( "sysMesgExtReportCutoff" )

if reportSysMsgsExtCutoff == "":
    # 0 = no cutoff
    reportSysMsgsExtCutoff = 0
    database.dictionarySet( "sysMesgExtReportCutoff", "0" )

sysStateCollectionRate = database.dictionaryGet( "sysStateColRate" )
if sysStateCollectionRate == "":
    sysStateCollectionRate = 30
    database.dictionarySet( "sysStateColRate", "30" )

sysStateReportRate = database.dictionaryGet( "sysStateRepRate" )
if sysStateReportRate == "":
    sysStateReportRate = 60
    database.dictionarySet( "sysStateRepRate", "0" )

# Store Gateway Configuration
# and PC information
gateway = {}

#if systemType == "viper" or systemType == "vipergateway":
#    gateway["model_id"] = 35
#    gateway["manufacturer_id"] = 1
#else:
#    gateway["model_id"] = 2
#    gateway["manufacturer_id"] = 1

gateway["hostname"] = socket.gethostname()
gateway["software_version"] = softwareVersion

#Stores AWS Information

parser = SafeConfigParser()
parser.optionxform = str 
parser.read("/opt/Refresh/IoT/config")
if parser.has_section('aws') == False or parser.has_section('gateway') == False:
    Logger.critical("Gateway", "Invalid or missing config file")
    spin()

aws = {}
options = parser.options('aws')
for name, value in parser.items('aws'):
    aws[name] = value

for name, value in parser.items('gateway'):
    gateway[name] = value

Logger.critical("Gateway", "Config: gateway[model_id]: " + gateway["model_id"] )
Logger.critical("Gateway", "Config: gateway[manufacturer_id]: " + gateway["manufacturer_id"]  )

#Connection to MQTT
for key in ("certCa", "certPriv", "certCert"):
    if key in aws:
        aws[key] = os.path.join("/opt/Refresh/IoT", aws[key])
aws["cliendId"] =  aws["thing_name"] 

#check that we have all certs

#need to add code to logger add error to database
if (os.path.isfile(aws["certCa"]) and os.path.isfile(aws["certPriv"]) and os.path.isfile(aws["certCert"]) ):
    certs_good = True
else:
    certs_good = False
    Logger.critical("Gateway", "Certs not found")
    refreshWrapper.insertSystemMessage("IoT", "Critical", "Certificates not found")
    spin()

# Instantiate mqtt
mqttClient = None
mqttClient = CorcoMqttClient(aws["cliendId"])
mqttClient.config( aws["endpoint"], aws["port"], aws["certCa"], aws["certCert"], aws["certPriv"] )

# Scan 10 minutes after every hour
hour10ElapsedTime = time.strftime("%H") + ":10"
hour10Elapsed = False

# Scan on Minute
minuteElapsedTime = time.strftime("%M")
minuteElapsed = False

# Time last time network was restarted
networkTestTime = time.time()

#
#  Receives Signal from Downloader that download is done
#
def handleCompleteDownloads():
    global download_complete_queue
    global mqttClient
    global Logger

    try:
        success, url = download_complete_queue.get_nowait()
        if success:
            payload = {}
            payload['Request'] = {}
            payload['Request']['gw-update'] = {}
            payload['Request']['gw-update']['file'] = url
            mqttClient.updateShadow(payload)
            refreshWrapper.clearPendingDownload()
        else:
            Logger.info("Gateway", "Download Failed " + url)
    except Queue.Empty:
        pass # No new complete downloads.

#
# Publish json payload to configuration
#
def publishToConfig( payload ):
    Logger.info("Gateway", "Publishing Config")
    if payload is not None:
        try:
            mqttClient.updateConfig(payload)
        except Exception as inst:
            Logger.error("Gateway", "Publish Config Failed %s" % inst)
            refreshWrapper.insertSystemMessage("IoT", "Error", "Publish config failed")
    else:
        Logger.warning("Gateway", "Config Payload is empty")
  
#
# Publish json payload to Totals
#  
def publishToTotals( payload ):
    Logger.info("Gateway", "Publishing Total")
    if payload is not None:
        try:
            mqttClient.updateTotal(payload)
        except Exception as inst:
            Logger.error("Gateway", "Publish Total Failed %s" % inst)
            refreshWrapper.insertSystemMessage("IoT", "Error", "Publish total failed")
    else:
        Logger.warning("Gateway", "Total Payload is empty")

#
# Publish json payload to Marketing
#  
def publishToMarketing( payload ):
    Logger.info("Gateway", "Publishing Marketing")
    status = False
    if payload is not None:
        try:
            status = mqttClient.updateMarketing(payload)
        except Exception as inst:
            Logger.error("Gateway", "Publish Marketing Failed %s" % inst)
            refreshWrapper.insertSystemMessage("IoT", "Error", "Publish marketing failed")
    else:
        Logger.warning("Gateway", "Marketing Payload is empty")
    return status

#
# Publish json payload to  thing shadow
#
def publishToShadow( payload ):
    Logger.info("Gateway", "Publishing Shadow")
    if payload is not None:
        try:
            mqttClient.updateShadow(payload)
        except Exception as inst:
            Logger.error("Gateway", "Publish Shadow Failed %s" % inst)
            refreshWrapper.insertSystemMessage("IoT", "Error", "Publish shadow failed")
    else:
        Logger.warning("Gateway", "Shadow Payload is empty")

#
# Publish json payload to Error Logs
#
def publishToELog( payload ):
    Logger.info("Gateway", "Publishing Elog")
    if payload is not None:
        try:
            mqttClient.updateELog(payload)
        except Exception as inst:
            Logger.error("Gateway", "Publish Elog Failed %s" % inst)
            refreshWrapper.insertSystemMessage("IoT", "Error", "Publish error log failed")
    else:
        Logger.warning("Gateway", "Shadow Elog is empty")
    

#
# Publish json payload to Totals
#  
def publishToSysMsgsExt( payload ):
    Logger.info("Gateway", "Publishing System Messages Extended")
    if payload is not None:
        try:
            mqttClient.updateSysMsgsExt(payload)
            refreshWrapper.setSysMesgsExtSyncd()
        except Exception as inst:
            Logger.error("Gateway", "Publish SysMsgsExt Failed %s" % inst)
            refreshWrapper.insertSystemMessage("IoT", "Error", "Publish SysMsgsExt failed")
    else:
        Logger.warning("Gateway", "SysMsgsExt Payload is empty")

#
# Publish json payload to system_state
#  
def publishToSystemState( payload ):
    Logger.info("Gateway", "Publishing System State")
    if payload is not None:
        try:
            mqttClient.updateSystemState(payload)
        except Exception as inst:
            Logger.error("Gateway", "Publish System State Failed %s" % inst)
            refreshWrapper.insertSystemMessage("IoT", "Error", "Publish System State failed")
    else:
        Logger.warning("Gateway", "System State Payload is empty")

#
# Finds key in array
# Helper
#
def keyExists(key, data):
    return data.has_key(key)

#
# This callback is received from AWS.
# The payload is a delta change in the thing shadow.
# Will  be used to get content / playlist 
#
def on_mqtt_callback_delta(payload):
    global dBusWorker
    global Downloader

    Logger.data("Gateway", "Delta Called") 

    try:
        json_payload = json.loads(str(payload))
    except Exception as inst:
        Logger.data("Gateway", "Callback Respond Delta Got exception: (%s)" % inst)
        traceback.print_exc(file=sys.stdout)
            

    # print "on_mqtt_callback_delta:", json_payload

    # Get/set extended system messages filter (mesg level & cutoff time)
    if keyExists('Request', json_payload['state']):
        if keyExists("sysmsgsext", json_payload['state']['Request']):
            if keyExists("level", json_payload['state']['Request']['sysmsgsext']):
                reportSysMsgsExtSev = json_payload['state']['Request']['sysmsgsext']['level']
                database.dictionarySet( "sysMesgExtReportLevel", str(reportSysMsgsExtSev) )

                Logger.data("Gateway", "Set sys mesgs ext level: %s" % str(reportSysMsgsExtSev)) 

            if keyExists("cutofftime", json_payload['state']['Request']['sysmsgsext']):
                reportSysMsgsExtSev = json_payload['state']['Request']['sysmsgsext']['cutofftime']
                database.dictionarySet( "sysMesgExtReportCutoff", str(reportSysMsgsExtCutoff) )

                Logger.data("Gateway", "Set sys mesgs ext cutoff time: %s" % str(reportSysMsgsExtSev)) 

            payload = {}
            payload['Request'] = {}
            payload['Request']['sysmsgsext'] = {}
            payload['Request']['sysmsgsext']['level'] = reportSysMsgsExtSev
            payload['Request']['sysmsgsext']['cutofftime'] = reportSysMsgsExtCutoff
            
            mqttClient.updateShadow(payload)

    #
    # Intant Video Download
    #
    if keyExists('Request', json_payload['state']):
        if keyExists("instant-video", json_payload['state']['Request']):
            if keyExists("file", json_payload['state']['Request']['instant-video']):
                url = json_payload['state']['Request']['instant-video']['file']

                Logger.data("Gateway", "File to download %s" % url) 

                payload = {}
                payload['Request'] = {}
                payload['Request']['instant-video'] = {}
                payload['Request']['instant-video']['file'] = url
                
                #
                # Download the instant video and update the shadow on success
                # Since we don't have any errors messaging system yet
                # we may need to update the shadow even on failure.
                #
                #localFilePath = Downloader.getFile(url, "/opt/Refresh/Videos/CMS/tmp")
                #if localFilePath is not None:

                # Tell Video Player what to download

                mqttClient.updateShadow(payload)
                dBusWorker.setPlaylist(url[0:url.find('|')])


    if keyExists('Request', json_payload['state']):
        if keyExists("mrss-url", json_payload['state']['Request']):
            if keyExists("url", json_payload['state']['Request']['mrss-url']):
                url = json_payload['state']['Request']['mrss-url']['url']

                Logger.data("Gateway", "New mrss feed %s" % url) 

                payload = {}
                payload['Request'] = {}
                payload['Request']['mrss-url'] = {}
                payload['Request']['mrss-url']['url'] = url
                
                mqttClient.updateShadow(payload)
                dBusWorker.setMRSS(url[0:url.find('|')])

    #
    # Download Playlist XML File
    #
    if keyExists('Request', json_payload['state']):
        if keyExists("playlist", json_payload['state']['Request']):
            if keyExists("file", json_payload['state']['Request']['playlist']):
                url = json_payload['state']['Request']['playlist']['file']

                Logger.data("Gateway", "File to download %s" % url) 

                payload = {}
                payload['Request'] = {}
                payload['Request']['playlist'] = {}
                payload['Request']['playlist']['file'] = url
                
                #
                # Download the Playlist xml file and update the shadow on success
                #
                #playlistXmlFile = Downloader.getFile(url,  "/opt/Refresh/Videos/CMS/tmp")
                #if playlistXmlFile is not None:
                # Tell Video Player what to download
                mqttClient.updateShadow(payload)
                dBusWorker.setPlaylist(url[0:url.find('|')])
                    
    if keyExists('Request', json_payload['state']):
        if keyExists("gw-update", json_payload['state']['Request']):
            if keyExists("file", json_payload['state']['Request']['gw-update']):
                url = json_payload['state']['Request']['gw-update']['file']
                timestamp = json_payload["metadata"]["Request"]["gw-update"]["file"]["timestamp"]

                # Only try the same download so many times before giving up on
                # an unavailable download (in case IoT service is killed and
                # gets delta again frm shadow). There should be error codes sent
                # back to the DMS site that can be reviewed for debugging.
                refreshWrapper = RefreshWrapper() # Need refreshWrapper instance local to this thread.
                if refreshWrapper.canTryDownload(url, timestamp):
                    Logger.data("Gateway", "Update file to download %s" % url)
                    if not Downloader.beginDownload(url): # Try to download. Only one at a time.
                        Logger.warning("Gateway", "Downloader busy")
                else:
                    Logger.warning("Gateway", "Skipping download, already tried multiple times: %s" % url)
                

# This callback is triggered when AWS sends an MRSS URL update
# Pass along the URL to the videoplayer... that'll take
# care of saving it to the database.
# Regex matching to fix improperly formatted JSON content.
# Early versions of DMS sent bad content,
# DMS is fixed now, but leaving this code in here just as
# an extra precaution.
import re
# Extract only the http string up to the first space or quote or brace
# This is outside the function so that it can be pre-compiled.
# ^ = start of line
# .*{ = match everything until open brace
# [^:]*: = match everything that is NOT a colon until it hits a colon
# [^h]* = match everything up to h
# ( .... ) = group (used later)
# (h[^ \"'}]*) = match starting with h until it hits space or quote or
#                 single quote or closing brace 
#                 - and store this match in group 1
# [ \"'\}] = match the end of the http text
# .*$ = match everything to the end of the line
fixJsonPattern=re.compile("^.*{[^:]*:[^h]*(h[^ \"'}]*)[ \"'}].*$")

def on_mqtt_callback_mrss_url(payload):
    Logger.info( "Gateway", "MRSS update: " + payload )
    
    # payload = "{\"url\": \\\"https://something.else\\\" }"
    # payload = "{ \"url\" : https://dev.channelvalet.com/feeds/4315da7d-2618-4eb9-a71e-3718392bfb39.xml }"
    # Early versions of DMS sent invalid JSON format, fix it:
    payload = fixJsonPattern.sub( '{ "url" : "\\g<1>" }', str(payload) )
    # Remove any backslashes, this was also a problem
    # Backslashes are never legal in a URL
    payload = payload.replace( '\\', '' )
    Logger.info( "Gateway", "MRSS fixed: " + payload )

    try:
        JSON_dict = json.loads(str(payload))
        payload_url = JSON_dict.get(u'url')
        dBusWorker.setMRSS(payload_url)


    except Exception as inst:
        Logger.data("Gateway", "Callback on_mqtt_callback_mrss_url exception: (%s)" % inst)
        traceback.print_exc()
        pass

# Command published from MIM dashboard
def on_mqtt_callback_remote_cmd(payload):
    Logger.info( "Gateway", "Remote CMD: " + payload )


#
# This callback is received from AWS.
# The payload should be entire unit record from Enterprise Database.
# Function checks unit state to allow scanning and sending data 
# to the shadow.
#
def on_mqtt_callback_res_unitState(payload):
    global unitCommissioned
  
    # print "unit state callback"
    JSON_dict = json.loads(str(payload))
  
    unitCommissioned = False
  
    try:
        payload_status = JSON_dict.get(u'status')

        if payload_status == "error":
            raise Exception('Unit could not be found in Enterprise Platform database')
            spin()

        else:

            payload_unit = JSON_dict.get(u'unit')
    
            Logger.data("Gateway", "xxxxxxxxxDMS model_id: %s, manufacturer_id: %s" % ( payload_unit['model_id'], payload_unit['manufacturer_id'] ))
            # Logger.data("Gateway", "SYSTEM State model_id: %s, manufacturer_id: %s" % ( gateway['model_id'], gateway['manufacturer_id'] ))

            if (int(payload_unit['model_id']) == int(gateway['model_id'])) and (int(payload_unit['manufacturer_id']) == int(gateway['manufacturer_id'])):
                unitCommissioned = True
      
    except Exception as inst:
        Logger.data("Gateway", "Callback Respond Unit State Got exception: (%s)" % inst)
        traceback.print_exc(file=sys.stdout)
        pass

#
# This callback is called when mqtt receives disconnect callback
#
def on_mqtt_callback_disconnect():
    Logger.data("Gateway", "Lost mqtt connection!")
    updateConfigurationChanges  = True

def on_mqtt_callback_connect():    
    Logger.data("Gateway", "Connect callback!")  
    Logger.data("Gateway", str(autoConnect))

    if (autoConnect):
        if mqttClient.shadowInit() is True:
            if mqttClient.connected(True):
                mqttClient.setting(gateway)
                mqttClient.subscribeWithCallback(mqttClient._TOPIC_RES_UNIT_STATE, on_mqtt_callback_res_unitState)
                mqttClient.subscribeWithCallback(mqttClient._TOPIC_MRSS_URL, on_mqtt_callback_mrss_url)
                # mqttClient.subscribeWithCallback(mqttClient._TOPIC_REMOTE_CMD, on_mqtt_callback_remote_cmd)
                updateConfigurationChanges  = True
    Logger.data("Gateway", "Connect callback done!")

def testNetworkConnection():
   global networkTestTime

   interfaces = netifaces.interfaces()
   have_ip = False
   for x in interfaces:
      if x == "lo":
        continue
      addrs = netifaces.ifaddresses(x)
      try:
        addr = addrs[netifaces.AF_INET]
        ip = addr[0]['addr']
        have_ip = True
      except:
        pass

   if (time.time() - networkTestTime > (60 * 15)) and have_ip == False:
	Logger.info("Gateway", "Restart network daemon")
        os.system("/etc/init.d/wicd restart")
        networkTestTime = time.time()
	

# Return True when data is available in the socket
def checkUnixDirectSocket():
    global socketDisconnected
    global sock
    global server_address
    global sockPayloadPending
    global sockPayloadRemaining 
    global sockPayloadState

    result = False

    # Logger.info( "Gateway", "checkUnixDirectSocket" )

    if socketDisconnected:
        Logger.info( "Gateway" , 'UDS connecting to %s' % server_address )

        try:
            if not sock:
                # New socket needs to be created
                sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)

                # Prevent problems in case it tries to connect too soon after a disconnect.
                sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                sock.settimeout( 0.1 )

            sock.connect( server_address )
            Logger.info( "Gateway" , "UDS CONNECTED" )
            socketDisconnected = False
            # sock.close()
        except socket.error, msg:
            Logger.info( "Gateway" , "UDS CONNECT FAILED: " + str( msg ) )
            # print msg

    if not socketDisconnected:

        try:
            # Check if anything is ready to be processed
            # 1s timeout
            read_ready, write_ready, except_ready = select.select( [sock], [sock], [sock], 1.0 )
        except select.timeout:
            Logger.info( "Gateway" , "UDS select TIMEOUT" );
        except select.error, msg:
            closeSocket = True
            socketDisconnected = True
            Logger.info( "Gateway" , "UDS select error: " + str( msg ) )
        except IOError as e:
            closeSocket = True
            socketDisconnected = True
            Logger.info( "Gateway" , "UDS select IOError: " )
            # print "IOError", e
        except Exception as e:
            closeSocket = True
            socketDisconnected = True
            Logger.info( "Gateway" , "UDS select Exception: " )
            # print "Exception", e

        if len( except_ready ) > 0:
            Logger.info( "Gateway", "UDS EXCEPTED" )

        if len( write_ready ) == 0:
            Logger.info( "Gateway", "UDS WRITE NOT READY" )

        if len( read_ready ) > 0:
            result = True

    return result

def getUDSpayload():
    global socketDisconnected
    global sock
    global server_address
    global sockPayloadPending
    global sockPayloadRemaining 
    global sockPayloadState
    
    sockPayload = []

    Logger.info( "Gateway" , "UDS reading " + server_address + " ..." )
    closeSocket = False

    # The start of a message is a NULL
    # followed by the length of data
    # followed by another NULL
    # followed by data
    # terminated by NULL

    # The loop will terminate when all data is read
    # or a timeout exception (or error exception)
    payloadSize = 0
    while checkUnixDirectSocket():
        try:
            data = sock.recv( sockPayloadRemaining + 1 )
            dataCount = len( data )
        except socket.timeout:
            # No more data
            dataCount = len( data )
            # Logger.info( "Gateway", "UDS TIMEOUT: " + str( dataCount ) )

            if dataCount == 0:
                break;
        except select.error, msg:
            # Something failed
            Logger.info( "Gateway", "UDS error: " + str( msg ) )
            dataCount = 0

        if dataCount == 0:
            # If it returns with no data then the socket
            # is disconnected
            closeSocket = True
            sockPayloadRemaining = 0
            sockPayloadState = "WaitStart"
            break

        dataIndex = 0
        # Logger.info( "Gateway", "UDS dataCount: " + str(dataCount) )
        while( dataCount > 0 ):
            dataCount -= 1

            dataByte = data[dataIndex]
            dataIndex += 1

            # Logger.info( "Gateway", "UDS " + sockPayloadState + " - RX " + str( dataIndex - 1 ) + " : " + str( ord( dataByte ) ) + " (" + dataByte + ")" )

            if sockPayloadState == "WaitStart":
                # Wait for start of message
                if ord( dataByte ) == 0:
                    sockPayloadState = "GetLength"
                
                continue

            elif sockPayloadState == "GetLength":
                # Get length of message
                if ord( dataByte ) == 0:
                    # Logger.info( "Gateway", "UDS sockPayloadRemaining = " + str( sockPayloadRemaining ) )
                    sockPayloadState = "GetContent"
                else:
                    sockPayloadRemaining *= 10

                    try:
                        # Convert to number
                        # Exception if its not a number
                        sockPayloadRemaining += int(dataByte)
                    except ValueError:
                        Logger.info( "Gateway", "UDS Invalid length: " + dataByte )
                        sockPayloadRemaining = 0
                        sockPayloadState = "WaitStart"

                continue

            elif sockPayloadState == "GetContent":
                # Get the content of the message
                if ord( dataByte ) == 0:
                    # End of payload
                    # Logger.info( "Gateway", "UDS sockPayload[" + str(len(sockPayload)) + "] = " + sockPayloadPending)
                    try:
                        json_payload = json.loads(sockPayloadPending)
                        sockPayload.append( json_payload )
                    except Exception as inst:
                        Logger.data("Gateway", "UDS payload invalid: (%s)" % inst)
                        traceback.print_exc(file=sys.stdout)
                    sockPayloadPending = ""
                    sockPayloadState = "WaitStart"
                else:
                    # Index and count have already been incremented, so step them back
                    # to grab the current byte
                    if (dataCount + 1) >= sockPayloadRemaining:
                        # All of the data is in the buffer
                        sockPayloadPending += data[dataIndex - 1 : sockPayloadRemaining + (dataIndex - 1)]
                        dataCount -= (sockPayloadRemaining - 1)
                        sockPayloadRemaining = 0
                    else:
                        # Only partial data in the buffer
                        # end = (dataIndex - 1) + (dataCount + 1) == dataCount + dataIndex
                        sockPayloadPending += data[dataIndex - 1 : dataCount + dataIndex]
                        sockPayloadRemaining -= (dataCount + 1)
                        dataCount = 0

                    # Logger.info( "Gateway", "UDS sockPayloadRemaining: " + str(sockPayloadRemaining) + ", dataCount: " + str(dataCount) + ", sockPayloadPending: " + sockPayloadPending )
            else:
                sockPayloadState = "WaitStart"
                sockPayloadPending = 0

        # Limit the number of messages to prevent overloading the network or MIM
        # There's currently a problem in MIM that prevents it from taking more than
        # approx. 1K of data, so limit it to 1 message at a time for now. (2/4/19)
        if len( sockPayload ) > 1:
            break;

    if closeSocket:
        Logger.info( "Gateway", "UDS CLOSING" )
        socketDisconnected = True
        sock.close()
        # Discard the socket, it can't be re-used
        sock = {}

    final = {}

    # Choose wether to send a bulk or single payload
    if len( sockPayload ) > 1:
        final["bulk"] = sockPayload
    elif len( sockPayload ) > 0:
        final = sockPayload[0]

    return final

def IoTGateway():
#
# Main Loop
#
    global hour10Elapsed
    global hour10ElapsedTime
    global minuteElapsed
    global minuteElapsedTime
    global unitCommissioned
    global initialUpdate
    global updateConfigurationChanges
    global updateTotals
    global updateELog
    global updateSysMsgsExt
    global reportSysMsgsExtSev
    global updateErrors
    global updateMarketing 
    global updateConnection
    global dBusWorker
    global boot_time
    global last_connect_time

    global socketDisconnected
    global sock
    global server_address
    global sockPayloadPending
    global sockPayloadRemaining 
    global sockPayloadState

    updateSystemState = False

    Logger.info("Gateway", '*** STARTING ***')
    try:
        os.remove("/tmp/RefreshUpdate.lck")
    except:
        pass
    dBusWorker.start()
    
    socketDisconnected = True
    server_address = '/dev/shm/snapshot'

    sock = {}
    sockPayloadPending = ""
    sockPayloadRemaining = 0
    sockPayloadState = "WaitStart"

    updateSystemState = checkUnixDirectSocket()

    Logger.log("Beginning of the main loop")
    while True:

        hour10Elapsed = hour10ElapsedTime != time.strftime("%H") + ":10"
        minuteElapsed = minuteElapsedTime != time.strftime("%M")
        minute10Elapsed = (int(time.strftime("%M")) % 10) == 0 and (int(time.strftime("%S"))) == 0

        if time.strftime("%M:%S") == "30:00":
            Logger.info("Gateway", "Restarting on the half hour")
            os.system ("sudo corco IoT kill &")

        testNetworkConnection()

        handleCompleteDownloads()

        #
        # 1. Loop until connection is established with AWS
        #
        while True:
            if mqttClient.connected(False): 
               break

            autoConnect = False
            if (time.time() - last_connect_time) >= 60: 
                last_connect_time = time.time()
                try:
                    Logger.info("Gateway", "Try connect")
                    if mqttClient.connect(aws["thing_name"], on_mqtt_callback_delta, on_mqtt_callback_connect, on_mqtt_callback_disconnect):
                        if mqttClient.shadowInit() is True:
                            if mqttClient.connected(False):

                                mqttClient.setting(gateway)
                                mqttClient.subscribeWithCallback(mqttClient._TOPIC_RES_UNIT_STATE, on_mqtt_callback_res_unitState)
                                mqttClient.subscribeWithCallback(mqttClient._TOPIC_MRSS_URL, on_mqtt_callback_mrss_url)
                                # mqttClient.subscribeWithCallback(mqttClient._TOPIC_REMOTE_CMD, on_mqtt_callback_remote_cmd)
                                mqttClient.updateShadowConnection(mqttClient._PAHO_MQTT_CLIENT_CONNECTED, (systemType == "viper" or systemType == "vipergateway"))   
                                autoConnect = True
                            else:
                                continue
                        break

                except ValueError:
                    Logger.critical("Gateway", "Certificates Invalid")
                    refreshWrapper.insertSystemMessage("IoT", "Critical", "Certificates Invalid")
                    time.sleep(1800)
                    continue

                except Exception as inst:
                    Logger.critical("Gateway", "Something Went wrong: %s" % inst)
                    refreshWrapper.insertSystemMessage("IoT", "Critical", "Something Went wrong: %s" % inst)
                    spin()

            time.sleep(30)
            continue

        #
        # 2. Check unit state
        # Do not go any further without Enterprise Platform approval
        #
        
        if mqttClient.connected(False) == False:
            continue

        if unitCommissioned is not True:
            Logger.info( "Gateway", "not commissioned")
            mqttClient.requestUnitState()
            time.sleep(5)

            # Continue Main Loop
            continue

        #
        # 3. Scan Data
        #
        try:
            if mqttClient.connected(False) == False:
                continue
            
            if minute10Elapsed is True:
                mqttClient.updateShadowConnection(mqttClient._PAHO_MQTT_CLIENT_CONNECTED, (systemType == "viper" or systemType == "vipergateway"))   


            #
            # Statistics / Total data should be send every passed hour
            #
            if hour10Elapsed is True :
                # Logger.info( "Gateway", "minuteElapsed")
                updateTotals = True
                hour10ElapsedTime = time.strftime("%H") + ":10"

                if systemType == "viper" or systemType == "vipergateway":
                    # Check for any config changes
                    updateConfigurationChanges = True
#                updateMarketing = True
                
            if initialUpdate:
                updateConfigurationChanges = True
                updateMarketing = False
                updateTotals = False
                initialUpdate = False

            #
            # check errors once a minute
            #

            if minuteElapsed is True:
                updateSystemState = checkUnixDirectSocket()
                minuteElapsedTime = time.strftime("%M")

                #
                # check if eLog exists ( Error Log )
                #
                # Logger.info( "Gateway", "CHECK ELOG")
                updateELog  = refreshWrapper.checkELog()
                
                #
                # check if errors exists ( Current Errors )
                #
                updateErrors = refreshWrapper.checkErrors()

                # Check extended system messages...
                if reportSysMsgsExtSev == "":
                    # 4 = errors & critical
                    reportSysMsgsExtSev = 4
                
                # Logger.info( "Gateway" , "reportlevel = " + str( reportSysMsgsExtSev) + ", cutoff = " + str( reportSysMsgsExtCutoff ) )

                updateSysMsgsExt = (refreshWrapper.checkSysMesgsExt( reportSysMsgsExtSev, reportSysMsgsExtCutoff ) > 0)

            if mqttClient.connected(False) == False:
                continue
 
            if updateSystemState:
                Logger.info("Gateway", "Update system state")
                payload = getUDSpayload();
                # Logger.info( "Gateway", "PAYLOAD: " + str(payload) )

                if payload is not None:
                    publishToSystemState( payload )

                    # # Assemble data for realtime reporting
                    # realtimePayload = {}

                    # bulkPay = payload["bulk"]

                    # realtimePayload["b1_visc"] = payload["bulk"][0]["b1_visc"]
                    # realtimePayload["b2_visc"] = payload["bulk"][0]["b2_visc"]
                    # realtimePayload["b3_visc"] = payload["bulk"][0]["b3_visc"]
                    # realtimePayload["b4_visc"] = payload["bulk"][0]["b4_visc"]
                    
                    # realtimePayload["b1_temp"] = payload["bulk"][0]["b1_temp"]
                    # realtimePayload["b2_temp"] = payload["bulk"][0]["b2_temp"]
                    # realtimePayload["b3_temp"] = payload["bulk"][0]["b3_temp"]
                    # realtimePayload["b4_temp"] = payload["bulk"][0]["b4_temp"]

                    # realtimePayload["volt"] = payload["bulk"][0]["volt"]
                    # realtimePayload["freq"] = payload["bulk"][0]["freq"]

                    # publishToShadow( realtimePayload )


                # Check if there's any more data pending
                updateSystemState = checkUnixDirectSocket()


             # Update Confuguration
            # Send configuration initially and then only when changed
            #
            if mqttClient.connected(False) == False:
                continue
            if updateConfigurationChanges:
                Logger.info("Gateway", "Update Configuration")
                payload = refreshWrapper.getConfiguration()

                if len(payload) > 0:
                    mySocket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
                    mySocket.connect(('8.8.8.8', 0))
                    gateway["ip_address"] = mySocket.getsockname()[0]
                    mySocket.close()
                    payload["sw_gw"] = gateway["software_version"]
                    payload['rtc'] =  time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))
                    publishToConfig( payload )
                updateConfigurationChanges  = False


            #
            # Update Totals / Statistics
            # Send unsynced statistics for passed hours
            # Seperate publishToTotals call per one record
            #
            if mqttClient.connected(False) == False:
                continue
            if updateTotals:
                Logger.info("Gateway", "Update Totals")
                payload = refreshWrapper.getTotals()
                payload['rtc'] =  time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))

                with open('/proc/uptime', 'r') as f:
                   uptime_seconds = float(f.readline().split()[0])

                if systemType == "viper" or systemType == "vipergateway": 
                    payload['up_time'] =  str(int(uptime_seconds))
                else:
                    payload['power_on'] =  str(int(uptime_seconds))

                # print payload
                publishToTotals( payload )
                updateTotals = False

            #
            # Update Totals / Statistics
            # Send unsynced statistics for passed hours
            # Seperate publishToTotals call per one record
            #
            if mqttClient.connected(False) == False:
                continue
            if updateMarketing:
                Logger.info("Gateway", "Update Marketing")
                stats  = refreshWrapper.getAllStatistics()
                while True:
                  payload = refreshWrapper.getNextStats()
                  if (len(payload) == 0):
                     break
                  Logger.info("Gateway", "publish marketing")
                  payload['rtc'] =  time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))
                  if (publishToMarketing( payload )):
                      if (stats != None):
                          refreshWrapper.setIotSynced()
                updateMarketing = False

            #
            # Update Elog
            # An error that has been cleared
            # Don't send current or active errors here
            #
            if mqttClient.connected(False) == False:
                continue
            if updateELog:
                Logger.info("Gateway", "Update Error Log")
                refreshWrapper.getAllErrors()
                while True:
                  payload = refreshWrapper.getNextErrors()
                  if (len(payload) == 0):
                     break
                  payload['rtc'] =  time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))
                  publishToELog( payload )
                updateConfiguration  = False
                updateELog = False

            if mqttClient.connected(False) == False:
                continue
            if updateSysMsgsExt:
                Logger.info("Gateway", "Update System Messages Extended")
                payload = refreshWrapper.getSysMesgsExt( reportSysMsgsExtSev, reportSysMsgsExtCutoff )
                if len( payload ) > 0:
                    publishToSysMsgsExt( payload )
                # Payload size is limited to prevent hogging the link
                # Check to make sure there are no more messages pending
                updateSysMsgsExt = ( refreshWrapper.checkSysMesgsExt( reportSysMsgsExtSev, reportSysMsgsExtCutoff ) > 0 )

            #
            # Update Errors
            # Current status and error information
            #
            if mqttClient.connected(False) == False:
                continue
            if updateErrors:
                Logger.info("Gateway", "Update Errors")
                payload = refreshWrapper.getErrors()
                payload['rtc'] =  time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))
                publishToShadow( payload )
                updateErrors  = False

        except Exception as inst:
            Logger.critical("Gateway", "Main got exception: %s" % inst)
            traceback.print_exc(file=sys.stdout)
            #refreshWrapper.insertSystemMessage("IoT", "Critical", "IoT got exception: %s" % inst)
            Logger.info("Gateway", "IoT restarting on exception")
            os.system ("sudo corco IoT kill &")
 
        time.sleep(1)


if __name__ == "__main__":
   IoTGateway()

