#!/usr/bin/python

# ******************************************************************************
# *                                                                            *
# *   Copyright 2014 Cornelius, Inc. All rights reserved.                      *
# *                                                                            *
# *   This file is confidential and a trade secret of Cornelius, Inc.          *
# *   The receipt of or possession of this file does not convey any rights     *
# *   to reproduce or disclose its contents or to manufacture, use, or sell    *
# *   anything it may describe, in whole, or in part, without the specific     *
# *   written consent of Cornelius, Inc.                                       *
# *                                                                            *
# ******************************************************************************
# ------------------------------------------------------------------------------
#
#    DEVELOPED BY : Michal Augustyniak
#
# ------------------------------------------------------------------------------

import json
import time
import sys
import ssl
import paho.mqtt.client as paho_mqtt
from logger import Logger
import traceback

def waitUntilConnected(userdata, timeout = 5):
    timer = 0
    while userdata._PAHO_MQTT_CLIENT_CONNECTED == False:
        time.sleep(1)
        timer += 1
        if timer >= timeout:
            Logger.info("MQTT", "waitUntilConnected TIMEOUT")
            break

def waitUntilDisconnected(userdata, timeout = 10):
    timer = 0
    Logger.info("MQTT", "waitUntilDisconnected")
    while userdata._PAHO_MQTT_CLIENT_CONNECTED == True:
        time.sleep(1)
        timer += 1
        if timer >= timeout:
            Logger.info("MQTT", "waitUntilDisconnected TIMEOUT")
            break

def waitUntilShadowInit(userdata, timeout = 10):
    timer = 0
    while userdata._SHADOW_INIT_SUCCESS == False:
        time.sleep(1)
        timer += 1
        if timer >= timeout:
            Logger.info("MQTT", "waitUntilShadowSuccess TIMEOUT")
            break

def waitUntilUnitStateRespond(userdata, timeout = 10):
    timer = 0
    while userdata._UNIT_STATE_RESPOND == False:
        time.sleep(1)
        timer += 1
        if timer >= timeout:
            Logger.info("MQTT", "waitUntilUnitStateRespond TIMEOUT")
            break    

class CorcoMqttClient:

    _HOST = None
    _PORT = 8883
    _KEEP_ALIVE = 45 # seconds to disconnect

    _THINGHNAME = None
    _CLIENT_ID = None

    _SHADOW_TOPIC_UPDATE_DELTA = None
    _SHADOW_TOPIC_UPDATE = None
    _TOPIC_REQ_UNIT_STATE = "unit/state/request"
    _TOPIC_RES_UNIT_STATE = None
    _TOPIC_LWT = None
    _TOPIC_TOTAL = None
    _TOPIC_CONFIG = None
    _TOPIC_MARKETING = None
    _TOPIC_MRSS_URL = None
    _TOPIC_SYS_MSGS_EXT = None
    _TOPIC_SYSTEM_STATE = None
    # _TOPIC_REMOTE_CMD = None


    _PAYLOAD_LWT_OFFLINE = "{\"state\":{\"reported\":{\"UnitState\": {\"gateway_connected\": false, \"device_connected\": false}}}}"

    #_PAYLOAD_LWT_ONLINE = "{\"state\":{\"reported\":{\"connected\":\"true\"}}}"
    #_PAYLOAD_LWT_OFFLINE = "{\"state\":{\"reported\":{\"connected\":\"false\",\"unit_connected\": \"false\"}}}"

    _SHADOW_INIT_MID = None
    _SHADOW_INIT_SUCCESS = False
    _UNIT_STATE_RESPOND = False
    _MRSS_UPDATE = False
    _REMOTE_CMD = False

    _CERT_CA = None
    _CERT_PRIV = None
    _CERT_CERT = None

    _PAHO_MQTT_CLIENT = None
    _PAHO_MQTT_CLIENT_CONNECTED = False
    _PAHO_MQTT_CONN_TIMEOUT = 5 # seconds

    _CALLBACK_DELTA = None
    _CALLBACK_RESP_UNIT_STATE = None
    _CALLBACK_MRSS_UPDATE = None
    _CALLBACK_DISCONNECT = None
    _CALLBACK_CONNECT = None
    _CALLBACK_REMOTE_CMD = None


    _SETTING = None

    pingLogCount = 10
    pingReqCount = 0
    pingRespCount = 0
   
    @staticmethod
    def on_log(client, userdata, level, buf):
        writeLog = True

        if "Sending PINGREQ" in buf:
            # Don't bother logging this, its just noise
            userdata.pingReqCount += 1
            return

        if "Received PINGRESP" in buf:
            writeLog = False
            # Reduce this noise to one in 10
            userdata.pingLogCount += 1
            userdata.pingRespCount += 1

            if userdata.pingLogCount >= 10:
                Logger.info( "MQTTLOG", "PINGS req: " + str( userdata.pingReqCount ) + ", resp: " + str( userdata.pingRespCount ))
                userdata.pingLogCount = 0
                userdata.pingReqCount = 0
                userdata.pingRespCount = 0
                writeLog = True

        if writeLog:
            Logger.info("MQTTLOG", str(level) + ": " + buf)
 
    @staticmethod
    def on_connect(client, userdata, flags, rc):
        Logger.info("MQTT", "on_connect %s" % rc)
        userdata._PAHO_MQTT_CLIENT_CONNECTED = True if rc == 0 else False
        userdata._CALLBACK_CONNECT()


    @staticmethod
    def on_disconnect(client, userdata, rc):
        Logger.info("MQTT", "on_disconnect %s" % rc)
        try:
            userdata._PAHO_MQTT_CLIENT.loop_stop()
        except:
            pass
        userdata._PAHO_MQTT_CLIENT_CONNECTED = False 
        userdata._CALLBACK_DISCONNECT()
        

    @staticmethod
    def on_subscribe(client, userdata, mid, granted_qos):
        Logger.info("MQTT", "on_subscribe")
        Logger.data("MQTT", "mid: %s" % mid)
        Logger.data("MQTT", "granted_qos: %s" % granted_qos)

        if (mid == userdata._SHADOW_INIT_MID):
            userdata._SHADOW_INIT_SUCCESS = True


    @staticmethod
    def on_unsubscribe(client, userdata, mid):
        Logger.info("MQTT", "on_unsubscribe")

    @staticmethod
    def on_message(client, userdata, message):
        Logger.info("MQTT", "on_message")
        Logger.data("MQTT", "topic  : %s" % message.topic)
        Logger.data("MQTT", "payload: %s" % message.payload)
        Logger.data("MQTT", "qos    : %s" % message.qos)

        # Return respond to the delta colback
        if userdata._SHADOW_TOPIC_UPDATE_DELTA == message.topic:
            try:
                userdata._CALLBACK_DELTA(message.payload)
            except Exception as inst:
                Logger.error("MQTT", "Callback Delta Failed, topic: '%s', (%s)" % (message.topic, inst))
            traceback.print_exc(file=sys.stdout)
        
        # Return respond to the unit state request colback
        elif userdata._TOPIC_RES_UNIT_STATE == message.topic:
            try:
                userdata._CALLBACK_RESP_UNIT_STATE(message.payload)
                userdata._UNIT_STATE_RESPOND = True
            except Exception as inst:
                Logger.error("MQTT", "Callback Unit State Failed, topic: '%s', (%s)" % (message.topic, inst))
        elif userdata._TOPIC_MRSS_URL == message.topic:
            try:
                userdata._CALLBACK_MRSS_UPDATE(message.payload)
                userdata._MRSS_UPDATE = True
            except Exception as inst:
                Logger.error("MQTT", "Callback MRSS update Failed, topic: '%s', (%s)" % (message.topic, inst))
                traceback.print_exc()
        # elif userdata._TOPIC_REMOTE_CMD == message.topic:
        #     try:
        #         userdata._CALLBACK_REMOTE_CMD(message.payload)
        #         userdata._REMOTE_CMD = True
        #     except Exception as inst:
        #         Logger.error("MQTT", "Callback remote command Failed, topic: '%s', (%s)" % (message.topic, inst))
        #         traceback.print_exc()



    def __init__(self, clientID):
        try:
            self._CLIENT_ID = clientID
            self._PAHO_MQTT_CLIENT = paho_mqtt.Client( client_id = clientID, userdata = self)
            self._PAHO_MQTT_CLIENT.on_connect = self.on_connect
            self._PAHO_MQTT_CLIENT.on_disconnect = self.on_disconnect
            self._PAHO_MQTT_CLIENT.on_subscribe = self.on_subscribe
            self._PAHO_MQTT_CLIENT.on_unsubscribe = self.on_unsubscribe
            self._PAHO_MQTT_CLIENT.on_message = self.on_message
            self._PAHO_MQTT_CLIENT.on_log = self.on_log

        except Exception as inst:
              Logger.error("MQTT", "Couldn't instantiate paho mqtt client (%s)" % inst)


    def mqtt(self):
        return self._PAHO_MQTT_CLIENT


    def config(self, host, port, file_ca, file_cert, file_priv):
        self._HOST = host
        self._PORT = port
        self._CERT_CA = file_ca
        self._CERT_CERT = file_cert
        self._CERT_PRIV = file_priv
        try:
            self._PAHO_MQTT_CLIENT.tls_set( self._CERT_CA, self._CERT_CERT, self._CERT_PRIV, ssl.CERT_REQUIRED, ssl.PROTOCOL_SSLv23)
        except:
            Logger.error("MQTT", "Couldn't configure")


    def setting(self, setting):
        Logger.info("MQTT", "Configuring Settings")
        self.SETTING = setting
        self._TOPIC_TOTAL = "data/historical/%s/%s/totals" % (self.SETTING['manufacturer_id'], self.SETTING['model_id'])
        self._TOPIC_ELOG = "data/historical/%s/%s/errors" % (self.SETTING['manufacturer_id'], self.SETTING['model_id'])
        self._TOPIC_CONFIG = "data/configuration/%s/%s" % (self.SETTING['manufacturer_id'], self.SETTING['model_id'])
        self._TOPIC_MARKETING = "data/historical/%s/%s/marketing" % (self.SETTING['manufacturer_id'], self.SETTING['model_id'])
        self._TOPIC_SYS_MSGS_EXT = "data/historical/%s/%s/messages" % (self.SETTING['manufacturer_id'], self.SETTING['model_id'])
        self._TOPIC_SYSTEM_STATE = "data/historical/%s/%s/system_state" % (self.SETTING['manufacturer_id'], self.SETTING['model_id'])
      

    def connect(self, thingName, callback_delta, callback_connect, callback_disconnect):
        Logger.info("MQTT", "Connecting")
        self._CALLBACK_DELTA = callback_delta
        self._CALLBACK_DISCONNECT = callback_disconnect
        self._CALLBACK_CONNECT = callback_connect
        self._THINGHNAME = thingName
        self._SHADOW_TOPIC_UPDATE_DELTA = "$aws/things/%s/shadow/update/delta" % (self._THINGHNAME)
        self._TOPIC_LWT = "lwt/updateShadow/%s" % (self._THINGHNAME)
        self._TOPIC_RES_UNIT_STATE = "%s/unit/state/respond" % (self._THINGHNAME)
        self._TOPIC_MRSS_URL = "cms/%s/updateURL" % (self._THINGHNAME)
        # self._TOPIC_REMOTE_CMD = "data/%s/remote" % (self._THINGHNAME)

        Logger.info("MQTT", "%s %s" % (self._TOPIC_LWT, self._PAYLOAD_LWT_OFFLINE))
        try:
            self._PAHO_MQTT_CLIENT.loop_stop()
            self._PAHO_MQTT_CLIENT.will_set(self._TOPIC_LWT, self._PAYLOAD_LWT_OFFLINE, 0, False)
            self._PAHO_MQTT_CLIENT.connect(self._HOST, port=self._PORT, keepalive=self._KEEP_ALIVE)
            self._PAHO_MQTT_CLIENT.loop_start()
        except ssl.SSLError as e:
            Logger.critical("MQTT", "Certificate Invalid")
            raise ValueError("Certificate Invalid")
            
        except:
            Logger.error("MQTT", "Couldn't connect paho mqtt client")


        Logger.info("MQTT", "Wait for connected")
        waitUntilConnected(self, 10)

        return self.connected(True)


    def disconnect(self):
        Logger.info("MQTT", "disconnect")
        try:
            self._PAHO_MQTT_CLIENT.disconnect()
        except:
            return False

        waitUntilDisconnected(self, 10)

        return self.connected(True) == False


    def connected(self, verbose):
        if self._PAHO_MQTT_CLIENT_CONNECTED:
            if (verbose):
                Logger.info("MQTT", "Connected --")
        else:
            if (verbose):
                Logger.warning("MQTT", "Not Connected")
	  
        return self._PAHO_MQTT_CLIENT_CONNECTED


    def subscribe(self, topic, qos = 1):
        Logger.info("MQTT", "Subscribing: %s" % topic)
        key = None
        try:
            key = self._PAHO_MQTT_CLIENT.subscribe(topic, qos)
        except:
            Logger.error("MQTT", "Couldn't subscript to %s" % topic)

        return key


    def unsubscribe(self, topic):
        Logger.info("MQTT", "unsubscribe: %s" % topic)
        try:
            self._PAHO_MQTT_CLIENT.subscribe(topic, 1)
        except:
            Logger.error("MQTT", "Couldn't unsubscript %s" % topic)


    def loopForever(self):
        self._PAHO_MQTT_CLIENT.loop_forever()


    def shadowInit(self):
        try:
            (result, self._SHADOW_INIT_MID) = self.subscribe(self._SHADOW_TOPIC_UPDATE_DELTA)
            Logger.info("MQTT", "shadowInit: result = %s, mid = %s" % (str(result), str(self._SHADOW_INIT_MID)))
        except:
            Logger.error("MQTT", "Couldn't initialize shadow: %s" % (self._SHADOW_TOPIC_UPDATE_DELTA))
            return False

        waitUntilShadowInit(self, 20)

        return self._SHADOW_INIT_SUCCESS
      

    def updateConfig(self,payload):
        Logger.info("MQTT", "Publishing Config to %s" % self._TOPIC_CONFIG)
      
        try:
            payload["thing_name"] = self._THINGHNAME
            json_payload = json.dumps(payload)
        except Exception as inst:
            Logger.error("MQTT", "Json Error (%s)" % inst)
	
        Logger.data("MQTT", "sent payload: %s" % json_payload)
      
        try:
            self._PAHO_MQTT_CLIENT.publish(self._TOPIC_CONFIG, json_payload, 1, False)
        except:
            Logger.error("MQTT", "Couldn't publish to %s" % self._TOPIC_CONFIG)
            return false
        return True        

    def updateTotal(self,payload):
        Logger.info("MQTT", "Publishing Totals to %s" % self._TOPIC_TOTAL)
      
        try:
            payload["thing_name"] = self._THINGHNAME
            json_payload = json.dumps(payload)
        except Exception as inst:
            Logger.error("MQTT", "Json Error (%s)" % inst)
	
        Logger.data("MQTT", "sent payload: %s" % json_payload)
      
        try:
            self._PAHO_MQTT_CLIENT.publish(self._TOPIC_TOTAL, json_payload, 1, False)
        except:
            Logger.error("MQTT", "Couldn't publish to %s" % self._TOPIC_TOTAL)
      

    def updateMarketing(self,payload):
        Logger.info("MQTT", "Publishing Marketing to %s" % self._TOPIC_MARKETING)
      
        try:
            payload["thing_name"] = self._THINGHNAME
            json_payload = json.dumps(payload)
        except Exception as inst:
            Logger.error("MQTT", "Json Error (%s)" % inst)
    
        Logger.data("MQTT", "sent payload: %s" % json_payload)
      
        try:
            self._PAHO_MQTT_CLIENT.publish(self._TOPIC_MARKETING, json_payload, 1, False)
        except:
            Logger.error("MQTT", "Couldn't publish to %s" % self._TOPIC_MARKETING) 
            return false
        return True

    def updateELog(self,payload):
        Logger.info("MQTT", "Publishing Elog to %s" % self._TOPIC_ELOG)
      
        try:
            payload["thing_name"] = self._THINGHNAME
            json_payload = json.dumps(payload)
        except Exception as inst:
            Logger.error("MQTT", "Json Error (%s)" % inst)
	
        Logger.data("MQTT", "sent payload: %s" % json_payload)
    
        try:
            self._PAHO_MQTT_CLIENT.publish(self._TOPIC_ELOG, json_payload, 1, False)
        except:
            Logger.error("MQTT", "Couldn't publish to %s" % self._TOPIC_ELOG)
            return false
        return True

    def updateShadow(self, payload, desired = False):
        Logger.info("MQTT", "Publishing shadow")
        self._SHADOW_TOPIC_UPDATE = "$aws/things/" + self._THINGHNAME + "/shadow/update"

        try:
            json_payload = {}
            if desired:
                json_payload = json.loads("{\"state\":{\"desired\":{}}}")
                json_payload["state"]["desired"] = payload
            else:
                json_payload = json.loads("{\"state\":{\"reported\":{}}}")
                json_payload["state"]["reported"] = payload
            json_payload["clientToken"] = self._CLIENT_ID
            json_payload = json.dumps(json_payload)
        except:
            Logger.error("MQTT", "Json Error")

        Logger.data("MQTT", "sent payload: %s" % json_payload)

        try:
            self._PAHO_MQTT_CLIENT.publish(self._SHADOW_TOPIC_UPDATE, json_payload, 1, False)
        except:
            Logger.error("MQTT", "Couldn't publish to %s" % self._SHADOW_TOPIC_UPDATE)
            return false
        return True

    def updateShadowConnection(self, gatewayConnected, deviceConnected):
        Logger.info("MQTT", "Publishing shadow [System Connected: gateway: %s, device: %s]" % (gatewayConnected, deviceConnected) )
        
        payload = {}
        payload["UnitState"] = {}
        payload["UnitState"]["gateway_connected"] = gatewayConnected
        payload["UnitState"]["device_connected"] = deviceConnected
	
        self.updateShadow(payload)

    def updateSysMsgsExt(self,payload):
        Logger.info("MQTT", "Publishing SysMsgsExt to %s" % self._TOPIC_SYS_MSGS_EXT)
      
        try:
            payload["thing_name"] = self._THINGHNAME
            json_payload = json.dumps(payload)
        except Exception as inst:
            Logger.error("MQTT", "Json Error (%s)" % inst)
            return False
    
        Logger.data("MQTT", "sending payload: %s" % json_payload)
      
        try:
            self._PAHO_MQTT_CLIENT.publish(self._TOPIC_SYS_MSGS_EXT, json_payload, 1, False)
        except:
            Logger.error("MQTT", "Couldn't publish to %s" % self._TOPIC_SYS_MSGS_EXT)
            return False

        return True

    def updateSystemState(self,payload):
        Logger.info("MQTT", "Publishing System State to %s" % self._TOPIC_SYSTEM_STATE)
      
        try:
            payload["thing_name"] = self._THINGHNAME
            json_payload = json.dumps(payload)
        except Exception as inst:
            Logger.error("MQTT", "Json Error (%s)" % inst)
            return False
    
        Logger.data("MQTT", "sending payload: %s" % json_payload)
      
        try:
            self._PAHO_MQTT_CLIENT.publish(self._TOPIC_SYSTEM_STATE, json_payload, 1, False)
        except:
            Logger.error("MQTT", "Couldn't publish to %s" % self._TOPIC_SYSTEM_STATE)
            return False

        return True

    # def updateShadowGatewayConnection(self, connected):
    #     Logger.info("MQTT", "Publishing shadow [MQTT Connected: %s]" % connected )
        
    #     payload = {}
    #     payload["UnitState"] = {}
        
    #     if connected:
    #         payload["UnitState"]["gateway_connected"] = True
    #     else:
    #         # This should not be a case due LWT :)
    #         payload["UnitState"]["gateway_connected"] = False
    #         payload["UnitState"]["device_connected"] = False
    
    #     self.updateShadow(payload)

    def subscribeWithCallback(self, topicName, callback):
        if topicName == self._TOPIC_RES_UNIT_STATE:
            self._CALLBACK_RESP_UNIT_STATE = callback
        elif topicName == self._TOPIC_MRSS_URL:
            self._CALLBACK_MRSS_UPDATE = callback
        # elif topicName == self._TOPIC_REMOTE_CMD:
        #     self._CALLBACK_REMOTE_CMD = callback

        self.subscribe(topicName)

    def requestUnitState(self):
        Logger.info("MQTT", "Requesting unit state from Enterptise Platform")
        json_payload = json.loads("{}")
        json_payload["thing_name"] = self._THINGHNAME
        json_payload = json.dumps(json_payload)

        Logger.data("MQTT", "sent payload: %s" % json_payload)

        try:
            self._PAHO_MQTT_CLIENT.publish(self._TOPIC_REQ_UNIT_STATE, json_payload, 1, False)
            Logger.data("MQTT", "Called Topic %s" % self._TOPIC_REQ_UNIT_STATE)
        except:
            Logger.error("MQTT", "Couldn't publish to %s" % self._TOPIC_REQ_UNIT_STATE)

        self._UNIT_STATE_RESPOND = False
        self._MRSS_UPDATE = False
        waitUntilUnitStateRespond(self, 10)
