
#!/usr/bin/python

# ******************************************************************************
# *                                                                            
# *   Copyright 2014 Cornelius, Inc. All rights reserved.                      
# *                                                                            
# *   This file is confidential and a trade secret of Cornelius, Inc.         
# *   The receipt of or possession of this file does not convey any rights     
# *   to reproduce or disclose its contents or to manufacture, use, or sell    
# *   anything it may describe, in whole, or in part, without the specific     
# *   written consent of Cornelius, Inc.                                       
# *                                                                            
# ******************************************************************************
# ------------------------------------------------------------------------------
#
#    DEVELOPED BY : Michal Augustyniak
#                   Modified. M. Singer
#
# ------------------------------------------------------------------------------


from logger import Logger
import time
import os
import sys
import urllib2
import os.path
from urllib2 import urlparse
import traceback
import threading
import Queue
import signal
import subprocess
import DatabaseHelper

database = DatabaseHelper.DatabaseConnection("/opt/Refresh/Database/refresh_database_enc.db")
systemType = database.dictionaryGet( "systemType" )
Logger.info( "Gateway", "System Type: " + systemType )

if systemType == "viper" or systemType == "vipergateway":
    from viperWrapper import RefreshWrapper
else:
    from refreshWrapper import RefreshWrapper

tmp_updates_directory = "/opt/Refresh/Updates/"

# Remove file at path. If the file does not exist, nothing happens.
def removeFile(path):
    try:
        os.remove(path)
    except OSError, e:
        pass # File does not exist -- safe.

def fileNameFromURL(download_url):
    chunks = urlparse.urlsplit(download_url)
    filePath = chunks.path
    return filePath.split("/")[-1]

class Downloader(threading.Thread):
    def __init__(self, download_complete_queue):

        threading.Thread.__init__(self)
        self.thread = threading.Thread
        self.queue = Queue.Queue()
        self.download_complete_queue = download_complete_queue
        self.mutex = threading.Lock()
        self.bad_files = []
        self.url  = ""
        self.downloading = False
        self.download_process = None
        self.SHOULD_RUN = True

        # Setup signal handler to stop download process when this process is
        # killed.
        signal.signal(signal.SIGTERM, self.stopDownloadAndExit)

    def isAlive(self):
        print "Downloader alive"
        return "yes"

    def beginDownload(self, url):
        self.mutex.acquire()
        try:
            if self.downloading:
                return False

            self.downloading = True
            self.resetFiles()
            f = file("/tmp/RefreshUpdate.lck","w")
            self.queue.put(url)
            self.queue.put(None)
        finally:
            self.mutex.release()

        return True

    def terminate(self):
        self.SHOULD_RUN = False

    def resetFiles(self):
        self.bad_files = []

    def stopDownloadAndExit(self, signum, frame):
        # Stop any currently running download process. Not doing this causes
        # wget to continue to run even after this process is killed.
        self.mutex.acquire()
        if self.download_process:
            self.download_process.terminate()
        self.mutex.release()

        # Stop the entire python process since this is triggered by a SIGTERM
        # signal. Otherwise, we'll block default processing and keep running.
        os._exit(0)

    def downloadFile(self, download_url):
        # Start wget to download the file.
        save_directory = tmp_updates_directory # Directory to save into. Will use filename from URL.
        retries = "3" # How many times to try downloading file before giving up.
        timeout_seconds = "30" # How long to wait until connecton is considered dropped.
        wait_seconds = "30" # How long to wait between retries.

        self.mutex.acquire()
        self.download_process = subprocess.Popen(["wget", "-c", "-P", save_directory, "-t", retries, "-T", timeout_seconds, "-w", wait_seconds, "--progress=dot:giga", download_url])
        self.mutex.release()

        # Spin until the file downloads or it fails. Spinning here is required so
        # the other Python thread continues to process MQTT messages.
        while True:
            time.sleep(0.1)
            self.mutex.acquire()
            result = self.download_process.poll()
            if result is not None:
                self.download_process = None
                self.mutex.release()
                return result
            self.mutex.release()

    def onDownloadReady(self, refreshWrapper, download_url, save_as):
        # Figure out what type of download it is and handle it appropriately.
        # It can be a backup configuration file that should be restored or a
        # system update. Configuration files are installed immediately. System
        # updates wait until manual intervention or daily restart.
        file_name = os.path.basename(save_as)
        if ("-config" in file_name) and file_name.endswith(".tar.gz"):
            # try to install configuration file.
            os.system("sudo /opt/Refresh/Versions/Refresh-current/ServiceUI/Scripts/configurationRestore.sh \"%s\"" % save_as)
            Logger.info("Downloader", "Installing config file %s" % (download_url,))
            refreshWrapper.insertSystemMessage("IoT", "Message", "Installed config file")
            # remove configuration file since it's been installed or is corrupted.
            removeFile(save_as)
            # wait for shadow to update or we'll just keep redownloading and
            # restarting.
            self.download_complete_queue.put((True, self.url))
            while not self.download_complete_queue.empty():
                time.sleep(0.1)
            # Restart Corco to use new configuration file immediately.
            os.system("nohup sudo corco restart")
            return True
        else:
            # validate downloaded file as an update
            download_valid = os.system("sudo corco validate update \"%s\"" % save_as) == 0
            if download_valid:
                # valid software was downloaded
                return True
            else:
                # file was corrupted or otherwise not valid software
                Logger.warning("Downloader", "error validating %s" % (download_url,))
                refreshWrapper.insertSystemMessage("IoT", "Error", "Update self-check failed")
                removeFile(save_as)
        return False

    def run(self):
        refreshWrapper = RefreshWrapper() # Must be initialized in the thread it will be used!!

        while self.SHOULD_RUN:
            # hack so we can terminate thread on corco stop
            if (self.queue.empty()) :
                time.sleep(1)
                continue
            download_url = self.queue.get(True)
            # sentinal
            if not download_url:
                Logger.info("Downloader", "Download queue complete")
                self.mutex.acquire()
                self.downloading = False
                download_success = self.bad_files == []
                self.mutex.release()
                self.download_complete_queue.put((download_success, self.url))
                try:
                    os.remove("/tmp/RefreshUpdate.lck")
                except:
                    pass
                continue

            file_name = fileNameFromURL(download_url)
            save_as = os.path.join(tmp_updates_directory, file_name)
            self.url = download_url

            # remove old or incomplete file if it's still around
            removeFile(save_as)

            # download file
            try:
                Logger.info("Downloader", 'downloading %s' % (save_as))
                result = self.downloadFile(download_url)
                if result == 0:
                    Logger.info("Downloader",  'download complete %s' % (save_as))
                    if self.onDownloadReady(refreshWrapper, download_url, save_as):
                        continue
                else:
                    Logger.warning("Downloader", "error downloading %s" % (download_url,))
                    refreshWrapper.insertSystemMessage("IoT", "Error", "Update download failed")
            except Exception, e:
                Logger.warning("Downloader", "error downloading %s: %s" % (download_url, e))
                refreshWrapper.insertSystemMessage("IoT", "Error", "Update download internal error")
                traceback.print_exc(file=sys.stdout)

            self.mutex.acquire()
            if save_as not in self.bad_files:
                self.bad_files.append(save_as)
            self.mutex.release()



#class Downloader():
#  
#    @staticmethod
#    def getFile(url, directory):
#        Logger.info("Downloader", "Downloading file")
#
#        chunks = urlparse.urlsplit(url)
#        filePath = chunks[2]
#        fileName = filePath.split('/')[-1]
#
#        localFIlePath = directory + "/" + fileName
#        print localFIlePath
#        try:
#            r = urllib2.urlopen(url)
#            if r.getcode() == 200:
#                content = r.read()
#                f = open( localFIlePath, 'w' )
#                f.write( content )
#                f.close()
#            return localFIlePath
#        except Exception as inst:
#            Logger.info("Downloader", "Downloading failed %s" % inst)
#            traceback.print_exc(file=sys.stdout)
#        return None
    
