#!/usr/bin/python

# ******************************************************************************
# *                                                                            
# *   Copyright 2014 Cornelius, Inc. All rights reserved.                      
# *                                                                            
# *   This file is confidential and a trade secret of Cornelius, Inc.          
# *   The receipt of or possession of this file does not convey any rights     
# *   to reproduce or disclose its contents or to manufacture, use, or sell   
# *   anything it may describe, in whole, or in part, without the specific     
# *   written consent of Cornelius, Inc.                                       
# *                                                                            
# ******************************************************************************
# ------------------------------------------------------------------------------
#
#    DEVELOPED BY : Michal Augustyniak
#
# ------------------------------------------------------------------------------

import logging
from datetime import date
import DatabaseHelper
import json
import sys
import time
import csv
import os
import subprocess
from logger import Logger


def uTime2dbTime(utime):
    return time.strftime("%d-%m-%y %H:%M:%S", time.localtime(utime))


def dbTime2uTime(atime):
    dt = atime.split(' ')
    d = dt[0].split('-')
    t = dt[1].split(':')
    return time.mktime((int(d[2]) + 2000, int(d[1]), int(d[0]), int(t[0]), int(t[1]), int(t[2]), 0, 1, -1))
 
def getMFVVersion(boardNumber):
    command = "dbus-send --print-reply --dest=org.refresh.hardwareManager / org.refresh.hardwareManager.uiRequestVersion int32:%i" % boardNumber
    process = subprocess.Popen(command.split(), stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    output,error = process.communicate()
    split_output = output.split(" ")
    if error or len(split_output) < 10 or split_output[8] != "string":
        return "0.0"
    return split_output[9].strip().replace("\"","")

def getNetworkMACs():
    # Return all all network devices and mac addresses as a comma delimiter
    # list.
    network_devices = os.listdir("/sys/class/net/")
    mac_addresses = []
    for network_device in network_devices:
        if network_device == "lo":
            continue
        mac_address = file("/sys/class/net/%s/address" % network_device, "rb").read().strip()
        mac_addresses.append(mac_address)
    return ",".join([",".join(x) for x in zip(network_devices,mac_addresses)])

print getNetworkMACs()

class RefreshWrapper():

    def __init__(self):
        self.stats = []
        self.index = 1

        # Connect to database
        self.refreshDatabase = DatabaseHelper.DatabaseConnection("/opt/Refresh/Database/refresh_database_enc.db")

        # create dictionary to convert messages to a number for AWS
        self.errorTable = {}
        with open("Refresh_Errors.csv", "rb") as f:
            reader = csv.reader(f, delimiter=',')
            for row in reader:
                if row[1] != '':
                    self.errorTable.update({row[3]: row[1]})       


    def getTotals(self):
        payload = {}
        self.totals = self.refreshDatabase.getValveTotals() 
        if len(self.totals) == 0:
            return payload
        for t in self.totals:
            name = t[0].lower() + '_total'
            payload[name] = int((float(t[1]) / 1000.) + .5)
    
        return payload
    #
    # SQL Query Statistics
    # 
    # Refresh statistics are stored on hourly basis
    # This function should query data for a passed hour.
    #
    # Once send correctly, send records should be deleted 
    # from Refresh Database or marked as synced
    # 
    # The idea is to eventually replicate this database on Enterprise
    # If different please discuss Michal, VInce and Slaw
    # 
    # returns json dictionary
    #
    # Michal's hardcoded data left in place for now with changed names

    def getAllStatistics(self):
        self.stats = self.refreshDatabase.getPourStats()
        self.index = 0
        return self.stats
        
    def getNextStats(self):
        payload = {}
        if (self.index >= len(self.stats)):
            return payload

        stats = self.stats[self.index]
        flavors = stats[2].split(',')

        payload['syrup_name'] = stats[1]
        payload['flavor_name_1'] = None
        payload['flavor_name_2'] = None
        payload['flavor_name_3'] = None
        payload['flavor_name_4'] = None

        try:
            if (flavors[0] != '0'):
                payload['flavor_name_1'] = self.flavors[int(flavors[0])]
            if (len(flavors) > 1) :
                payload['flavor_name_2'] = self.flavors[int(flavors[1])]
            if (len(flavors) > 2) :
                payload['flavor_name_3'] = self.flavors[int(flavors[2])]
            if (len(flavors) > 3) :
                payload['flavor_name_4'] = self.flavors[int(flavors[3])]
        except:
            pass

        payload['count'] = int(stats[4])
        payload['time'] = int(stats[5])
        payload['date'] = stats[3]
        self.index += 1
        return payload

    def setIotSynced(self):
        self.refreshDatabase.setIotSynced()

    
    def  getStatistics_example(self):
        today = date.today()
        
        payload = {}
        payload['syrup_name'] = "Coke"
        payload['flavor_name_1'] = None
        payload['flavor_name_2'] = "Lime"
        payload['flavor_name_3'] = None
        payload['flavor_name_4'] = None
        payload['count'] = 11
        payload['time'] = 1258
        payload['date'] =   "2016-04-11 12:00:00"

        return payload

    #
    # SQL Query  Syrups
    # 
    # returns json dictionary
    #
    def getConfiguration(self):
        payload = {}
        self.flavors = {}
        
        #please change mainui to write version to database so I can read it

        payload["sw_ui"] = "1.3.2" 

        rows = self.refreshDatabase.getSyrupBrandConfig()

        
        lv = 0
        rv = 0
        for r in rows:
           if (r[5] != '' and r[5] != None):    #left valve
              lv += 1
              payload["l_brand_name_" + str(lv)] = r[2]
              payload["l_brand_ratio_" + str(lv)] = r[4]
              payload["l_brand_water_" + str(lv)] = r[7]
           if (r[6] != '' and r[6] != None):    #right valve
              rv += 1
              payload["r_brand_name_" + str(rv)] = r[2]
              payload["r_brand_ratio_" + str(rv)] = r[4] 
              payload["r_brand_water_" + str(rv)] = r[7]

        rows = self.refreshDatabase.getSyrupFlavorConfig()

        lv = 0
        rv = 0
        for r in rows:
           self.flavors[int(r[0])] = r[2]
           if (r[5] != '' and r[5] != None):    #left valve
              lv += 1
              payload["l_flavor_name_" + str(lv)] = r[2]
              payload["l_flavor_ratio_" + str(lv)] = r[4]
           if (r[6] != '' and  r[6] != None):    #right valve
              rv += 1
              payload["r_flavor_name_" + str(rv)] = r[2]
              payload["r_flavor_ratio_" + str(rv)] = r[4]

        payload["mrss_url"] = self.refreshDatabase.dictionaryGet("revelUrl")
        crew_serve = self.refreshDatabase.dictionaryGet("portionControlEnabled").lower() == "true"
        payload["crew_serve"] = crew_serve
        payload["config_kit"]= self.refreshDatabase.dictionaryGet("config")
        flow_rates = self.refreshDatabase.getWaterFlowRates()
        payload["water_fr_pw"] = flow_rates["PW"]
        payload["water_fr_cw"] = flow_rates["CW"]
        payload["water_fr_mw"] = flow_rates["MW"]

        payload["l_mfv"] = getMFVVersion(1)
        payload["r_mfv"] = getMFVVersion(2)
        payload["unit_sn"] = self.refreshDatabase.manufacturerSetupGet("serialNumber")
        payload["door_sn"] = self.refreshDatabase.manufacturerSetupGet("doorSerialNumber")
        payload["ebox_sn"] = self.refreshDatabase.manufacturerSetupGet("eBoxSerialNumber")
        payload["modem_sn"] = self.refreshDatabase.manufacturerSetupGet("modemSerialNumber")
        payload["modem_mac"] = self.refreshDatabase.manufacturerSetupGet("modemMACAddress")
        payload["network_macs"] = getNetworkMACs()

        payload["sysmesg_level"] = self.refreshDatabase.dictionaryGet("sysMesgExtReportLevel")
        payload["sysmesg_cutoff"] = self.refreshDatabase.dictionaryGet("sysMesgExtReportCutoff")

        #print 'This is the configuration'
        #print payload
        #print 'This is the end of configuration'
        return payload
        

    def getConfiguration_example(self):
        payload = {}
        payload["sw_ui"] = "1.3.2" 

        payload["l_brand_name_1"] = "Coke"
        payload["l_brand_name_2"] = "Sobe Lifewater Yumberry"
        payload["l_brand_name_3"] = "Coke Zero"
        payload["l_brand_name_4"] = "Fanta Orange"
        payload["l_brand_name_5"] = "Fanta Ginger Ale"
        payload["l_brand_name_6"] = "Diet Pepsi"
        payload["l_brand_name_7"] = "Sprite"
        payload["l_brand_name_8"] = "Diet Mt Dew"
        payload["l_flavor_name_1"] = "Cherry"
        payload["l_flavor_name_2"] = "Lime"
        payload["l_flavor_name_3"] = "Vanilla"
        payload["l_flavor_name_4"] = "Raspberry"

        payload["l_brand_ratio_1"] = 3.2
        payload["l_brand_ratio_2"] = 3.1
        payload["l_brand_ratio_3"] = 3.1
        payload["l_brand_ratio_4"] = 3.1
        payload["l_brand_ratio_5"] = 3.1
        payload["l_brand_ratio_6"] = 3.1
        payload["l_brand_ratio_7"] = 3.1
        payload["l_brand_ratio_8"] = 3.1

        payload["l_flavor_ratio_1"] = 10.1
        payload["l_flavor_ratio_2"] = 10.1
        payload["l_flavor_ratio_3"] = 10.1
        payload["l_flavor_ratio_4"] = 10.1

        payload["r_brand_name_1"] = "Dr Pepper"
        payload["r_brand_name_2"] = "Diet Coke"
        payload["r_brand_name_3"] = "Coke Zero"
        payload["r_brand_name_4"] = "Fanta Orange"
        payload["r_brand_name_5"] = "Pepsi"
        payload["r_brand_name_6"] = "Diet Pepsi"
        payload["r_brand_name_7"] = "Sprite"
        payload["r_brand_name_8"] = "Diet Mt Dew"

        payload["r_flavor_name_1"] = "Cherry"
        payload["r_flavor_name_2"] = "Lime"
        payload["r_flavor_name_3"] = "Vanilla"
        payload["r_flavor_name_4"] = "Lemon"

        payload["r_brand_ratio_1"] = 3.1
        payload["r_brand_ratio_2"] = 3.1
        payload["r_brand_ratio_3"] = 3.1
        payload["r_brand_ratio_4"] = 3.1
        payload["r_brand_ratio_5"] = 3.1
        payload["r_brand_ratio_6"] = 3.1
        payload["r_brand_ratio_7"] = 3.1
        payload["r_brand_ratio_8"] = 3.1
        
        payload["r_flavor_ratio_1"] = 10.1
        payload["r_flavor_ratio_2"] = 10.1
        payload["r_flavor_ratio_3"] = 10.1
        payload["r_flavor_ratio_4"] = 10.1

        return payload
        
    def canTryDownload(self, url, timestamp):
        DICTIONARY_KEY = "iotUpdateDownload"
        URL_KEY = "url"
        TIMESTAMP_KEY = "timestamp"
        TRY_COUNT_KEY = "trycount"
        MAX_TRY_COUNT = 3 # Maximum number of attempts allowed on a single download.

        json_values = self.refreshDatabase.dictionaryGet(DICTIONARY_KEY)
        try:
            values = json.loads(json_values)
        except ValueError:
            values = {}
        old_url = values.get(URL_KEY, "")
        old_timestamp = values.get(TIMESTAMP_KEY, "")
        try:
            try_count = int(values.get(TRY_COUNT_KEY, 0))
        except ValueError:
            try_count = 0

        # Examine last requested download url and timestamp to see how they
        # compare to the current ones. If they are the same, make sure to only
        # try up to MAX_TRY_COUNT times in a row. This should save bandwidth and
        # there should be error codes sent over MQTT that can be used to debug
        # remotely. If the url or timestamp have changed, go ahead and download.
        if url == old_url and timestamp == old_timestamp:
            if try_count >= MAX_TRY_COUNT:
                return False
            else:
                values[TRY_COUNT_KEY] = try_count + 1
                json_values = json.dumps(values)
                self.refreshDatabase.dictionarySet(DICTIONARY_KEY, json_values)
                return True
        else:
            values[URL_KEY] = url
            values[TIMESTAMP_KEY] = timestamp
            values[TRY_COUNT_KEY] = 1
            json_values = json.dumps(values)
            self.refreshDatabase.dictionarySet(DICTIONARY_KEY, json_values)

        return True

    def clearPendingDownload(self):
        DICTIONARY_KEY = "iotUpdateDownload"
        self.refreshDatabase.dictionarySet(DICTIONARY_KEY, "")

    #
    # checkElog
    # Checks if there is anythng to send to Elog
    # 
    # returns json dictionary
    # 
    def checkELog(self):
        # get time of last check
        last_time_ascii = str(self.refreshDatabase.dictionaryGet("iotErrorTime"))
        utime = dbTime2uTime(last_time_ascii)
        # print 'last_time',  utime, last_time_ascii
        self.ELog = self.refreshDatabase.getSystemErrorMessages(utime)
        #set time of last send
        self.refreshDatabase.dictionarySet("iotErrorTime", uTime2dbTime(time.time()))
        return len(self.ELog) > 0
        
    def getAllErrors(self):
        self.index = 0
        return
        
    def getNextErrors(self):
        payload = {}
        if (self.index >= len(self.ELog)):
            return payload
        elog = self.ELog[self.index]
        try:
            payload['error_type'] = self.errorTable[elog[2]]
        except:
            payload['error_type'] = '9999'
        payload['start'] = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(elog[0]))
        payload['cleared'] = ""
        self.index += 1
        return payload
    #
    #
    # checkErrors
    # Checks if there is anythng to send to errors
    # 
    # returns boolean
    #
    def checkErrors(self):
        return False

    #
    # getErrors
    # A collection of current errors
    # An array of error code and message
    # eg. code: 1000, message:  Left nozzle mfv disconnected
    # Submiting empty array wil clear errors table in status enterprise page
    # 
    # returns json dictionary
    #
    def getErrors(self):

        payload = {}
        payload['Errors'] = {}
        # payload['Errors']['1000'] = "Left nozzle mfv disconnected"


        return payload



    def insertSystemMessage(self,  service_name, mtype, msg):
      self.refreshDatabase.insertSystemMessage(service_name, mtype, msg)

    def checkSysMesgsExt( self, severity, cutoff ):
        return self.refreshDatabase.checkSysMesgsExt( severity, cutoff )

    def getSysMesgsExt( self, severity, cutoff ):
        # Need to create a bulk update payload
        # format: { bulk = [ {payload}, {payload}, .... ] }
        payload = {}        
        bulk = []        
        final = {}        
        # Result is 0=id, 1=time, 2=level, 3=module, 4=state, 5=msg_id, 6=text   
        mesgs = self.refreshDatabase.getSysMesgsExt( severity, cutoff )
        if len(mesgs) > 0:            
            # Create a bulk message            
            # { "bulk" : [ {payload}, {payload}, ... ] }            
            index = 0            
            for row in mesgs:                
                payload = {}                
                payload["time"] = row[1]                
                payload["level"] = row[2]                
                payload["module"] = row[3]                
                payload["state"] = row[4]                
                payload["msg_id"] = row[5]                
                payload["text"] = row[6]                
                bulk.append( payload )            
            final["bulk"] = bulk        
        return final


    def setSysMesgsExtSyncd( self ):
        self.refreshDatabase.setSysMesgsExtSyncd()
