#!/usr/bin/python

# ******************************************************************************
# *                                                                            
# *   Copyright 2014 Cornelius, Inc. All rights reserved.                      
# *                                                                            
# *   This file is confidential and a trade secret of Cornelius, Inc.          
# *   The receipt of or possession of this file does not convey any rights     
# *   to reproduce or disclose its contents or to manufacture, use, or sell   
# *   anything it may describe, in whole, or in part, without the specific     
# *   written consent of Cornelius, Inc.                                       
# *                                                                            
# ******************************************************************************
# ------------------------------------------------------------------------------
#
#    DEVELOPED BY : Michal Augustyniak
#
# ------------------------------------------------------------------------------

import logging
from datetime import date
import DatabaseHelper
import json
import sys
import time
import csv
import os
import subprocess
from logger import Logger


def uTime2dbTime(utime):
    return time.strftime("%d-%m-%y %H:%M:%S", time.localtime(utime))


def dbTime2uTime(atime):
    dt = atime.split(' ')
    d = dt[0].split('-')
    t = dt[1].split(':')
    return time.mktime((int(d[2]) + 2000, int(d[1]), int(d[0]), int(t[0]), int(t[1]), int(t[2]), 0, 1, -1))
 
def getMFVVersion(boardNumber):
    return "0.0"

def getNetworkMACs():
    # Return all all network devices and mac addresses as a comma delimiter
    # list.
    network_devices = os.listdir("/sys/class/net/")
    mac_addresses = []
    for network_device in network_devices:
        if network_device == "lo":
            continue
        mac_address = file("/sys/class/net/%s/address" % network_device, "rb").read().strip()
        mac_addresses.append(mac_address)
    return ",".join([",".join(x) for x in zip(network_devices,mac_addresses)])

print getNetworkMACs()

class RefreshWrapper():

    def __init__(self):
        self.stats = []
        self.index = 1

        # Connect to database
        self.refreshDatabase = DatabaseHelper.DatabaseConnection("/opt/Refresh/Database/refresh_database_enc.db")
        self.systemType = self.refreshDatabase.dictionaryGet( "systemType" )


    def getTotals(self):
        payload = {}

        self.totals = self.refreshDatabase.getViperTotals() 
        
        if len(self.totals) > 0:
            for t in self.totals:
                payload[t[0]] = t[1]
        
        return payload
    #
    # SQL Query Statistics
    # 
    # Refresh statistics are stored on hourly basis
    # This function should query data for a passed hour.
    #
    # Once send correctly, send records should be deleted 
    # from Refresh Database or marked as synced
    # 
    # The idea is to eventually replicate this database on Enterprise
    # If different please discuss Michal, VInce and Slaw
    # 
    # returns json dictionary
    #
    # Michal's hardcoded data left in place for now with changed names

    def getAllStatistics(self):
        self.stats = {} # self.refreshDatabase.getPourStats()
        self.index = 0
        return self.stats
        
    def getNextStats(self):
        payload = {}
        #if (self.index >= len(self.stats)):
        return payload

        # stats = self.stats[self.index]
        # flavors = stats[2].split(',')

        # payload['syrup_name'] = stats[1]
        # payload['flavor_name_1'] = None
        # payload['flavor_name_2'] = None
        # payload['flavor_name_3'] = None
        # payload['flavor_name_4'] = None

        # try:
        #     if (flavors[0] != '0'):
        #         payload['flavor_name_1'] = self.flavors[int(flavors[0])]
        #     if (len(flavors) > 1) :
        #         payload['flavor_name_2'] = self.flavors[int(flavors[1])]
        #     if (len(flavors) > 2) :
        #         payload['flavor_name_3'] = self.flavors[int(flavors[2])]
        #     if (len(flavors) > 3) :
        #         payload['flavor_name_4'] = self.flavors[int(flavors[3])]
        # except:
        #     pass

        # payload['count'] = int(stats[4])
        # payload['time'] = int(stats[5])
        # payload['date'] = stats[3]
        # self.index += 1
        # return payload

    def setIotSynced(self):
        #self.refreshDatabase.setIotSynced()
        return

    
    # def  getStatistics_example(self):
    #     today = date.today()
        
    #     payload = {}
    #     payload['syrup_name'] = "Coke"
    #     payload['flavor_name_1'] = None
    #     payload['flavor_name_2'] = "Lime"
    #     payload['flavor_name_3'] = None
    #     payload['flavor_name_4'] = None
    #     payload['count'] = 11
    #     payload['time'] = 1258
    #     payload['date'] =   "2016-04-11 12:00:00"

    #     return payload

    #
    # SQL Query  Syrups
    # 
    # returns json dictionary
    #
    def getConfiguration(self):
        payload = {}
        
        # We need to put version somewhere that can be accessed everywhere

        payload["sw_ui"] = "1.3.2" 

        payload["mrss_url"] = self.refreshDatabase.dictionaryGet("revelUrl")
        
        self.config = self.refreshDatabase.getViperConfig() 
        
        if len(self.config) > 0:
            #count = 14
            for row in self.config:
                # MiM can't handle blank text for numeric values.
                # Unfortunately, there's no way to check what MiM is expecting
                # So this has to be hard-coded...
                if (row[1] is not None) and (len(row[1]) > 0):
                    payload[row[0]] = row[1]
                    # print t[0], " = ", t[1]
                elif 'rng' in row[0] or 'address' in row[0] or 'mrss' in row[0] or 'name' in row[0] or 'sw_' in row[0]:
                    payload[row[0]] = row[1]
                    # print t[0], " = ", t[1]
                # count -= 1
                # if (count <= 0):
                #     break

        self.config = self.refreshDatabase.getViperBrands() 
        
        if len(self.config) > 0:
            for row in self.config:
                # Barrel numbers are concatenated together when the same brand
                # is in mutiple barrels.  eg. 31 = barrel 3 & barrel 1
                for barNum in row[0]:
                    if (row[1] is not None) and (len(row[1]) > 0):
                        # A name was specified, use it
                        payload["b" + str(barNum) + "_name"] = row[1]
                    else:
                        # Extract the name from the file name
                        fileName = os.path.basename(row[2])
                        brandName, ext = os.path.splitext( fileName )
                        payload["b" + str(barNum) + "_name"] = brandName

        payload["sysmesg_level"] = self.refreshDatabase.dictionaryGet("sysMesgExtReportLevel")
        payload["sysmesg_cutoff"] = self.refreshDatabase.dictionaryGet("sysMesgExtReportCutoff")

        #print 'This is the configuration'
        #print payload
        #print 'This is the end of configuration'
        return payload

    def setEventsSyncd( self ):
        self.refreshDatabase.setViperEventsSynced()
        
    def canTryDownload(self, url, timestamp):
        DICTIONARY_KEY = "iotUpdateDownload"
        URL_KEY = "url"
        TIMESTAMP_KEY = "timestamp"
        TRY_COUNT_KEY = "trycount"
        MAX_TRY_COUNT = 3 # Maximum number of attempts allowed on a single download.

        json_values = self.refreshDatabase.dictionaryGet(DICTIONARY_KEY)
        try:
            values = json.loads(json_values)
        except ValueError:
            values = {}
        old_url = values.get(URL_KEY, "")
        old_timestamp = values.get(TIMESTAMP_KEY, "")
        try:
            try_count = int(values.get(TRY_COUNT_KEY, 0))
        except ValueError:
            try_count = 0

        # Examine last requested download url and timestamp to see how they
        # compare to the current ones. If they are the same, make sure to only
        # try up to MAX_TRY_COUNT times in a row. This should save bandwidth and
        # there should be error codes sent over MQTT that can be used to debug
        # remotely. If the url or timestamp have changed, go ahead and download.
        if url == old_url and timestamp == old_timestamp:
            if try_count >= MAX_TRY_COUNT:
                return False
            else:
                values[TRY_COUNT_KEY] = try_count + 1
                json_values = json.dumps(values)
                self.refreshDatabase.dictionarySet(DICTIONARY_KEY, json_values)
                return True
        else:
            values[URL_KEY] = url
            values[TIMESTAMP_KEY] = timestamp
            values[TRY_COUNT_KEY] = 1
            json_values = json.dumps(values)
            self.refreshDatabase.dictionarySet(DICTIONARY_KEY, json_values)

        return True

    def clearPendingDownload(self):
        DICTIONARY_KEY = "iotUpdateDownload"
        self.refreshDatabase.dictionarySet(DICTIONARY_KEY, "")

    #
    # checkElog
    # Checks if there is anythng to send to Elog
    # 
    # returns json dictionary
    # 
    def checkELog(self):
        return False
        
    def getAllErrors(self):
        self.index = 0
        return
        
    def getNextErrors(self):
        payload = {}
        return payload
    #
    #
    # checkErrors
    # Checks if there is anythng to send to errors
    # 
    # returns boolean
    #
    def checkErrors(self):
        return False

    #
    # getErrors
    # A collection of current errors
    # An array of error code and message
    # eg. code: 1000, message:  Left nozzle mfv disconnected
    # Submiting empty array wil clear errors table in status enterprise page
    # 
    # returns json dictionary
    #
    def getErrors(self):

        payload = {}
        return payload



    def insertSystemMessage(self,  service_name, mtype, msg):
      self.refreshDatabase.insertSystemMessage(service_name, mtype, msg)

    def checkSysMesgsExt( self, severity, cutoff ):
        return self.refreshDatabase.checkSysMesgsExt( severity, cutoff )

    def getSysMesgsExt( self, severity, cutoff ):
        # Need to create a bulk update payload
        # format: { bulk = [ {payload}, {payload}, .... ] }
        payload = {}        
        bulk = []        
        final = {}        
        # Result is 0=id, 1=time, 2=level, 3=module, 4=state, 5=msg_id, 6=text, 7=element   
        mesgs = self.refreshDatabase.getSysMesgsExt( severity, cutoff )
        if len(mesgs) > 0:            
            # Create a bulk message            
            # { "bulk" : [ {payload}, {payload}, ... ] }            
            index = 0            
            for row in mesgs:                
                payload = {}                
                payload["time"] = row[1]                
                payload["level"] = row[2]                
                payload["module"] = row[3]                
                payload["state"] = row[4]                
                payload["msg_id"] = row[5]                
                payload["text"] = row[6]                
                payload["element"] = row[7]
                bulk.append( payload )            
            final["bulk"] = bulk        
        return final


    def setSysMesgsExtSyncd( self ):
        self.refreshDatabase.setSysMesgsExtSyncd()
