#!/bin/bash
#
# Michal Augustyniak : michal.augustyniak@cornelius.com
#

# System name. Should not contain spaces
SYSTEM_NAME='Refresh'

# The type is separate from the name because the name is used
# to create directory structures.  There's a lot of stuff
# hard-coded for "Refresh", so we can't simply change this
# without a complete code review.
# Most importantly, the name of the database is hard-coded.
SYSTEM_TYPE=""

setupflag='FALSE'
portionControlEnabled='false'

###########################
cd $CORCO_TOOL_LOCATION
###########################

if [ -e /opt/Refresh/Database/refresh_database_enc.db ]; then
	# Get the name from the dictionary
	SYSTEM_TYPE=$(python ./DictionaryHelper.py systemType)
fi

if ( [ -z "$SYSTEM_TYPE" ] || [ "$SYSTEM_TYPE" == "None" ] ) && [ -e /opt/Refresh/systemType.txt ]; then
	# In case something is wrong with the database
	# read this file
	SYSTEM_TYPE=$(</opt/Refresh/systemType.txt)

	setupflag=$(python ./setupCheck.py)
fi

# TBD we need to develop a way of handling an invalid system name
# For now, we'll just revert to IDC
case $SYSTEM_TYPE in
	'idc')
	;;
	'viper')
	;;
	*)
		# If not specified, then default to IDC (Refresh)
		echo "Unexpected SYSTEM_TYPE = $SYSTEM_TYPE, reverting to 'IDC'"
		SYSTEM_TYPE='idc'
	;;
esac

if [ "$1" != "quiet" ] ; then
	# Only print this if there are no command line parameters
	echo "SYSTEM_TYPE: $SYSTEM_TYPE"
fi


# All Services paths should be relative to the system current location
# Example: 
# SYSTEM_CURRENT_LOCATION=/opt/SYSTEM_NAME/Versions/SYSTEM_NAME-current
# service path should be SERVICE_NAME/SERVICE_NAME_BIN

# System services, in the order you wish to start
SERVICES['WatchDog']='WatchDog/WatchDog.sh'
if [ "$SYSTEM_TYPE" != "viper" ]; then
	SERVICES['HardwareMgr']='HardwareMgr/HardwareMgr'
fi
SERVICES['ServiceUI']='ServiceUI/ServiceUI'
SERVICES['VideoPlayer']='VideoPlayer/VideoPlayer.py'
SERVICES['MainUI']='MainUI/MainUI'
SERVICES['IoT']='IoT/IoTMain.py'
SERVICES['Waiter']='manage-Waiter.sh'

if [ "$SYSTEM_TYPE" == "idc" ]; then
	if [ -e /opt/Refresh/Database/refresh_database_enc.db ]; then
	   portionControlEnabled=$(python ./DictionaryHelper.py portionControlEnabled)
	fi

	#echo "setupCustomized flag $setupflag"
	if [ "$setupflag" == "true" ] || [ "$setupflag" == "TRUE" ]; then
	        if [ "$portionControlEnabled" == "true" ]; then
	            # Don't start VideoPlayer when portion control enabled
	            # I am giving 10 seconds of time between ServiceUI and MainUI
	            # MainUI receives "database is locked" error if launched to quickly
	            # I assume ServiceUI making initialization yet.
	            START_ORDER=('HardwareMgr' 'ServiceUI' 10 'MainUI' 'IoT' 3 'WatchDog' )
	        else
	            START_ORDER=('HardwareMgr' 'ServiceUI' 'Waiter' 'VideoPlayer' 'MainUI' 'IoT' 3 'WatchDog' )
	        fi
	else
	        touch /tmp/service.lock
	        START_ORDER=('HardwareMgr' 'ServiceUI' 'Waiter' 'VideoPlayer' 'MainUI' 'IoT' 3 'WatchDog' )
	fi

	STOP_ORDER=('WatchDog' 1 'ServiceUI' 1 'MainUI' 1 'VideoPlayer' 1 'HardwareMgr' 1 'IoT' )
	
elif [ "$SYSTEM_TYPE" == "viper" ]; then
  if [ "$setupflag" != "true" ] && [ "$setupflag" != "TRUE" ]; then
    # If setup is required, then show init display during startup
    # This is used by manage-waiter.sh to pause corco during startup and
    # wait for the user to select one of the options on the screen.
    # This file is deleted in ServiceUI/qml/InitialCustomize.qml
    touch /tmp/service.lock
  fi
  START_ORDER=('ServiceUI' 'Waiter' 'VideoPlayer' 1 'MainUI' 'IoT' 3 'WatchDog' )
	STOP_ORDER=('WatchDog' 1 'ServiceUI' 1 'MainUI' 1 'VideoPlayer' 1 'IoT' )
fi

# Other services
AID_SERVICES['Updater']='SoftwareUpdater/SoftwareUpdater'

# WatchDog services
# Don't watch VideoPlayer when portion control enabled
if [ "$portionControlEnabled" != "true" ]; then
    WATCHDOG_SERVICES['VideoPlayer']='VideoPlayer/VideoPlayer.py'
fi

WATCHDOG_SERVICES['MainUI']='MainUI/MainUI'
WATCHDOG_SERVICES['ServiceUI']='ServiceUI/ServiceUI'

if [ "$SYSTEM_TYPE" == "idc" ]; then
	WATCHDOG_SERVICES['HardwareMgr']='HardwareMgr/HardwareMgr'
fi

WATCHDOG_SERVICES['IoT']='IoT/IoTMain.py'

###########################
# Return to starting directory
# Don't print the directory
cd - > /dev/null
###########################

