#!/bin/bash
#
# Michal Augustyniak : michal.augustyniak@cornelius.com
#

# CorcoTool location should be in 	ROOT_LOCATION/Libs/CorcoTool
# CorcoTool expects scripts in		ROOT_LOCATION/Scripts
# CorcoTool expects environment setup	ROOT_LOCATION/Scripts/corco-environment.sh
# , start, stop, kill script per service, example:
# 					ROOT_LOCATION/Scripts/stop-serviceName.sh
# 					ROOT_LOCATION/Scripts/start-serviceName.sh
# 					ROOT_LOCATION/Scripts/kill-serviceName.sh

declare LINK_NAME=`basename $0`
declare LINK_LOCATION=`dirname $0`
declare LINK_FULL_PATH="$LINK_LOCATION/$LINK_NAME"
declare CORCO_TOOL_FULL_PATH=`readlink -f $LINK_FULL_PATH`

declare -A AID_SERVICES
declare -A WATCHDOG_SERVICES
declare -A SERVICES
declare -A SYSTEM_VERSIONS

declare SYSTEM_TYPE
declare SYSTEM_NAME
declare SYSTEM_VERSION
declare ROOT_LOCATION
declare SYSTEM_SERVICE_LOG_LOCATION
declare START_ORDER
declare CORCO_TOOL_LOCATION=`dirname $CORCO_TOOL_FULL_PATH`
declare CORCO_TOOL_SCRIPTS_LOCATION

exitWithError() {
  echo "Error: $1"
  exit 1
}

envFile="$CORCO_TOOL_LOCATION/corco-environment.sh"
if [ ! -f $envFile ]; then
  echo "Error: missing $envFile file"
  exit 1
fi

. $envFile "quiet"
. $CORCO_TOOL_LOCATION/support-env.sh
. $CORCO_TOOL_LOCATION/support-help.sh
. $CORCO_TOOL_LOCATION/support-logger.sh
. $CORCO_TOOL_LOCATION/support-service.sh
. $CORCO_TOOL_LOCATION/support-system.sh
. $CORCO_TOOL_LOCATION/support-watchdog.sh
. $CORCO_TOOL_LOCATION/support-update.sh

export DISPLAY=:0

ARG1=$1
ARG2=$2
ARG3=$3

# create log location
mkdir -p /var/log/$SYSTEM_NAME

if [ "$ARG1" == "" ];then
  help  

elif [ "$ARG1" == "WatchDog" ] && [ "$ARG2" == "check" ]; then
  watchdog_check
  
elif service_exist $ARG1;  then
  service $ARG1 $ARG2 $ARG3
    
elif service_aid_exist $ARG1;  then
  service $ARG1 $ARG2 $ARG3
    
elif [ "$ARG1" == "start" ]; then
  service_start_all
  
elif [ "$ARG1" == "kill" ]; then
  service_kill_all
  killall -SIGKILL -r tail &> /dev/null
  
elif [ "$ARG1" == "stop" ]; then
  service_stop_all
  
elif [ "$ARG1" == "restart" ]; then
  echo "Restarting $SYSTEM_NAME"
  service_stop_all
  service_start_all
  
elif [ "$ARG1" == "status" ]; then
  print_service_status_all
  
elif [ "$ARG1" == "version" ]; then
  print_system_current_version
  
elif [ "$ARG1" == "import" ]; then
  update_import "$2"

elif [ "$ARG1" == "versions" ]; then
  print_system_versions

elif [ "$ARG1" == "install" ]; then
  update_install $2

elif [ "$ARG1" == "validate" ] && [ "$ARG2" == "update" ]; then
  update_validate "$3"

elif [ "$ARG1" == "firmware" ] && [ "$ARG2" == "update" ]; then
  update_firmware $3 $4

elif [ "$ARG1" == "mem" ]; then
  print_service_memory_all

elif [ "$ARG1" == "help" ]; then
  help
elif [ "$ARG1" == "env" ]; then
  print_system_env
else
  help
fi