#!/bin/bash

SERVICE_NAME='IoT'
SERVICE_LOCATION="../$SERVICE_NAME"
SERVICE_BIN='IoTMain.py'
SERVICE_LOG_FILE='/dev/null'
SERVICE_LOG_FILE="/var/log/Refresh/$SERVICE_NAME.log"


function printUsage {
  echo "USAGE: $0 start/stop/kill/status" 
  exit 1
}

 
function executeStart {  

  cd $SERVICE_LOCATION
  ./$SERVICE_BIN >> $SERVICE_LOG_FILE 2>&1 &  
  exit 0
}


function executeKill {

  # build a list of all the processes that have the following text in their names
  iotProcessList=`ps -ef | grep "IoTMain.py" | awk '{ print $2 }'`
  numProcesses=`ps -ef  | grep "IoTMain.py" | awk '{ print $2 }' | wc -l`

  # if we have no processes to kill, then exit
  if [ $numProcesses -lt 1 ]; then
    echo "no IoT PIDs to kill"
    exit 0
  fi

  # iterate thru the list and kill all the processes
  while read -r line;
  do
    #echo "killing IoT PID $line"
    kill "$line" 2> /dev/null
  done <<< "$iotProcessList"

  exit 0
}


function executeStop {
  executeKill
}


function executeStatus {

  numProcesses=`pgrep -x "IoTMain.py" | wc -l`
  
  if [[ numProcesses -gt 0 ]]; then
    # we found at least one process
    exit 0
  else
    # no processes found
    exit 1
  fi
}


# check for the correct number of parameters
if [ "$#" -lt 1 ]; then  
  printUsage
fi

case $1 in
"start")
  executeStart
  ;;
"stop")
  executeStop
  ;;
"kill")
  executeKill
  ;;
"status")
  executeStatus
  ;;
esac









