#!/bin/bash

SERVICE_NAME='MainUI'
SERVICE_LOCATION="../$SERVICE_NAME"
SERVICE_BIN='MainUI'
SERVICE_LOG_FILE="/var/log/Refresh/$SERVICE_NAME.log"


function printUsage {
  echo "USAGE: $0 start/stop/kill/status" 
  exit 1
}


function executeStart {  

##########################
  cd $SERVICE_LOCATION
##########################

	if [ "$SYSTEM_TYPE" == "viper" ]; then
	  nice --20 ./$SERVICE_BIN >> $SERVICE_LOG_FILE 2>&1 &
	else
	    # Check if the system has the Zytronic diver installed...
	    # Get the Linux version
	    linuxVer=$(uname -r)
	    # Find any file name that starts with zxy...
	    zxyFound=$(find /lib/modules/$linuxVer -name zxy*)

	    if [ -z "$zxyFound" ]; then
	      echo "Zytronics control missing"
	      ./$SERVICE_BIN >> $SERVICE_LOG_FILE 2>&1 &
	    else
	  	  n=0
	  	  while [ $n -lt 5 ]; do
    
	  	    INTERFACE=`ls -l /dev/input/by-id/ | grep 'Zytronic' | awk '{ print $11 }' | awk -F'/' '{ print $2 }'`
	  	    if [ "$INTERFACE" == "" ]; then
	  	      echo "Attempting to install Zytronics modules..."
	  	      modprobe -r zxy200_usb
	  	      modprobe -r zxy200_common
        
	  	      modprobe zxy200_usb
	  	      modprobe zxy200_common
	  	      sleep 5
	  	    else 
	  	      break
	  	    fi
    
	  	    let n=n+1
	  	  done

	      ./$SERVICE_BIN -plugin EvdevTouch:/dev/input/$INTERFACE -platform xcb >> $SERVICE_LOG_FILE 2>&1 &
	    fi

	fi
	
  exit 0
}


function executeKill {

  pkill -x $SERVICE_BIN
  exit 0
}


function executeStop {
  dbus-send --type=method_call --session --dest=org.refresh.mainUI / org.refresh.mainUI.quitApp
  exit 0
}


function executeStatus {

  numProcesses=`pgrep -x $SERVICE_NAME | wc -l`
  
  if [[ numProcesses -gt 0 ]]; then
    #echo "we found at least on process"
    exit 0
  else
    #echo "no processes found"
    exit 1
  fi
}


# check for the correct number of parameters
if [ "$#" -lt 1 ]; then  
  printUsage
fi

case $1 in
"start")
  executeStart
  ;;
"stop")
  executeStop
  ;;
"kill")
  executeKill
  ;;
"status")
  executeStatus
  ;;
esac


