#!/bin/bash
#
# additional refresh setup
#
# Michal Augustyniak : michal.augustyniak@cornelius.com
#

echo ""

read -p "* Set logrotation ? [y/n]" -n 1 -r
echo    # (optional) move to a new line
if [[ $REPLY =~ ^[Y|y]$ ]]
then
  ./set-logrotate.sh
fi

read -p "* Fix sudorex ? [y/n]" -n 1 -r
echo    # (optional) move to a new line
if [[ $REPLY =~ ^[Y|y]$ ]]
then
  ./set-sudo.sh
fi

read -p "* Set autostart ? [y/n]" -n 1 -r
echo    # (optional) move to a new line
if [[ $REPLY =~ ^[Y|y]$ ]]
then
  ./set-autostart.sh
fi

read -p "* Fix ttyUSB mount access ? [y/n]" -n 1 -r
echo    # (optional) move to a new line
if [[ $REPLY =~ ^[Y|y]$ ]]
then
  ./fix-ttyUSB-mount.sh
fi

read -p "* Change boot logo ? [y/n]" -n 1 -r
echo    # (optional) move to a new line
if [[ $REPLY =~ ^[Y|y]$ ]]
then
  ./change-boot-logo.sh
fi

read -p "* Fix Grub ? [y/n]" -n 1 -r
echo    # (optional) move to a new line
if [[ $REPLY =~ ^[Y|y]$ ]]
then
  ./fix-grub.sh
fi

read -p "* Install software dependencies ? [y/n]" -n 1 -r
echo    # (optional) move to a new line
if [[ $REPLY =~ ^[Y|y]$ ]]
then
  echo "Installing Software dependencies"
  add-apt-repository ppa:mc3man/trusty-media -y
  apt-get update -y

  apt-get install \
    build-essential \
    libdbus-c++-dev \
    libdbus-1-dev \
    libssl-dev \
    dbus \
    libftdi-dev \
    libpoppler-qt5-1 \
    libpoppler-qt5-dev \
    freeglut3-dev \
    lm-sensors -y

  apt-get install \
    python-gi \
    gir1.2-gtksource-3.0 \
    python-netifaces \
    gstreamer1.0* \
    gstreamer0.10-ffmpeg \
    gstreamer0.10 -y        

  apt-get install \
    gksu \
    sqlite3 \
    sqlitebrowser -y
  
  apt-get install \
    sqlcipher \
    libsqlcipher-dev \
    libsqlcipher0 \
    libsqlcipher0-dbg -y
    
  apt-get install ssh vim git gdb -y
  
  echo "Install SqlCipher"
  git clone https://github.com/sqlcipher/sqlcipher
  cd sqlcipher
  ./configure \
  --enable-tempstore=yes \
  CFLAGS="-DSQLITE_HAS_CODEC -DSQLITE_ENABLE_FTS3 -DSQLITE_ENABLE_FTS3_PARENTHESIS" \
  LDFLAGS="-lcrypto"
  make
  sudo make install
  cd ..
  
  echo "Install Python sqlcipher support"
  git clone https://github.com/leapcode/pysqlcipher/
  cd pysqlcipher
  python setup.py build_sqlcipher  # Build against the system libsqlcipher
  sudo python setup.py install 
  cd ..
  echo "Done"
fi

read -p "* Upgrade system? [y/n]" -n 1 -r
echo    # (optional) move to a new line
if [[ $REPLY =~ ^[Y|y]$ ]]
then
  echo "Upgrading"
  apt-get upgrade -y
  echo "Done"
fi

read -p "* Install ZyConfig ? [y/n]" -n 1 -r
echo    # (optional) move to a new line
if [[ $REPLY =~ ^[Y|y]$ ]]
then
  ./install-zyconfig.sh
fi
read -p "* Start ZyConfig ? [y/n]" -n 1 -r
echo    # (optional) move to a new line
if [[ $REPLY =~ ^[Y|y]$ ]]
then
  ZyConfig
fi

read -p "* Downoad and install QT ? [y/n]" -n 1 -r
echo    # (optional) move to a new line
if [[ $REPLY =~ ^[Yy]$ ]]
then
  cd;
  
  if [ ! -f "qt-opensource-linux-x64-5.3.1.run" ]
  then
    wget http://download.qt-project.org/official_releases/qt/5.3/5.3.1/qt-opensource-linux-x64-5.3.1.run
  fi
  
  chmod +x qt-opensource-linux-x64-5.3.1.run
  ./qt-opensource-linux-x64-5.3.1.run
  QTDIR="/opt/Qt5.3.1"
  while [ ! -f $QTDIR/5.3/gcc_64/bin/qmake ]; do
    echo -n "Where is Qt5.3.1 directory ? gimme the ful path!: "
    read QTDIR
    echo "You typed: $QTDIR"
  done
  sudo mkdir -p /usr/local/bin
  sudo rm -f /usr/bin/qmake
  sudo rm -f /usr/local/bin/qmake
  sudo ln -s $QTDIR/5.3/gcc_64/bin/qmake /usr/local/bin/qmake
fi

read -p "* Install MFV firmware ? [y/n]" -n 1 -r
echo    # (optional) move to a new line
if [[ $REPLY =~ ^[Yy]$ ]]
then  
  sudo mkdir -p /opt/Refresh/Versions
  sudo cp /home/cornelius/Refresh/scripts/resources/RefreshMFV_*.hex /opt/Refresh/Versions/;  
fi

# add the USB automount directory
sudo mkdir -p /media/cornelius



echo "Finito"
