#!/bin/bash
#
# Michal Augustyniak : michal.augustyniak@cornelius.com
#

service_get_bin_from_path()
{
  servicePath=$1
  bin=${servicePath##*/}
  echo $bin
}

service_get_bin_from_name()
{
  serviceName=$1
  servicePath=$(services_get_relative_path $serviceName)
  echo $(service_get_bin_from_path $servicePath)
}

service_get_root_from_path()
{
  servicePath=$1
  echo $(echo "$servicePath" | cut -d "/" -f1)
}

# returns
# SERVICE_FOLDER/SERVICE_BIN
services_get_relative_path()
{
  serviceName=$1
  if service_exist $serviceName ; then
    echo ${SERVICES[$serviceName]}
  elif service_aid_exist $serviceName ;then
    echo ${AID_SERVICES[$serviceName]}
  elif service_watchdog_exist $serviceName ;then
    echo ${WATCHDOG_SERVICES[$serviceName]}
  fi
}

# returns
# SERVICE_FOLDER
services_get_path()
{
  serviceName=$1
  if service_exist $serviceName ; then
    echo $(service_get_root_from_path ${SERVICES[$serviceName]})
  elif service_aid_exist $serviceName ;then
    echo $(service_get_root_from_path ${AID_SERVICES[$serviceName]})
  elif service_watchdog_exist $serviceName ;then
    echo $(service_get_root_from_path ${WATCHDOG_SERVICES[$serviceName]})
  fi
}

# returns
# ROOT_SYSTEM/SERVICE_FOLDER/SERVICE_BIN
service_get_absolute_path()
{
  serviceName=$1
  echo "$ROOT_LOCATION/$(services_get_path $serviceName)"
}

service_stop()
{
  cd $CORCO_TOOL_SCRIPTS_LOCATION
  serviceName=$1
  
  if [ ! -e "manage-$serviceName.sh" ]; then
    echo "Couldn't find 'manage-$serviceName.sh' in $CORCO_TOOL_SCRIPTS_LOCATION"
    return;
  fi
  
  ./manage-$serviceName.sh "stop"
  
  n=0
  while [ $n -lt 5 ]; do
    if service_is_running $serviceName; then
      echo -en "\033[100DStopping Service $serviceName $n"s 
    else
      echo -en "\E[100D"
      echo -en "\E[K"
      log "Stopping Service $serviceName" $LOG_OK "OK"
      return
    fi
  sleep 1
  let n=n+1
done

echo -en "\E[100D"
echo -en "\E[K"
log "Stopping Service $serviceName" $LOG_FAILED "FAILED"

if [ $n == 5 ]; then
  service_kill $serviceName
fi
}

service_start()
{
  cd $CORCO_TOOL_SCRIPTS_LOCATION
  serviceName=$1
  
  if [ ! -e "manage-$serviceName.sh" ]; then
    echo "Couldn't find 'manage-$serviceName.sh' in $CORCO_TOOL_SCRIPTS_LOCATION"
    return;
  fi
  
  ./manage-$serviceName.sh "start"
  
  if service_is_running $serviceName ; then
    log "Starting Service $serviceName" $LOG_OK "OK"
  else
    log "Starting Service $serviceName" $LOG_FAILED "FAILED"
  fi
}

service_kill()
{
  cd $CORCO_TOOL_SCRIPTS_LOCATION

  serviceName=$1
  
  if [ ! -e "manage-$serviceName.sh" ]; then
    echo "Couldn't find 'manage-$serviceName.sh' in $CORCO_TOOL_SCRIPTS_LOCATION"
    return;
  fi
  
  ./manage-$serviceName.sh "kill"
  
  if service_is_running $serviceName ; then
    log "Killing Service $serviceName" $LOG_FAILED "FAILED"
  else
    log "Killing Service $serviceName" $LOG_OK "OK"
  fi
}

service_stop_all()
{
  print_system_current_version
  
  for i in "${STOP_ORDER[@]}"
  do
    if [ "$i" -eq "$i" ] 2>/dev/null; then
      echo "Sleep $1 seconds/s"
      sleep $i
    else
      serviceName=$i
      service_stop $serviceName
    fi
  done    
  
  echo ""
}

service_start_all()
{
  print_system_current_version
  
  for i in "${START_ORDER[@]}" 
  do
    if [ "$i" -eq "$i" ] 2>/dev/null; then
      echo "Sleep $i second/s"
      sleep $i
    else
      serviceName=$i
      service_start $serviceName
    fi
  done 
  
  echo ""
}

service_kill_all()
{
  print_system_current_version
  
  for i in "${!SERVICES[@]}"
  do
    serviceName=$i
    service_kill $serviceName
  done

  
  for i in "${!AID_SERVICES[@]}"
  do
    serviceName=$i
    service_kill $serviceName
  done
  
  for i in "${!WATCHDOG_SERVICES[@]}"
  do
    serviceName=$i
    service_kill $serviceName
  done
  echo ""
}

print_service_status_all()
{
  print_system_current_version
  
  echo "[Services]"
  for i in "${!SERVICES[@]}"
  do
    serviceName=$i
    print_service_status $serviceName
  done

  if [ ! "${#AID_SERVICES[@]}" == "0" ]; then
    echo -e "\n[AID Services]"
    for i in "${!AID_SERVICES[@]}"
    do
      serviceName=$i
      print_service_status $serviceName
    done
  fi
  
  if [ ! "${#WATCHDOG_SERVICES[@]}" == "0" ]; then
    echo -e "\n[Watchog Services]"
    for i in "${!WATCHDOG_SERVICES[@]}"
    do
      serviceName=$i
      print_service_status $serviceName
    done
  fi
  
  echo ""
}

print_service_status()
{
  serviceName=$1
  if service_is_running $serviceName; then
    log "$serviceName" "$LOG_OK" "RUNNING"
  else
    log "$serviceName" "$LOG_FAILED" "NOT RUNNING"
  fi
}

print_service_ui_status()
{
  serviceName=$1
  if service_is_running $serviceName; then
    echo "RUNNING"
  else
    echo "NOT RUNNING"
  fi
}

service_exist()
{
  printf "%s\n" ${!SERVICES[@]} | grep -q $1 && return 0 || return 1
}

service_aid_exist()
{
  printf "%s\n" ${!AID_SERVICES[@]} | grep -q $1 && return 0 || return 1
}

service_watchdog_exist()
{
  printf "%s\n" ${!WATCHDOG_SERVICES[@]} | grep -q $1 && return 0 || return 1
}

print_service_memory_all()
{
  print_system_current_version
  
  for i in "${!SERVICES[@]}"
  do
    serviceName=$i
    print_service_memory $serviceName false
  done
  
  echo -e ""
}

print_service_memory()
{
  serviceName=$1
  
  bin=$(service_get_bin_from_name $serviceName)
  MYPID=`pgrep -x $bin`
  totalOnly=$2
  
  Mem=0
  Rss=0
  Shared=0
  Private=0
  Swap=0
  Pss=0

  if [ -f /proc/$MYPID/smaps ]; then
    Rss=`echo 0 $(cat /proc/$MYPID/smaps  | grep Rss | awk '{print $2}' | sed 's#^#+#') | bc;`
    Shared=`echo 0 $(cat /proc/$MYPID/smaps  | grep Shared | awk '{print $2}' | sed 's#^#+#') | bc;`
    Private=`echo 0 $(cat /proc/$MYPID/smaps  | grep Private | awk '{print $2}' | sed 's#^#+#') | bc;`
    Swap=`echo 0 $(cat /proc/$MYPID/smaps  | grep Swap | awk '{print $2}' | sed 's#^#+#') | bc;`
    Pss=`echo 0 $(cat /proc/$MYPID/smaps  | grep Pss | awk '{print $2}' | sed 's#^#+#') | bc;`
    Mem=`echo "$Rss + $Shared + $Private + $Swap + $Pss"|bc -l`
  fi
  
  if [  "$totalOnly" == true ]; then
    echo "Service " $1 
    echo "Pid     " "/proc/$MYPID/smaps"
    echo "Rss     " $Rss"KB"
    echo "Shared  " $Shared"KB"		
    echo "Private " $Private"KB"
    echo "Swap    " $Swap"KB"
    echo "Pss     " $Pss"KB"
    echo "========================";
    echo "Mem "$Mem"KB"
    echo "";
  else
    echo "$serviceName" "$((Mem / 1024))MB"
  fi
}

service_is_running()
{        
  cd $CORCO_TOOL_SCRIPTS_LOCATION
  serviceName=$1
  
  if [ ! -e "manage-$serviceName.sh" ]; then
    echo "Couldn't find 'manage-$serviceName.sh' in $CORCO_TOOL_SCRIPTS_LOCATION"
    return;
  fi
  
  ./manage-$serviceName.sh "status"  
  if [[ $? > 0 ]]; then
    #echo "service was not running"
    return 1
  else
    #echo "service was running"
    return 0
  fi 
}

service()
{
  serviceName=$1
  option=$2
  
  cd $CORCO_TOOL_SCRIPTS_LOCATION
  
  if [ "$option" == "start" ]; then
    service_start $serviceName

  elif [ "$option" == "stop" ]; then
    service_stop $serviceName

  elif [ "$option" == "kill" ]; then
    service_kill $serviceName

  elif [ "$option" == "log" ]; then
    tail -f -n 200 /var/log/$SYSTEM_NAME/$serviceName.log

  elif [ "$option" == "mem" ]; then
    print_system_current_version
    print_service_memory $serviceName true

  elif [ "$option" == "status" ]; then
    print_service_status $serviceName

  elif [ "$option" == "ui_status" ]; then
    print_service_ui_status $serviceName

  else
    help

  fi
}
