#!/bin/bash
#
# Michal Augustyniak : michal.augustyniak@cornelius.com
#

update_import()
{
  file="$1"
  cd "$CORCO_TOOL_SCRIPTS_LOCATION"
  ./import.sh "$file"
}

update_install()
{
  versionNumber=$1
  if [ "$versionNumber" == "" ]
  then
    echo "Error: Update version not specified"
    exit 1
  fi  
  
  if [ ! -f $SYSTEM_VERSIONS_LOCATION/$versionNumber/Scripts/install.sh ]; then
    echo "Error: Couldn't find installer for this version"
    echo "...location: $SYSTEM_VERSIONS_LOCATION/$versionNumber/Scripts/install.sh"
    exit 1
  fi  
  
  cd "$SYSTEM_VERSIONS_LOCATION/$versionNumber/Scripts"
  ./install.sh
  exit 0
}

do_update_validate()
{
    filePath=$1
    tmpDirectory=$2

    # Unpack version into this file.
    tar -xf "$filePath" -C "$tmpDirectory"
    if [[ $? != 0 ]]; then
        echo "Could not unpack $filePath"
        return 1
    fi

    # Make sure checksum matches the expected value...
	
	# Expected update filename is "cornelius-[system name]-x.y.z.tar.gz"
	# and its expected to contain a directory called "[system name]-x.y.z"
	# Extract the filename
    softwareDirectory=$(basename "$filePath")
	# Remove extension
	# If you started with /opt/Refresh/Updates/cornelius-whatever-x.y.z.tar.gz
	# you'll end up with cornelius-whatever-x.y.z
	softwareDirectory=${softwareDirectory%%.tar.gz}
	# Finally remove the cornelius- prefix
	softwareDirectory=${softwareDirectory##cornelius-}

	if [ ! -d "$tmpDirectory/$softwareDirectory" ] ; then
		echo "Missing expected extracted directory: $softwareDirectory"
		return 1
	fi
	
    cd "$tmpDirectory/$softwareDirectory"

    [ -f checksum.md5sum ]
    if [[ $? != 0 ]]; then
        echo "checksum.md5sum is missing"
        return 1
    fi
    EXPECTED_CHECKSUM=`cat checksum.md5sum`
    CALCULATED_CHECKSUM=`find . -type f -not -name "*.md5sum" -exec md5sum {} + | awk '{print $1}' | sort | md5sum`
    if [[ "$EXPECTED_CHECKSUM" != "$CALCULATED_CHECKSUM" ]]; then
        echo "Checksums did not match. Expected $EXPECTED_CHECKSUM, Calculated $CALCULATED_CHECKSUM"
        return 1
    fi

    echo "Software valid"
    return 0
}

update_validate()
{
    filePath=$1

    # Create a temp directory to unpack into.
    tmpDirectory=$(mktemp -d) || exit 1

    # Unpack and validate.
    do_update_validate "$filePath" "$tmpDirectory"
    result=$?

    # Clean-up.
    rm -rf tmpDirectory

    exit $result
}

update_firmware()
{
  if [ ! -f "$1" ]; then 
    echo "Error: Firmware file not specified"
    exit 1;
  fi
  
  cd $SYSTEM_VERSIONS_LOCATION/Refresh-current/SoftwareUpdater
  echo "Updating firmware, please wait..."
  ./SoftwareUpdater --update $1  
  exit 0
}
