#!/bin/bash
#
# Backs up current Refresh configuration
#
# Jeff Joray : jeff.joray@cornelius.com
#
#
#
NORMAL=0
ERROR=1

exitWithError() {
  echo "!!!!!!!!!!!!!!!!!!!!!!!!!!"
  echo "!!! Exiting with error !!!"
  echo "!!!!!!!!!!!!!!!!!!!!!!!!!!"
  echo "Info: $1"
  
  #show an exit countdown
  displayCountdown 20
  
  exit $ERROR
}

displayCountdown() {
  max=$1
  for ((i=$max;i>0;i-=1)); do
    echo -en "\rexiting in $i  "
    sleep 1
  done
  echo -e "\rdone           "
}

# check that the destination location was passes in as the first arg
[ $# -gt 0 ] || exitWithError "target location was not specified"
targetLocation="$1" || exitWithError "unable to set the target location from provided input parameters"
systemType="$2"
echo "targetLocation = $targetLocation, systemType = $systemType"

declare -a DATABASE_TABLE_LIST=("dictionary" "syrups" "waterMapping" "playlist" "playlistFiles" "flavorCombos")

if [ "$systemType" == "viper" ] ; then
  DATABASE_TABLE_LIST=("dictionary" "brands" "playlist" "playlistFiles" )
else
  systemType="refresh"
fi


REFRESH_ROOT_FOLDER="/opt/Refresh/"
REFRESH_VIDEOS_FOLDER="/Videos/Shared/"
REFRESH_IMAGES_FOLDER="/Pictures/Shared/"
REFRESH_MANUALS_FOLDER="/Manuals/"
TEMP_FOLDER_NAME="${systemType}-config"
TAR_CONFIG_FILENAME="$TEMP_FOLDER_NAME-`date \"+%F_%H%M\"`.tar.gz"
DATABASE_LOCATION="/Database/"
DATABASE_FILE="refresh_database_enc.db"
SCRIPTS_FOLDER="$REFRESH_ROOT_FOLDER/Versions/Refresh-current/ServiceUI/Scripts"


# go to the starting directory
cd "$REFRESH_ROOT_FOLDER" || exitWithError "Refresh root folder doesn't exist on this machine"

# create the temp? configuration folder
echo "creating temporary directory"
mkdir -p "$TEMP_FOLDER_NAME" || exitWithError "Unable to create temporary folder to hold configuration data"

# copy videos into the configuration folder
echo "copying video files. please wait..."
cp -r ".$REFRESH_VIDEOS_FOLDER." "$TEMP_FOLDER_NAME/Videos" || exitWithError "Unable to copy videos into configuration folder"

# copy images into the configuration folder
echo "copying image files. please wait..."
cp -r ".$REFRESH_IMAGES_FOLDER." "$TEMP_FOLDER_NAME/Images" || exitWithError "Unable to copy images into configuration folder"

# copy manuals into the configuration folder
echo "copying manuals files. please wait..."
cp -r ".$REFRESH_MANUALS_FOLDER." "$TEMP_FOLDER_NAME/Manuals" || exitWithError "Unable to copy images into configuration folder"


# export all the needed database tables
for tableName in ${DATABASE_TABLE_LIST[@]}; do 
  echo "exporting table $tableName"
  #sqlite3 -csv ".$DATABASE_LOCATION$DATABASE_FILE" "SELECT * FROM $tableName;" > "$TEMP_FOLDER_NAME/$tableName.csv" || exitWithError "couldn't export table $tableName"

  python $SCRIPTS_FOLDER/configurationBackupHelper.pyc ".$DATABASE_LOCATION$DATABASE_FILE"   "$TEMP_FOLDER_NAME/$tableName.csv" $tableName || exitWithError "couldn't export $tableName from file"

done

# create the tarball
echo "creating configuration package. please wait..."
tar -zcvf "$TAR_CONFIG_FILENAME" "$TEMP_FOLDER_NAME" || exitWithError "Unable to compress the configuration folder"
rm -r $TEMP_FOLDER_NAME || exitWithError "Unable to delete temporary configuration folder"
  
# copy tar to the media device
echo "copying configuration package to target. please wait..."
rsync --info=progress2 "$TAR_CONFIG_FILENAME" "$targetLocation/" || exitWithError "Unable to copy configuration file to target destination"

# delete the local tar file
rm "$TAR_CONFIG_FILENAME" 

echo "Configuration backup completed successfully"
displayCountdown 5
exit $NORMAL

