#!/bin/bash
#
# Restores Refresh configuration from file
#
# Jeff Joray : jeff.joray@cornelius.com
#
#
NORMAL=0
ERROR=1

exitWithError() {
  #echo "!!!!!!!!!!!!!!!!!!!!!!!!!!"
  #echo "!!! Exiting with error !!!"
  #echo "!!!!!!!!!!!!!!!!!!!!!!!!!!"
  echo "error Info: $1"
  sleep 1
  echo "ERROR CASE"
  #show an exit countdown
  displayCountdown 10
  exit $ERROR
}

displayCountdown() {
  max=$1
  for ((i=$max;i>0;i-=1)); do
    #echo -en "\rexiting in $i"
    sleep 1
  done
  echo -e "\rdone           "
}

RESTART=$2
TRUE="true"
REFRESH_ROOT_FOLDER="/opt/Refresh"
REFRESH_VIDEOS_FOLDER="/Videos/Shared"
REFRESH_IMAGES_FOLDER="/Pictures/Shared"
REFRESH_MANUALS_FOLDER="/Manuals"
DATABASE_LOCATION="/Database"
DATABASE_FILE="$REFRESH_ROOT_FOLDER$DATABASE_LOCATION/refresh_database_enc.db"
TEMP_FOLDER_NAME="refresh-config"
SCRIPTS_FOLDER="$REFRESH_ROOT_FOLDER/Versions/Refresh-current/ServiceUI/Scripts"

# check that all the refresh resources exist
echo "checking system configuration..."
cd $REFRESH_ROOT_FOLDER ] || exitWithError "unable to change to the Refresh root directory ($REFRESH_ROOT_FOLDER)"
[ -d .$REFRESH_VIDEOS_FOLDER ] || exitWithError "refresh videos folder was missing. please check that refresh software is correctly installed on this machine"
[ -d .$REFRESH_IMAGES_FOLDER ] || exitWithError "refresh images folder was missing. please check that refresh software is correctly installed on this machine"
[ -d .$REFRESH_MANUALS_FOLDER ] || exitWithError "refresh manuals folder was missing. please check that refresh software is correctly installed on this machine"
[ -f $DATABASE_FILE ] || exitWithError "refresh database file not found. please check that refresh software is correctly installed on this machine"
# check that a configuration file is all good in the hood
echo "checking input parameters..."
[ $# -gt 0 ] || exitWithError "configuration file was not specified"
fullConfigurationFilePath="$1" || exitWithError "unable to set the configuration file from provided input parameters"
configurationFilePath=${fullConfigurationFilePath%\/*}\/
configurationFileName=${fullConfigurationFilePath##*\/}
#echo "config file path: $configurationFilePath"
#echo "config file name: $configurationFileName"
[ -d "$configurationFilePath" ] || exitWithError "provided configuration file path ($configurationFilePath) was not found"
cd $configurationFilePath || exitWithError "unable to change to directory ($configurationFilePath)"
[ -f "$configurationFileName" ] || exitWithError "provided configuration file name ($configurationFileName) was not found"
cd $REFRESH_ROOT_FOLDER || exitWithError "unable to change to directory ($REFRESH_ROOT_FOLDER) after configuration file check"
  
# copy the configuration file onto the system
echo "copying configuration file to system..."
rsync --info=progress2 $configurationFilePath$configurationFileName $REFRESH_ROOT_FOLDER || exitWithError "unable to copy configuration file to machine"

# untar the configuration file
echo "unpacking configuration file..."
tar -xf "$configurationFileName" -C $REFRESH_ROOT_FOLDER || exitWithError "unable to unpack configuration file"

# SHOULD WE DO THIS
# delete the configuration file
echo "removing configuration file..."
rm "$configurationFileName" || exitWithError "unable to remove tar configuration file"

# check that the temp directory we are expecting exists now
#echo "checking for temporary configuration folder..."
[ -d $TEMP_FOLDER_NAME ] || exitWithError "temp configuration folder ($TEMP_FOLDER_NAME) was not found"

# delete all the existing videos and images
echo "deleting existing media..."
rm -r "$REFRESH_ROOT_FOLDER$REFRESH_VIDEOS_FOLDER" || exitWithError "unable to delete the contents of $REFRESH_VIDEOS_FOLDER"
mkdir "$REFRESH_ROOT_FOLDER$REFRESH_VIDEOS_FOLDER" || exitWithError "unable to re-create $REFRESH_VIDEOS_FOLDER"
rm -r "$REFRESH_ROOT_FOLDER$REFRESH_IMAGES_FOLDER" || exitWithError "unable to delete the contents of $REFRESH_IMAGES_FOLDER"
mkdir "$REFRESH_ROOT_FOLDER$REFRESH_IMAGES_FOLDER" || exitWithError "unable to re-create $REFRESH_IMAGES_FOLDER"

rm -r "$REFRESH_ROOT_FOLDER$REFRESH_MANUALS_FOLDER" || exitWithError "unable to delete the contents of $REFRESH_MANUALS_FOLDER"
mkdir "$REFRESH_ROOT_FOLDER$REFRESH_MANUALS_FOLDER" || exitWithError "unable to re-create $REFRESH_MANUALS_FOLDER"


# copy over all the new videos and images
echo "copying new media..."
cp -r "$TEMP_FOLDER_NAME/Videos/." "$REFRESH_ROOT_FOLDER$REFRESH_VIDEOS_FOLDER" || exitWithError "unable to copy new videos to refresh directory"
cp -r "$TEMP_FOLDER_NAME/Images/." "$REFRESH_ROOT_FOLDER$REFRESH_IMAGES_FOLDER" || exitWithError "unable to copy new images to refresh directory"

# not fatal if restoring older version
cp -r "$TEMP_FOLDER_NAME/Manuals/." "$REFRESH_ROOT_FOLDER$REFRESH_MANUALS_FOLDER"


# for each database table csv file
echo "import database tables..."
for filePath in "$REFRESH_ROOT_FOLDER/$TEMP_FOLDER_NAME"/*.csv; do
  fileName=${filePath##*\/}       # strip out the filename 
  tableName=${fileName%*.csv}     # strip out the tablename    

  # delete the existing database contents and update from csv file

  if [[ "$tableName" != "manufacturerSetup" ]]; then # Skip manufacturerSetup since that's unique to a machine.
      echo "fileName: $fileName"  
      python $SCRIPTS_FOLDER/configurationRestoreHelper.pyc "$DATABASE_FILE" $filePath $tableName || exitWithError "couldn't import $tableName from file"
  fi

  #sqlite3 "$DATABASE_FILE" "DELETE FROM $tableName;" || exitWithError "unable to delete from table $tableName"
  # import the new contents from file
  #sqlite3 -header -csv "$DATABASE_FILE" ".import $filePath $tableName" || exitWithError "couldn't import $tableName from file"
  
done
  
# delete the temp configuration file
echo "cleaning up..."
rm -r "$REFRESH_ROOT_FOLDER/$TEMP_FOLDER_NAME" || exitWithError "unable to delete the directory $REFRESH_ROOT_FOLDER$TEMP_FOLDER_NAME during cleanup"


echo "Configuration restore completed successfully restarting"
displayCountdown 3
if [ "$RESTART" == "$TRUE" ]
then
   nohup corco restart > /tmp/restart.log &
   sleep 2
fi
exit $NORMAL
