#!/bin/bash
#
# Backs up current Refresh diagnostics
#
#
NORMAL=0
ERROR=1

exitWithError() {
  echo "!!!!!!!!!!!!!!!!!!!!!!!!!!"
  echo "!!! Exiting with error !!!"
  echo "!!!!!!!!!!!!!!!!!!!!!!!!!!"
  echo "Info: $1"
  
  #show an exit countdown
  displayCountdown 10
  
  exit $ERROR
}

displayCountdown() {
  max=$1
  for ((i=$max;i>0;i-=1)); do
    echo -en "\rexiting in $i  "
    sleep 1
  done
  echo -e "\rdone           "
}

REFRESH_ROOT_FOLDER="/opt/Refresh/"
TEMP_FOLDER_NAME="refresh-diagnostics"
DATABASE_LOCATION="/Database/"
SCRIPTS_FOLDER="$REFRESH_ROOT_FOLDER/Versions/Refresh-current/ServiceUI/Scripts"
LOG_LOCATION="/var/log/"


# check that the destination location was passes in as the first arg
[ $# -gt 0 ] || exitWithError "target location was not specified"
targetLocation="$1" || exitWithError "unable to set the target location from provided input parameters"
echo "targetLocation = $targetLocation"

# go to the starting directory
cd "$REFRESH_ROOT_FOLDER" || exitWithError "Refresh root folder doesn't exist on this machine"

# create the temp? configuration folder
echo "creating temporary directory"
mkdir -p "$TEMP_FOLDER_NAME" || exitWithError "Unable to create temporary folder to hold configuration data"


# copy database into the configuration folder
echo "copying database files. please wait..."
cp -r ".$DATABASE_LOCATION." "$TEMP_FOLDER_NAME/Database" || exitWithError "Unable to copy database into configuration folder"

# copy log into the configuration folder
echo "copying database files. please wait..."
cp -r "$LOG_LOCATION" "$TEMP_FOLDER_NAME/log" || exitWithError "Unable to copy database into configuration folder"



# create the tarball
echo "creating configuration package. please wait..."
DATE=`date +%m%d%Y_%H%M%S`
TAR_CONFIG_FILENAME=$TEMP_FOLDER_NAME"_"$DATE".tar.gz"
echo $TEMP_FOLDER_NAME
echo $TAR_CONFIG_FILENAME
tar -zcvf "$TAR_CONFIG_FILENAME" "$TEMP_FOLDER_NAME" || exitWithError "Unable to compress the configuration folder"
rm -r $TEMP_FOLDER_NAME || exitWithError "Unable to delete temporary configuration folder"
  
# copy tar to the media device
echo "copying configuration package to target. please wait..."
#cp "$TAR_CONFIG_FILENAME" "/media/$(logname)/$targetLocation/" || exitWithError "Unable to copy configuration file to target destination"
cp "$TAR_CONFIG_FILENAME" "/media/cornelius/$targetLocation/" || exitWithError "Unable to copy configuration file to target destination"

# delete the local tar file
rm "$TAR_CONFIG_FILENAME" 

echo "Configuration backup completed successfully"
displayCountdown 5
exit $NORMAL

