#!/usr/bin/python
import optparse
import dbus
import dbus.service
import sys
from wicd import misc
from wicd.translations import _
import time
import netifaces

if getattr(dbus, 'version', (0, 0, 0)) < (0, 80, 0):
   import dbus.glib
else:
   from dbus.mainloop.glib import DBusGMainLoop
   DBusGMainLoop(set_as_default=True)
   

#DMSNetwork = "REFRESH"
#WEPKey = "C0rneliusdms"

def main():
   if len(sys.argv) != 3:
      print "Usage: hidden SSID KEY"
      sys.exit(1)

   DMSNetwork = sys.argv[1]
   WEPKey = sys.argv[2]

   bus = dbus.SystemBus()
   try:
       daemon = dbus.Interface(
           bus.get_object('org.wicd.daemon', '/org/wicd/daemon'),
           'org.wicd.daemon'
       )
       wireless = dbus.Interface(
           bus.get_object('org.wicd.daemon', '/org/wicd/daemon/wireless'),
           'org.wicd.daemon.wireless'
       )
       wired = dbus.Interface(
           bus.get_object('org.wicd.daemon', '/org/wicd/daemon/wired'),
           'org.wicd.daemon.wired'
       )
       config = dbus.Interface(
           bus.get_object('org.wicd.daemon', '/org/wicd/daemon/config'),
           'org.wicd.daemon.config'
       )
   except dbus.DBusException:
       print 'Error: Could not connect to the daemon. ' + \
           'Please make sure it is running.'
       sys.exit(3)

   if not daemon:
       print 'Error connecting to wicd via D-Bus. ' + \
           'Please make sure the wicd service is running.'
       sys.exit(3)

   #just so we dont crash if no wlan
   wiface = daemon.GetWirelessInterface()
   
   interfaces = netifaces.interfaces()
   for x in interfaces:
       if "wlan" in x:
           wiface = x
   print wiface
   count = 0
   while 1:
      daemon.Disconnect()
      [state, info] = daemon.GetConnectionStatus()
      wireless.EnableWirelessInterface()
      time.sleep(2)
      wireless.SetHiddenNetworkESSID(DMSNetwork)
   
      print "count", count
      wireless.Scan(True)
      networks = wireless.GetNumberOfNetworks()
      print "networks", networks
      connectTo = -1
      entries = []
      for i in range(0, networks):
         wireless.ReadWirelessNetworkProfile(i)
         essid = wireless.GetWirelessProperty(i, "essid")
         key = wireless.GetWirelessProperty(i, "key")
         quality =  wireless.GetWirelessProperty(i, "quality")
         strength = wireless.GetWirelessProperty(i, "strength")
         entries.append( [essid, key, quality])
         entries.sort(key=lambda x: -int(x[2]))


      for i in range(0, networks):   
         print entries[i][0], entries[i][1], entries[i][2]
         if (entries[i][0] == DMSNetwork):
             wireless.SetWirelessProperty(i, 'key', WEPKey)
             wireless.SaveWirelessNetworkProperty(i, "key")
             wireless.SetWirelessProperty(i, 'automatic', 1)
             wireless.SaveWirelessNetworkProperty(i, "automatic")
             connectTo = i
      count += 1
      if (connectTo != -1 or  count > 3):
         break
      time.sleep(3)
   if (connectTo != -1):
       wireless.ConnectWireless(connectTo)

       for i in range(0, 60):
            ip = ''
            try:
               ip=netifaces.ifaddresses(wiface)[2][0]['addr']
               print 'success'
               sys.exit(0)
            except:
               if ip != '':
                  sys.exit(0)
            time.sleep(1)
   print 'failure'
   sys.exit(1)

if __name__ == '__main__':
   main()
