#!/bin/bash
#
# Restores Refresh IoT configuration from file
#


NORMAL=0
ERROR=1

exitWithError() {
  #echo "!!!!!!!!!!!!!!!!!!!!!!!!!!"
  #echo "!!! Exiting with error !!!"
  #echo "!!!!!!!!!!!!!!!!!!!!!!!!!!"
  echo "error Info: $1"
  sleep 1
  echo "ERROR CASE"
  #show an exit countdown
  displayCountdown 10
  exit $ERROR
}

displayCountdown() {
  max=$1
  for ((i=$max;i>0;i-=1)); do
    #echo -en "\rexiting in $i"
    sleep 1
  done
  echo -e "\rdone           "
}


REFRESH_ROOT_FOLDER="/opt/Refresh"
REFRESH_IOT_FOLDER="/IoT"
SCRIPTS_FOLDER="$REFRESH_ROOT_FOLDER/Versions/Refresh-current/ServiceUI/Scripts"

# check that a configuration file is all good in the hood
echo "checking input parameters..."
[ $# -gt 0 ] || exitWithError "configuration file was not specified"
fullConfigurationFilePath="$1" || exitWithError "unable to set the configuration file from provided input parameters"
configurationFilePath=${fullConfigurationFilePath%\/*}\/
configurationFileName=${fullConfigurationFilePath##*\/}
#echo "config file path: $configurationFilePath"
#echo "config file name: $configurationFileName"
[ -d "$configurationFilePath" ] || exitWithError "provided configuration file path ($configurationFilePath) was not found"
cd $configurationFilePath || exitWithError "unable to change to directory ($configurationFilePath)"
[ -f "$configurationFileName" ] || exitWithError "provided configuration file name ($configurationFileName) was not found"
cd $REFRESH_ROOT_FOLDER || exitWithError "unable to change to directory ($REFRESH_ROOT_FOLDER) after configuration file check"
# copy the configuration file onto the system
echo "copying IoT configuration file to system..."
cp $configurationFilePath$configurationFileName $REFRESH_ROOT_FOLDER || exitWithError "unable to copy configuration file to machine"

# untar the configuration file
echo "unpacking IoT configuration file..."
mkdir $REFRESH_ROOT_FOLDER/IoT 
tar -xvf "$configurationFileName" -C $REFRESH_ROOT_FOLDER/IoT || exitWithError "unable to unpack configuration file"

# SHOULD WE DO THIS
# delete the configuration file
echo "removing configuration file..."
rm "$configurationFileName" || exitWithError "unable to remove tar configuration file"

echo "IoT Configuration completed successfully"
displayCountdown 5
sleep 2
echo "Restarting IoT"
sudo corco IoT stop
# Let the watchdog restart it to prevent any race condition
# that could end up with two instances running at the same time.
# sudo corco IoT start
exit $NORMAL
