#!/usr/bin/python

import dbus
import dbus.service
from dbus.mainloop.glib import DBusGMainLoop
import threading
import sys, time
from gi.repository import GObject
from logger import Logger

_MY_DBUS_SERVICE_NAME =   "org.refresh.videoPlayer"
_MY_DBUS_INTERFACE_NAME = "org.refresh.videoPlayer"
_MY_DBUS_LOC = "/"

GObject.threads_init()
DBusGMainLoop(set_as_default=True)

MetaGObject= type (GObject.GObject)
MetaDbusObject= type (dbus.service.Object)
MetaThreadObject = type (threading.Thread)


#############################################################
class MetaDBusWorker(MetaGObject, MetaDbusObject, MetaThreadObject):
  def __init__(self, name, bases, members):
    
    MetaGObject.__init__ (self, name, bases, members)
    MetaDbusObject.__init__ (self, name, bases, members)
    MetaThreadObject.__init__ (self, name, bases, members)

##############################################################    
class DBusWorker(GObject.GObject, dbus.service.Object, threading.Thread):
  
  __metaclass__= MetaDBusWorker
  _threads = []
  
  ######################################
  def __init__(self):
    #Logger.debug("DBusWorker",  "Init DBusWorker Object")
    busName = dbus.service.BusName(_MY_DBUS_SERVICE_NAME, bus=dbus.SessionBus())
    dbus.service.Object.__init__(self, busName, _MY_DBUS_LOC)

    bus = dbus.SessionBus()
    bus.add_signal_receiver(self.showServiceUISignalReceived,
                            dbus_interface = "org.refresh.mainUI",
                            signal_name = "showServiceUI")

    bus.add_signal_receiver(self.lockDBSignalReceived,
                            dbus_interface = "org.refresh.mainUI",
                            signal_name = "lockDB")

    bus.add_signal_receiver(self.unlockDBSignalReceived,
                            dbus_interface = "org.refresh.mainUI",
                            signal_name = "unlockDB")
    
    # This is triggered by auto-updates
    bus.add_signal_receiver(self.showServiceUISignalReceived,
                            dbus_interface = "org.refresh.serviceUI",
                            signal_name = "showServiceUI")

    bus.add_signal_receiver(self.showMainUISignalReceived,
                            dbus_interface = "org.refresh.serviceUI",
                            signal_name = "showMainUI")

    bus.add_signal_receiver(self.initVideoPlayerSignalReceived,
                            dbus_interface = "org.refresh.serviceUI",
                            signal_name = "initVideoPlayer")
    
    bus.add_signal_receiver(self.setPlaylistReceived,
                            dbus_interface = "org.refresh.serviceUI",
                            signal_name = "setPlaylist")


    bus.add_signal_receiver(self.initVideoPlayerSignalReceived,
                            dbus_interface = "org.refresh.iot",
                            signal_name = "initVideoPlayer")

    bus.add_signal_receiver(self.setPlaylistReceived,
                            dbus_interface = "org.refresh.iot",
                            #dbus_interface = None,
                            signal_name = "setPlaylist")

    bus.add_signal_receiver(self.setMRSSReceived,
                            dbus_interface = "org.refresh.iot",
                            signal_name = "setMRSS")
                                   
    bus.add_signal_receiver(self.setSnapshotReceived,
                            dbus_interface = "org.refresh.serviceUI",
                            signal_name = "snapshot")

    GObject.GObject.__init__(self)
    threading.Thread.__init__(self)


  #####################################
  def lockDBSignalReceived(self):
    self.emitSignal("signal_dbus_lock_db");
    

  #####################################
  def unlockDBSignalReceived(self):
    self.emitSignal("signal_dbus_unlock_db");
    
    
  #####################################
  def showServiceUISignalReceived(self):
    Logger.debug("DBusWorker",  "dbus menuClick signal received")
    self.emitSignal("signal_dbus_hide_ui");
    
    
  #####################################
  def showMainUISignalReceived(self):
    Logger.debug("DBusWorker",  "dbus exitMenu signal received")
    self.emitSignal("signal_dbus_show_ui");
    
    
  #####################################
  def initVideoPlayerSignalReceived(self):
    Logger.debug("DBusWorker",  "dbus initVideoPlayer signal received")
    self.emitSignal("signal_dbus_init_video_player");


  #####################################
  def setPlaylistReceived(self,  url):
    Logger.debug("DBusWorker",  "dbus setPlaylist signal received")
    self.emitSignal("signal_dbus_set_playlist", url);

  #####################################
  def setMRSSReceived(self,  url):
    Logger.debug("DBusWorker",  "dbus setMRSS signal received")
    self.emitSignal("signal_dbus_set_mrss", url);
       
       
  ######################################
  def sendShowServiceUISignal(self, widget):    
    self.emitSignal("signal_dbus_hide_ui")
    self.showServiceUI()     
       

  #####################################
  # SIGNAL EXPOSED TO DBUS
  @dbus.service.signal(_MY_DBUS_INTERFACE_NAME)
  def showServiceUI(self):
    Logger.debug("DBusWorker",  "sending showServiceUI signal")
    pass


  ######################################
  # METHOD EXPOSED TO DBUS  
  @dbus.service.method(_MY_DBUS_INTERFACE_NAME)
  def areYouAlive(self):
   #for thread in self._threads:
   #  if thread.isAlive() != "yes":
   #   return "no"
   return "yes"
  
  ######################################
  # METHOD EXPOSED TO DBUS  
  @dbus.service.method(_MY_DBUS_INTERFACE_NAME)
  def quitApp(self):
   self.emitSignal("signal_dbus_exit_ui")


  ######################################
  # METHOD EXPOSED TO DBUS  
  @dbus.service.method(_MY_DBUS_INTERFACE_NAME)
  def setPlaylist(self, url):
    self.emitSignal("signal_dbus_set_playlist", url)
 
  #####################################    
  def emitSignal(self, *args):
    GObject.idle_add(GObject.GObject.emit, self,*args)


  #####################################    
  def setThreads( self, threads ):
    self._threads = threads;


  #####################################    
  def isAlive(self):
    return "yes"

  def terminate(self):
    self._running = False;

  def setSnapshotReceived(self, data):
    Logger.debug("DBusWorker",  "dbus setSnapshotReceived signal received: " + data )
    self.emitSignal("signal_dbus_snapshot", data);
