#!/usr/bin/python

import DatabaseHelper
import time
import datetime
import os.path
from logger import Logger

class PlayListController:
  
  _DATABASE = ""
  _PLAYLIST_ID = -1
  _PLAYLIST_NAME = "None"
  _PLAYLIST_INDEX = 0
  _PLAYLIST_ARRAY = []
  _PLAYLIST_FILE_PATH = ""
  _PLAYLIST_CURRENT_TIME = ""
  _DIRECTORY_PATH = "file:///opt/Refresh/Videos/Shared/"
  _CMS_DIRECTORY_PATH = "file:///opt/Refresh/Videos/CMS/"
  _EMPTY_PLAYLIST_VIDEO_PATH = "file:///opt/Refresh/Videos/MainUI/Default.mp4"
  _WALL_MODE_ENABLED = False
  _playlist_number = 0
 
 
  def __init__(self, db):
    self._DATABASE = db


  def reset(self):
    Logger.debug("PlaylistController", "Reset")
    self._PLAYLIST_ID = -1
    self._PLAYLIST_NAME = "None"
    self._PLAYLIST_INDEX = 0
    self._PLAYLIST_ARRAY = []
    self._PLAYLIST_FILE_PATH = ""
    self._PLAYLIST_CURRENT_TIME = ""

  
  def getVideoPath(self):
    path = ""
    if self._WALL_MODE_ENABLED == False:
      path = self.getVideoPathFromPlayList()
    else:
      path = self.getVideoPathFromDictionary()

    # When absolutely no videos are available, just fall back to our hard coded
    # default. It beats a white screen.
    if len(path) == 0:
        path = self._EMPTY_PLAYLIST_VIDEO_PATH

    return path


  def getVideoPathFromPlayList(self):
    self._PLAYLIST_CURRENT_TIME = time.strftime("%H:%M")
    results = self._DATABASE.getPlayListFromCurrentTime( self._PLAYLIST_CURRENT_TIME )

    # Get Default Playlist when not found from current time
    if results == None:
      results = self._DATABASE.getPlayListDefault()
    
    if results is not None and len(results) > 1:
        # Build new playlist if changed
        if self._PLAYLIST_ID != results[0]:
          self.buildNewList(results[0], results[1])
        
        # If list is empty for none 'default' playlist
        # failover to default playlist again
        if results[1] != 'default' and len(self._PLAYLIST_ARRAY) == 0:
          results = self._DATABASE.getPlayListDefault()
    else:
        Logger.debug("PlaylistController", "CAUGHT PROBLEM")

      
    # When nothing found get default video path
    if results == None or len(self._PLAYLIST_ARRAY) == 0:
      self._PLAYLIST_FILE_PATH = ""
      return self._PLAYLIST_FILE_PATH
    else:
      valid = False
      current_index = self._PLAYLIST_INDEX
      while (valid == False):
        self.constructVideoPathFromPlayList() 
        self.increasePlaylistIndex()

        # test if file is valid
        rows  = self._DATABASE.getPlaylistFileValidTimes(self._PLAYLIST_FILE_PATH[self._PLAYLIST_FILE_PATH.find("://")+3:])
        
        if (rows == []):
          valid = True

        for r in rows:
          st = int(float(r[0]))
          et = int(float(r[1]))

          now = datetime.datetime.now()
          midnight = datetime.datetime.combine(now.date(), datetime.time())
          seconds_since_midnight = (now - midnight).seconds

          if st == 946684800:
            valid = True
          elif st < 86400:
            #get seconds into day if valid time does not include a data
            if seconds_since_midnight >= st and seconds_since_midnight < et:
              valid = True
          else:
            current_unix_time = time.time()

            # first check total range
            if current_unix_time >= st and current_unix_time <= et:
              # now check tune within day
              st_p = time.gmtime(st)
              #print "st", st, st_p
              et_p = time.gmtime(et)
              #print "et", et, et_p
              st = st_p.tm_hour * 3600 + st_p.tm_min * 60 + st_p.tm_sec
              et = et_p.tm_hour * 3600 + et_p.tm_min * 60 + et_p.tm_sec
              xx = time.gmtime(st)
              #print "st %", xx, st
              xx = time.gmtime(et)
              #print "et %", xx, et

              lt = time.localtime(time.time())
              #if (lt.tm_isdst):
              #    current_unix_time += 3600
              current_unix_time = current_unix_time % 86400
              xx = time.gmtime(current_unix_time)
              #print "Unix %",xx, 


              if current_unix_time >= st and current_unix_time < et:
                valid = True
              elif (et < st) :
                if (current_unix_time < et): # midnight to stop
                   valid = True
                elif (current_unix_time > st):
                   valid = True

        if (valid == False):
          Logger.debug("PlaylistController",  self._PLAYLIST_FILE_PATH + " not valid time" )
        if valid == False and self._PLAYLIST_INDEX == current_index:
          Logger.debug("PlaylistController", "No valid files")
          self._PLAYLIST_FILE_PATH = ""
          return self._PLAYLIST_FILE_PATH
    
    self.debug()

    return self._PLAYLIST_FILE_PATH
  
  
  
  def constructVideoPathFromPlayList(self):
    self._PLAYLIST_FILE_PATH = self._DIRECTORY_PATH + self._PLAYLIST_ARRAY[ self._PLAYLIST_INDEX ]

    # if file does not exist, it might be a CMS file.  If it still doesnt exist
    # the player will remove it on the error (which is why no test here)
    if os.path.exists(self._PLAYLIST_FILE_PATH[7:]) == False:
      self._PLAYLIST_FILE_PATH = self._CMS_DIRECTORY_PATH + self._PLAYLIST_ARRAY[ self._PLAYLIST_INDEX ]

    
    Logger.debug("PlaylistController", "Playlist file path " + self._PLAYLIST_FILE_PATH)
    Logger.debug("PlaylistController",  "Playlist file path " + str(self._PLAYLIST_INDEX))
    self._playlist_number = self._PLAYLIST_INDEX
    Logger.debug("PlaylistController",  "playlist_number " + str(self._playlist_number))
  
  
  def increasePlaylistIndex(self):
    if (len(self._PLAYLIST_ARRAY) - 1) == self._PLAYLIST_INDEX:
      self._PLAYLIST_INDEX = 0
      Logger.debug("PlaylistController",  "Playlist length Index " + str(len(self._PLAYLIST_ARRAY) - 1))
      Logger.debug("PlaylistController",  "Playlist Index " + str(self._PLAYLIST_INDEX))
    else:
      self._PLAYLIST_INDEX += 1 
      Logger.debug("PlaylistController",  "Playlist Index incremented " + str(self._PLAYLIST_INDEX))
  
  
  def getVideoPathFromDictionary(self):
   
    # this is a hack for now until we do real cms work
    if "/tmp" in self._DATABASE.dictionaryGet("videoPath"):
      path = "file://" + self._DATABASE.dictionaryGet("videoPath")
    else:
      path = self._DIRECTORY_PATH + self._DATABASE.dictionaryGet("videoPath")
    Logger.debug("PlaylistController",  "Wall mode video path: " + path)
    return path
  
  def setWallEnabled(self, enabled):
    self._WALL_MODE_ENABLED = enabled
  
  def buildNewList(self, playlistID, playlistName):
    Logger.debug("PlaylistController",  "CONTROLLER PLAYLIST:       building new playlist " + str(playlistID))
    self._PLAYLIST_ID = playlistID;
    self._PLAYLIST_NAME = playlistName
    self._PLAYLIST_INDEX = 0
    self._PLAYLIST_ARRAY = self._DATABASE.getPlayListFilesFromId(playlistID)
    self.debug();
  
  
  def debug(self):
    Logger.debug("PlaylistController",  "CONTROLLER PLAYLIST ID:    " + str(self._PLAYLIST_ID))
    Logger.debug("PlaylistController",  "CONTROLLER PLAYLIST NAME:  " + self._PLAYLIST_NAME)
    Logger.debug("PlaylistController",  "CONTROLLER PLAYLIST INDEX: " + str(self._PLAYLIST_INDEX))
    Logger.debug("PlaylistController",  "CONTROLLER PLAYLIST ARRAY: " + str(self._PLAYLIST_ARRAY))
    Logger.debug("PlaylistController",  "CONTROLLER PLAYLIST FILE:  " + self._PLAYLIST_FILE_PATH)
    Logger.debug("PlaylistController",  "CONTROLLER PLAYLIST TIME:  " + self._PLAYLIST_CURRENT_TIME)
    Logger.debug("PlaylistController",  "CONTROLLER PLAYLIST Number: " + str(self._playlist_number))


  def deleteCorruptFile(self):
     #Logger.debug("PlaylistController",  "Corrupt playlist ID is" + str(self._PLAYLIST_ID)
     arraySize = len(self._PLAYLIST_ARRAY)
     result = self._DATABASE.deletePlaylistRecord(str(self._PLAYLIST_ID),str(self._playlist_number),self._PLAYLIST_ARRAY)
     self._PLAYLIST_FILE_PATH = ""
     self.buildNewList(str(self._PLAYLIST_ID),self._PLAYLIST_NAME)

  def deleteMissingFile(self):
     self.deleteCorruptFile()

    
if __name__ == "__main__":
  db = DatabaseHelper.DatabaseConnection("/opt/Refresh/Database/refresh_database_enc.db")
  controller = PlayListController(db)
  controller.getVideoPath();
  time.sleep(1)
  controller.getVideoPath();
  
  Logger.debug("PlaylistController",  "Finished")
  
