#!/usr/bin/python

import threading
import sys, time
from gi.repository import GObject
from socket import *
from netifaces import interfaces, ifaddresses, AF_INET
from time import sleep
import os.path
from logger import Logger

GObject.threads_init()

class UDPWorker(GObject.GObject, threading.Thread):
  
  SHOULD_RUN = True
  VIDEO_PATH = "";
  IP_ADDRESS = "unknown"
  last_data = ""
  basetime = 0
  masterIP = ""
  
  def __init__(self):
    GObject.GObject.__init__(self)
    threading.Thread.__init__(self)
  
  def setIPAddress( self ):
    for interface in interfaces():
      for key in ifaddresses(interface):
	if key == AF_INET:
	  for link in ifaddresses(interface)[AF_INET]:
	    if link['addr'] != "127.0.0.1":
              self.IP_ADDRESS = link['addr']
	      return
    return
  
  def getIPAddress(self):
    try:
       self.setIPAddress()
    except:
       self.IP_ADDRESS = "127.0.0.1" 
    return self.IP_ADDRESS
    
  def emitSignal(self, *args):
    GObject.idle_add(GObject.GObject.emit, self,*args)
    
  def sendBroadcastMessage( self, data ):
    try:
       Logger.debug("UDPWorker",  "Begin sending")
       lines = data.split("\n")
       for line in lines:
         Logger.debug("UDPWorker",  ">> " + line)
       send_socket = socket(AF_INET, SOCK_DGRAM)
       send_socket.setsockopt(SOL_SOCKET, SO_BROADCAST, 1)
       send_socket.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
       for i in range(0,5):
          send_socket.sendto(data, ('<broadcast>', 8000))
       Logger.debug("UDPWorker", "End")
    except:
       Logger.debug("UDPWorker","Send Failed")

  def respondBroadcastIpAddress( self ):
   data = 'wall-my-ipaddress'
   data += ';'
   data += self.IP_ADDRESS
   self.sendBroadcastMessage( data )

  def broadcastMasterIpAddress( self ):
    data = 'wall-master-ipaddress'
    data += ';'
    data += self.IP_ADDRESS
    self.sendBroadcastMessage( data )

  def sendBaseTime(self, base_time):
     data = 'wall-basetime'
     data += ';'
     data += str(base_time)
     self.sendBroadcastMessage( data )


  def getDataOption(self, data ):
    lines = data.split("\n")
    for line in lines:
      chunks = line.split(";")
      if len(chunks) == 2:
	ip = chunks[0]
        if self.IP_ADDRESS == ip:
	  return chunks[1]
    return None
  

  def setMasterIP(self, data):
      lines = data.split("\n")
      for line in lines:
        chunks = line.split(";")
        self.masterIP = chunks[0]
      Logger.debug("UDPWorker", "Master IP " + self.masterIP)

  def setVideoPath( self, data ):
    path = self.getDataOption( data )
    if path != None:
      self.emitSignal("signal_broadcast_set_video_path", path)
    else:
      Logger.debug("UDPWorkder",  "Error: video path is None")
  
  def setMode( self, data ):
    mode = self.getDataOption( data )
    if mode != None:
      self.emitSignal("signal_broadcast_set_mode", mode );
    else:
      Logger.debug("UDPWorker", "Error: mode is None")

  def setBaseTime( self, data ):
      lines = data.split("\n")
      for line in lines:
        chunks = line.split(";")
        self.basetime = int(chunks[1])

  def run(self):
    rec_socket = socket(AF_INET, SOCK_DGRAM)
    rec_socket.setsockopt(SOL_SOCKET, SO_BROADCAST, 1)
    rec_socket.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
    rec_socket.bind(('<broadcast>', 8000))
    rec_socket.setblocking(False)
    
    self.setIPAddress()
    Logger.debug("UDPWorker", "Found IP: " + self.IP_ADDRESS)
    
    while self.SHOULD_RUN:
      try:
        data = rec_socket.recv(1024)
	
      except timeout, e:
	sleep(0.1)
	
      except error, e:
        sleep(0.1)
      
      else:
	self.receiveMessage( data );

  def receiveMessage( self, data ):
    Logger.debug("UDPWorker", "Begin receiving")
    if (data == self.last_data):
        return
    self.last_data = data

    lines = data.split("\n")
    for line in lines:
       Logger.debug("UDPWorker", "<< " + line)
    Logger.debug("UDPWorker", "End")

    if "wall-broadcast-ipaddress" in data:
      self.respondBroadcastIpAddress()
      self.emitSignal("signal_broadcast_show_ip_address")


    if "wall-master-ipaddress" in data:
       self.setMasterIP(data)

    if "wall-start-video" in data:
      self.emitSignal("signal_broadcast_start")

    if "wall-stop-video" in data:
      self.emitSignal("signal_broadcast_stop")
      
    if "wall-set-video-path" in data:
      Logger.debug("UDPWorker",  "set video path")
      self.setVideoPath( data )
      
    if "wall-mode" in data:
      self.setMode( data )
      
    if "wall-initialize" in data:
      self.emitSignal("signal_broadcast_initialize_video_player")

    if "wall-basetime" in data:
       self.setBaseTime (data)

  def terminate(self):
    self.SHOULD_RUN = False
    self._running = False;

  def broadcastStartVideo(self):
    data = 'wall-start-video'
    self.sendBroadcastMessage( data )

  def __del__(self):
    Logger.debug("UDPWorker",  "Deleting UDPWorker Object")

  def isAlive(self):
    Logger.debug("UDPWorker", "UDPWorker alive")
    return "yes"
