#!/usr/bin/python

import DatabaseHelper
from PlayListController import PlayListController
import threading
import thread
import os.path
import os
import gi
import time
import datetime
import sys
import re
import traceback
import gc
gi.require_version("Gst", "1.0")
from gi.repository import Gst       #GStreamer1.0
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk       #GUI
from gi.repository import Gdk
from gi.repository import GdkX11    # get the xid of the drawing area
gi.require_version ("GstVideo", "1.0")
from gi.repository import GstVideo  #to play the vid in the drawing area	
from gi.repository import GObject   # to sync clocks
gi.require_version("GstNet", "1.0")
from gi.repository import GstNet
from UDPWorker import UDPWorker
from DBusWorker import DBusWorker
from Overlay import Overlay
from Button import Button
import feedparser
from urllib2 import urlparse
from downloader import Downloader
import iso8601
import Queue
from logger import Logger
import signal

video_path = "/opt/Refresh/Videos/CMS/"
tmp_video_path = "/opt/Refresh/Videos/CMS/tmp/"

_MODE_OFF = "off"
_MODE_SLAVE = "slave"
_MODE_MASTER = "master"


# INITIALIZE GST
Gst.init(None)

# SET UP DATABASE CONNECTION
refreshDatabase = DatabaseHelper.DatabaseConnection("/opt/Refresh/Database/refresh_database_enc.db")
windowHeight = int(refreshDatabase.dictionaryGet("videoHeight"))
windowWidth = int(refreshDatabase.dictionaryGet("videoWidth"))
windowOffsetHeight = int(refreshDatabase.dictionaryGet("videoXoffset"))
windowOffsetWidth = int(refreshDatabase.dictionaryGet("videoYoffset"))  # 55 
my_threads = []
PLController = PlayListController( refreshDatabase )

######################################
class Main(GObject.GObject):

  _WALL_MODE = _MODE_OFF
  
  LOOP_COUNTER = 0
  FILE_COUNTER = 0
  KILL_COUNT = 5000
  defaultFile = "default"

  ###################################
  def __init__(self):

    GObject.GObject.__init__(self)

    # Logger.debug("defVideoPlayer", "__init__");

    self.dBusWorker = DBusWorker()
    self.dBusWorker.connect('signal_dbus_show_ui', self.onSignalDbusShowUI)
    self.dBusWorker.connect('signal_dbus_hide_ui', self.onSignalDbusHideUI)
    self.dBusWorker.connect('signal_dbus_exit_ui', self.onSignalDbusExitUI)
    self.dBusWorker.connect('signal_dbus_lock_db', self.onSignalDbusLockDB)
    self.dBusWorker.connect('signal_dbus_lock_db', self.onSignalDbusUnLockDB)
    self.dBusWorker.connect('signal_dbus_init_video_player', self.onSignalDbusInitVideoPlayer)
    self.dBusWorker.connect('signal_dbus_set_playlist', self.onSignalSetPlaylist)
    self.dBusWorker.connect('signal_dbus_set_mrss', self.onSignalSetMRSS)
    self.dBusWorker.start()
  
    self.uDPWorker = UDPWorker()
    self.uDPWorker.connect('signal_broadcast_set_video_path', self.onSignalBroadcastSetPath)
    self.uDPWorker.connect('signal_broadcast_set_mode', self.onSignalBroadcastSetMode)
    self.uDPWorker.connect('signal_broadcast_start', self.onSignalBroadcastStart)
    self.uDPWorker.connect('signal_broadcast_stop', self.onSignalBroadcastStop)
    self.uDPWorker.connect('signal_broadcast_initialize_video_player', self.onSignalBroadcastInitVideoPlayer)
    self.uDPWorker.connect('signal_broadcast_show_ip_address', self.onSignalBroadcastShowIpAddress)
    self.uDPWorker.start()

    self.queue = Queue.Queue()
    self.Downloader = Downloader(self.queue)
    self.Downloader.connect('signal_download_started', self.onVideoDownloadStarted)
    self.Downloader.connect('signal_download_complete', self.onVideoDownloadComplete)
    self.Downloader.connect('signal_download_file_complete', self.onDownloadFileComplete)
    self.Downloader.start()
    
    #my_threads.append( self.dBusWorker )
    #my_threads.append( self.uDPWorker )
    #my_threads.append( self.Downloader )
    #my_threads.append( self )
    
    self.dBusWorker.setThreads( my_threads )

    # CREATE THE WINDOW
    self.window = Gtk.Window()
    self.window.set_decorated(False)
    self.window.move(windowOffsetWidth,windowOffsetHeight)
    self.window.set_border_width(0)
    self.window.connect("delete-event", Gtk.main_quit)

    # CREATE THE VIDEO AREA
    self.drawingArea = Gtk.DrawingArea()
    self.drawingArea.set_size_request(windowWidth, windowHeight)
    self.drawingArea.add_events(Gdk.EventMask.BUTTON_PRESS_MASK)
    self.drawingArea.connect("button-press-event", self.onButtonPress)

    # LABEL TO SHOW IP ADDRESS
    self.label = Gtk.Label()
    self.label.set_valign(Gtk.Align.CENTER)
    self.label.set_halign(Gtk.Align.CENTER)
    self.label.set_use_markup(True)

    # LABEL TO SHOW Downloading Message
    self.dl_label = Gtk.Label()
    self.dl_label.set_valign(Gtk.Align.CENTER)
    self.dl_label.set_halign(Gtk.Align.CENTER)
    self.dl_label.set_use_markup(True)
    
    # BOX ALL ITEMS IN THE WINDOW

    self.vbox = Gtk.Fixed()
    self.vbox.put(self.drawingArea, 0, 0)


    self.corners = Overlay();
    #self.corners.createCornerButtons(windowWidth, windowHeight)
    #self.vbox.put(self.corners, 0 , 0)    
    self.corners.connect('signal-show-service-ui', self.dBusWorker.sendShowServiceUISignal)    

    # CREATE OVERLAY VIDEO AND LABEL
    self.overlay = Gtk.Overlay()
    self.overlay.add(self.vbox)
    self.overlay.add_overlay(self.label)
    self.overlay.add_overlay(self.dl_label)
    self.window.add(self.overlay)
    

    
    # BUILD AN EMPTY GSTREAMER PIPE AS A PLACE HOLDER
    # A REAL ONE WILL BE CREATED IN initVideoPlayer()
    self.gstreamerPipeBuild("")   

    # IMPORT GTK CSS STYLES 
    cssProvider = Gtk.CssProvider()
    cssProvider.load_from_path('GtkStyles.css')
    screen = Gdk.Screen.get_default()
    styleContext = Gtk.StyleContext()
    styleContext.add_provider_for_screen(screen, cssProvider, Gtk.STYLE_PROVIDER_PRIORITY_USER)
 
    # SHOW THE WINDOW
    self.window.set_keep_above(True)
    self.window.show_all()
    self.label.hide()
    self.dl_label.hide()

    # SET THE WINDOW CURSOR (THIS MUST BE DONE AFTER SHOW_ALL IS CALLED ON THE WINDOW)
    watch_cursor = Gdk.Cursor(Gdk.CursorType.BLANK_CURSOR)
    self.window.get_window().set_cursor(watch_cursor)

    
    # NOW THAT THE WINDOW HAS BEEN CREATED, WE SHOULD INITIALIZE THE VIDEO PLAYER   
    self.last_url = []
    self.noDelete = []
    self.ui_hidden = False
    self.initVideoPlayer()
    self.button_counter = 0


    # for testing
    #self.onSignalSetPlaylist(None, "http://www.channelvalet.com/portals/0/Media%20RSS/CAM_a694feee-0dd8-42f6-b89e-2aa27285a4cb.xml")

    #self.onSignalSetPlaylist(None,"http://www.channelvalet.com/portals/0/Media%20RSS/CAM_b0ba80d6-1597-43a6-a4b7-456c3763d417.xml")

  def onButtonPress(self, widget, event):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name );
    if (event.x >= (windowWidth - 150) and event.y < 150) :
      self.button_counter = 1
      self.vbox.hide()

    if (event.x < 100 and event.y < 100):
      self.vbox.hide()
      if self.button_counter == 1 or self.button_counter == 2:
        self.button_counter = 2
      else:
        self.button_counter = 0

    if (event.x >= 0 and event.x <= 150  and event.y >= (windowHeight - 150)) :
      self.vbox.hide()
      if self.button_counter == 2 or self.button_counter == 3:
        self.button_counter = 3
      else:
        self.button_counter = 0


    if (event.x >= (windowWidth - 150) and event.y >= (windowHeight - 150)) :
      self.vbox.hide()
      if self.button_counter == 3:
        Logger.info("VideoPlay", "Load ServiceUI")
        self.corners.emitSignal("signal-show-service-ui")
        self.button_counter = 0
      else:
        self.button_counter = 0
        
    self.vtime = time.time()
    GObject.timeout_add_seconds(.5, self.showVBox)

  def showVBox(self):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name );
    if (time.time() - self.vtime < 0.5) :
      GObject.timeout_add_seconds(.1, self.showVBox)
    else:
      self.vbox.show()
      self.window.move(windowOffsetWidth,windowOffsetHeight)

        

  ###################################################
  def gstreamerPipeDestroy(self):
    # Logger.debug("defVideoPlayer", "gstreamerPipeDestroy");
    if hasattr(self, 'bus'):

      if (self.bus != None):      
        self.gstreamerStop()

        self.bus.disconnect(self.onEOS_ID)
        self.bus.disconnect(self.onError_ID)
        self.bus.remove_signal_watch()
        
        self.bus.disconnect(self.onSync_ID)
        self.bus.disable_sync_message_emission()
        
        #del self.bus
        #del self.playbin

      self.bus = None
      self.playbin = None
      gc.collect()
    
    
  ##############################################
  def gstreamerPipeBuild(self, myFile):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name + " " + myFile);
   
    Logger.debug("VideoPlayer", "Building Pipeline" + myFile)

    
    # DESTROY PREVIOUS PIPELINE
    self.gstreamerPipeDestroy()
    
    self.playbin = Gst.ElementFactory.make('playbin', None)
    self.bus = self.playbin.get_bus()

    self.bus.set_flushing(True)
    self.bus.add_signal_watch()
    self.onEOS_ID = self.bus.connect('message::eos', self.OnEOS)
    self.onError_ID = self.bus.connect('message::error', self.OnError)
    
    self.bus.enable_sync_message_emission()
    self.onSync_ID = self.bus.connect('sync-message::element', self.OnSyncMessage)

    self.playbin.set_property('uri', myFile)
    self.uri = myFile
    

  ####################################################
  def gstreamerPlay(self):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);
    self.xid = self.drawingArea.get_property('window').get_xid()

	# edk: I'm not sure what the following is supposed to do, seems
	# like its redundant, but it screws up the frame rate of the 
	# video player on Viper.
    # make sure some other clock isn't autoselected
    #clock = self.playbin.get_clock()
    #self.playbin.use_clock(clock)
    if self._WALL_MODE == _MODE_MASTER:
     # this will start a server listening on a UDP port
     Logger.info("VideoPlayer", 'Wall Master')
     clock_provider = GstNet.NetTimeProvider.new(clock, None, 8001)

     # we explicitly manage our base time
     base_time = clock.get_time()

     # disable the pipeline's management of base_time -- we're going
     # to set it ourselves.
     self.playbin.set_start_time(Gst.CLOCK_TIME_NONE)
     self.playbin.set_base_time(base_time)
     self.uDPWorker.sendBaseTime(base_time)

    elif self._WALL_MODE == _MODE_SLAVE:
     Logger.info("VideoPlayer", "Wall slave")
     if (self.uDPWorker.masterIP == ""):
         Logger.info("VideoPlayer", "No master ip yet")
     else:
         Logger.info("VideoPlayer", "Master at " + self.uDPWorker.masterIP)

         clock = GstNet.NetClientClock.new('clock0', self.uDPWorker.masterIP, 8001, self.uDPWorker.basetime)

         # use it in the pipeline
         self.playbin.set_base_time(self.uDPWorker.basetime)
         self.playbin.use_clock(clock)


    self.playActive = True
    self.playbin.set_state(Gst.State.PLAYING)
    Logger.info("VideoPlayer", "Playing")

  ####################################################
  def gstreamerStop(self):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);
    if (self.playbin == None):
      return
    self.playbin.set_state(Gst.State.READY)
    self.playbin.get_state(3 * Gst.SECOND);

    self.playbin.set_state(Gst.State.NULL)
    self.playbin.get_state(3 * Gst.SECOND);
    self.playActive = False

  ####################################################
  def OnEOS(self, bus, msg):    

    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);
    self.playActive = False
    Logger.info("VideoPlayer", "EOS")
    if self._WALL_MODE == _MODE_MASTER:
      self.broadcastWallPlay()
    elif self._WALL_MODE == _MODE_OFF:
      self.playListPlay()
    if (PLController._PLAYLIST_INDEX == 0):
       self.LOOP_COUNTER += 1
       Logger.info("VideoPlayer",  "Loop Counter: " + str(self.LOOP_COUNTER) )

    self.FILE_COUNTER += 1
    Logger.info("VideoPlayer",  "File Counter: " + str(self.FILE_COUNTER) )
    #if self.FILE_COUNTER == self.KILL_COUNT:
    #  Logger.info("VideoPlayer",  "KILL_COUNT reached. Terminating Program")
    #  self.onSignalDbusExitUI(None)


  #####################################################
  def OnError(self, bus, msg):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name + " " + str(msg.parse_error()) );
    ErrorType = str(msg.parse_error())
    if(("not-supported" in ErrorType) or  ("stream error" in ErrorType) or ("stream stopped" in ErrorType) or ("This file is incomplete" in ErrorType)  ):
      Logger.warning("VideoPlayer",  'OnError():' + str(msg.parse_error()))
      Logger.warning("VideoPlayer",  "Delete Corrupt file")
      PLController.deleteCorruptFile()
      # Stream errors might make valid videos appear corrupt if we don't restart
      # the play list.
      self.playListPlay();
    elif ("not found" in ErrorType):
      Logger.warning("VideoPlayer",  "Missing file " + self.uri)
      PLController.deleteMissingFile()
    elif ("Failed to start" in ErrorType):
      pass
    else:  
      Logger.error("VideoPlayer", 'OnError():' + str(msg.parse_error()))
      refreshDatabase.insertSystemMessage("VideoPlayer", "Error",  "Misc Error")
      Logger.info("VideoPlayer", "exit")
      self.onSignalDbusExitUI(None)

      


  #####################################################
  def OnSyncMessage(self, bus, msg):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name + " " + str(msg.type));
    if msg.type == Gst.MessageType.ELEMENT:
      if msg.has_name('prepare-window-handle'):
	try:
	  msg.src.set_window_handle(self.xid)
	except:
          Logger.critical("VideoPlayer", "Error occured while setting window handle")


  ########################################################   
  def onSignalBroadcastInitVideoPlayer(self, uDPWorker):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);
    self.initVideoPlayer()


  ###########################################################
  def onSignalBroadcastSetPath(self, uDPWorker, videPath):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name + " " + videPath);
    # SET NEW VIDEO PATH IF REQUESTED FROM MASTER
    self.updateWallMode()
    if self._WALL_MODE != _MODE_OFF:
      Logger.info("VideoPlayer", "on Signal Broadcast set new path: " + videPath)
      refreshDatabase.saveVideoPath( videPath )
      self.playListInit()

  #############################################################
  def onSignalDbusInitVideoPlayer(self, uDBusWorker):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);
    self.initVideoPlayer()



  ##############################################################
  def onSignalBroadcastStart(self, uDPWorker):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);
    # START STREAMING IF REQUESTED FROM MASTER
    if self._WALL_MODE != _MODE_OFF:
      self.label.hide()
      self.playListPlay()


  #############################################################
  def onSignalBroadcastStop(self, uDPWorker):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);
    # STOP STREAMING IF REQUESTED FROM MASTER
    if self._WALL_MODE != _MODE_OFF:
      self.gstreamerStop()


  ############################################################
  def onSignalBroadcastSetMode(self, uDPWorker, mode):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name + " " + str(mode) );
    # SET NEW MODE IF REQUESTED FROM MASTER
    # SHOW IP ADDRESS
    self.updateWallMode()
    if self._WALL_MODE != _MODE_OFF:
      refreshDatabase.dictionarySet("wallMode", mode)
      self._WALL_MODE = mode
      
      # RELOAD THE PLAYLIST
      self.playListInit()


  ##############################################################
  def onSignalBroadcastShowIpAddress(self, uDPWorker):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);
    self.label.set_markup('<span size="56000">'+self.uDPWorker.getIPAddress()+'</span>')
    self.label.show()


  #############################################################
  def onSignalDbusLockDB(self, uDBusWorker):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);
    return

  #############################################################
  def onSignalDbusUnLockDB(self, uDBusWorker):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);
    return

  #############################################################
  def onSignalSetMRSS(self, uDBusWorker, url):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name + " " + url);
    #save url in database... proceed as if set playlist

    self.onSignalSetPlaylist(uDBusWorker, url)
    refreshDatabase.dictionarySet("revelEnabled", "1")
    refreshDatabase.dictionarySet("revelUrl", url)
   

  #############################################################
  def onSignalSetPlaylist(self, uDBusWorker, url):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name + " " + url);

    chunks = urlparse.urlsplit(url)
    filePath = chunks[2]
    fname = filePath.split('/')[-1]
    Logger.info("VideoPlayer", 'Set playlist '+ fname)
    
    if self.Downloader.downloading:
      msg = "Request for " + fname + " ignored. Previous download not complete"
      Logger.warning("VideoPlayer", msg)
      return

    xml =  "channelvalet" in url
    if  xml == False:
        refreshDatabase.insertSystemMessage("VideoPlayer", "Message",  "Downloading new playlist " + fname)
    self.instantToPlaylist(url)   #"instant" file
    
 
  #############################################################
  def onSignalDbusExitUI(self, uDBusWorker):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);
    # EXIT THE APPLICATION
    
    sys.stdout.flush()
    self.uDPWorker.terminate()
    self.uDPWorker.join()
    
    self.dBusWorker.terminate()
    self.dBusWorker.join()

    self.Downloader.terminate()
    self.Downloader.join()

    
    Gtk.main_quit()    


  #############################################################
  def onSignalDbusHideUI(self, uDBusWorker):    
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);
    # HIDE THE WINDOW
    Logger.debug("VideoPlayer", "hide")

    try:
       self.gstreamerStop()
    except:
       pass
    self.window.set_keep_above(False)
    self.ui_hidden = True
    self.window.hide()
    return


  ############################################################
  def onSignalDbusShowUI(self, uDBusWorker):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);
    # SHOW THE WINDOW
    print "show ui", self.ui_hidden
    self.window.set_keep_above(True)
    self.window.show()
    self.ui_hidden = False
    return
  
  
  ###########################################################
  def broadcastWallPlay(self):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);
    # BROADCAST TO START VIDEOS
    # SHOULD INVOKE ONLY MASTER
    udp = UDPWorker()

    # send the broadcast to start videos.
    # if we fail, we should start the video manually so
    # the EOS will trigger another attempt when the video has finished
    try:
      udp.broadcastStartVideo()
    except:
      self.playListPlay()
    del udp



  #########################################################
  def playListInit(self):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);
    PLController.reset()
    PLController.setWallEnabled( False if self._WALL_MODE == _MODE_OFF else True )
   
  #######################################################
  def playNextSlide(self):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);
    if (time.time() - self.slide_time < self.slide_delay):
        GObject.timeout_add_seconds(.5, self.playNextSlide)
    else:
      self.playListPlay()
    return False

  #######################################################
  def playListPlay(self):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);
        
    try:
      if ".png" in self.uri or ".jpg" in  self.uri:
          self.playbin.set_state(Gst.State.PAUSED)
          try:
             delay = int(refreshDatabase.dictionaryGet("slideShowDelay")) 
             if (delay < 5):
                 delay = 5
             if delay > 180:
                 delay = (delay-180) * 60
          except:
              delay = 15
          self.uri = "" #clear so next time thru we load next file
          self.slide_time = time.time()
          self.slide_delay = delay
          GObject.timeout_add_seconds(delay, self.playNextSlide)
          return
      else:
         self.playbin.set_state(Gst.State.NULL)

      self.uri = self.playListGetNewPath()

      if self.uri != "":
        refreshDatabase.UpdateStats(self.uri, "")
        self.gstreamerPipeDestroy()
        #time.sleep(1.0)
        self.gstreamerPipeBuild(self.uri)
        
        #self.playbin.set_property('uri', self.uri)
        self.gstreamerPlay()
        

      else:
         Logger.debug("VideoPlayer", "Playlist empty")
         GObject.timeout_add_seconds(60, self.OnEOS, None, None)


    except:
      traceback.print_exc(file=sys.stdout)
      Logger.critical("VideoPlayer", "GStreamer fail - Running ServiceUI")
      self.corners.emitSignal("signal-show-service-ui")
  

  ########################################################  
  def playListGetNewPath(self):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);

    return PLController.getVideoPath()
 

  ########################################################
  def updateWallMode(self):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);
    self._WALL_MODE = refreshDatabase.dictionaryGet("wallMode")     
      
  ########################################################
  def initVideoPlayer(self):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);
 
    if (self.ui_hidden):
       return

    # Clear out MRSS schedule if its disabled
    revelEnabled = refreshDatabase.dictionaryGet( "revelEnabled" );
    if int(revelEnabled) == 0 :
       # Delete the entire extended file info
       # Maybe in the future we can do something
       # smarter here, but for now this is good enough
       # The problem here was that when existing files were
       # deleted and new files added, the file IDs were re-used
       # so the code applied extInfo to incorrect files.
       refreshDatabase.clearExtFileInfo()
    

    self.window.queue_draw()
    self.updateWallMode()
    self.playListInit()
    self.label.hide()
    self.LOOP_COUNTER = 0
    self.uri = ""

    # START PLAYING WALL MODE IF WALL ENABLED
    if self._WALL_MODE == _MODE_MASTER:
      self.broadcastWallPlay()
    
    # START PLAYING PLAYLIST IF WALL DISABLED
    if self._WALL_MODE == _MODE_OFF:
      self.playListPlay()


  
  ########################################################
  def isAlive(self):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);
    Logger.debug("VideoPlayer",  "VideoPlayer alive")
    return "yes"

  ########################################################
  def onDownloadFileComplete(self, arg, url):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name + " " + url);
    refreshDatabase.setFileDownloaded(url)
    

  ########################################################
  def xmlToPlaylist(self, xml_fname):
      # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name + " " + xml_fname);

      global video_path
      global tmp_video_path
    
      Logger.debug("VideoPlayer",  'Parsing ' +xml_fname)

      self.download_type = "xml"
      self.d = feedparser.parse(xml_fname)

      # loop over entries
      try:
        self.source = self.d.feed.generator
      except:
        Logger.warning("VideoPlayer", 'bad XML file ' + xml_fname)
        return

      #if xml is really an Revel MRSS file
      if self.source.find("MRSS") != -1:
        thread.start_new_thread(self.mrssToPlaylist, (xml_fname,))
        #self.mrssToPlaylist(xml_fname)
        return

      
      if self.source.find("Cornelius") == -1:
        Logger.warning("VideoPlayer", 'bad XML file' + xml_fname)
        return


      for e in self.d.entries:   #each entry is a playlist
        self.files = []
        self.url = []
        self.Downloader.resetFiles()
        for m in e.media_content: #each file in a playlist 

          # get url to file and parse out filename
          url = m['url']
          chunks = urlparse.urlsplit(url)
          filePath = chunks[2]
          fname = filePath.split('/')[-1]
          fname = fname.replace("%20", "_")
          self.files.append(fname)
          self.url.append(url)
          
          if os.path.isfile(video_path + fname):
            Logger.info("VideoPlayer", "Skipping download, " + fname + " already in directory")
          else:
            Logger.info("VideoPlayer xml2playlist",  'Downloading ' + fname)
            refreshDatabase.insertSystemMessage("VideoPlayer", "Message",  "Downloading " + fname)
            self.queue.put((url, tmp_video_path + fname))

      self.dl_count = 0
      self.queue.put((None, None))  

  def onVideoDownloadStarted(self,arg):
      # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name );
      self.dl_label.set_markup('<span size="10000">Updating Videos - ' + str(self.queue.qsize()) + ' Remaining</span>')
      self.dl_label.show()
      

  ########################################################
  def onVideoDownloadXMLComplete(self):
      # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);

      # tell Video player to stop
      self.playListInit()

      #delete old playlists
      refreshDatabase.deleteAllPlaylists()

      # everything ownloaded.  Now make playlists
      for e in self.d.entries:   #each entry is a playlist
        playlist_title = e.title

        # get start and stop times

        valid =   e.validity
        pieces = valid.split(';')
        valid_start = pieces[0].split('=')[1]
        valid_end = pieces[1].split('=')[1]
        
        #strip off seconds
        valid_start = valid_start[:valid_start.rfind(':')]
        valid_end = valid_end[:valid_end.rfind(':')]

                             
        bf = self.Downloader.bad_files

        # if any files failed remove from list
        if len(bf):
          for i in range(0, len(bf)):
            fname = bf[i][bf[i].rfind('/')+1:]
            Logger.warning("VideoPlayer", "Download failed " + fname)
            try:
              index = self.files.index(fname)
              self.files.pop(index)
              self.url.pop(index)
            except:
              index = -1
              pass
          if len(self.files) == 0:
            Logger.warning("VideoPlayer", "Download failed all files")
            self.playListPlay() #play old files
            return
        

        # update playlist in database
        refreshDatabase.insertSystemMessage("VideoPlayer", "Message",  "New playlist [" + playlist_title + "]")
        refreshDatabase.makeNewNamedPlaylist(playlist_title, self.files, self.url, valid_start, valid_end, self.source)

      # delete old CMS files
      os.system("rm /opt/Refresh/Videos/CMS/* 2>/dev/null")

      # move files from tmp
      os.system("mv  /opt/Refresh/Videos/CMS/tmp/* /opt/Refresh/Videos/CMS")

      # restart playlist
      
      self.playListInit()
      self.playListPlay()  

  ########################################################
  def mrssToPlaylist(self, rss_fname):
      # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name + " " + rss_fname);

      global video_path
      global tmp_video_path

      refreshDatabase = DatabaseHelper.DatabaseConnection("/opt/Refresh/Database/refresh_database_enc.db")
      
      Logger.info("VideoPlayer", 'Parsing ' + rss_fname)

      self.download_type = "rss"
      self.d = feedparser.parse(rss_fname)

      # loop over entries
      self.files = []
      self.url = []
      self.valid_start = []
      self.valid_end = []
      self.download_count = 0
      self.noDelete = []
      self.downloadList = []
      self.Downloader.resetFiles()
      try:
        self.source = self.d.feed.generator
      except:
        self.source = "Unknown"
      for e in self.d.entries:
          # get url to file and parse out filename
          for m in e.media_content:
            url = m['url']
            chunks = urlparse.urlsplit(url)
            filePath = chunks[2]
            fname = filePath.split('/')[-1]
            fname = fname.replace("%20", "_")
            self.files.append(fname)
            self.url.append(url)


          # get start and stop times

          valid =   e.validity
          pieces = valid.split(';')
          st = pieces[0].split('=')[1]
          et = pieces[1].split('=')[1]

          # convert to a python time and finally utc
          
          if (len(st) == 11):
            st += "00:00:00"

          st =  iso8601.parse_date(st)
          if (len(et) == 11):
            et += "23:59:59"
          et =  iso8601.parse_date(et)
          
          st_utc = int(time.mktime(st.timetuple()))
          et_utc = int(time.mktime(et.timetuple()))

          lt = time.localtime(time.time())
          if (lt.tm_isdst):
            st_utc -= 3600
            et_utc -= 3600

          print fname
          print 'st', st, st_utc, time.gmtime(st_utc)
          print 'et', et, et_utc, time.gmtime(et_utc)
          
          self.valid_start.append(st_utc)
          self.valid_end.append(et_utc)

          needToDownload = False
          
          # check if file exists

          if refreshDatabase.checkFileExists(url):
            vfname = video_path + fname
          else:
            needToDownload = True          

          if url in self.downloadList:
            Logger.info("VideoPlayer", "Duplicate download, " + fname + " already in queue")
            
            
          elif (needToDownload == False):

            Logger.info("VideoPlayer", "Skipping download, " + fname + " already in directory")
            #cmd = "nice cp " + video_path + fname + " " + tmp_video_path + fname
            #print cmd
            #os.system(cmd)
            self.noDelete.append(fname) 
      
          else:
            self.downloadList.append(url)
            self.download_count += 1
            Logger.info("VideoPlayer mrss", 'Downloading ' +  fname)
            refreshDatabase.insertSystemMessage("VideoPlayer", "Message",  "Downloading " + fname)
            self.queue.put((url, tmp_video_path + fname))
      
      self.dl_count = 0
      self.queue.put((None, None))  

  ########################################################
  def instantToPlaylist(self, url):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name + " " + url);
    global tmp_video_path

    self.download_type = "instant_video"
    chunks = urlparse.urlsplit(url)
    filePath = chunks[2]
    fname = filePath.split('/')[-1]
    fname = fname.replace("%20", "_")
    self.instant_fname = fname
    self.instant_url = url
    #Always download instant videos and might be same name, different content
    self.queue.put((url, tmp_video_path + fname))
    self.dl_count = 0
    self.queue.put((None, None))  
      
  ########################################################
  def onVideoDownloadComplete(self, arg):
    # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name );
    
    self.dl_label.hide()
    
    if self.download_type == "rss":
      self.onVideoDownloadMRSSComplete()
      refreshDatabase.clearFilesDownloaded()
    elif self.download_type == "xml":
      self.onVideoDownloadXMLComplete()
      refreshDatabase.clearFilesDownloaded()
    else:
      self.onVideoDownloadInstantComplete()

    
  ########################################################
  def onVideoDownloadMRSSComplete(self):
      # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);
      Logger.info("VideoPlayer", "Download Complete")
      bf = self.Downloader.bad_files

      # if any files failed remove from list
      for bad_file in bf:
          fname = os.path.basename(bad_file)
          refreshDatabase.insertSystemMessage("VideoPlayer", "Warning",  "Download failed " + fname)
          try:
              index = self.files.index(fname)
              self.files.pop(index)
              self.url.pop(index)
              self.valid_start.pop(index)
              self.valid_end.pop(index)
          except:
              pass
      if len(self.files) == 0 and self.download_count > 0:
        Logger.warning("VideoPlayer", "Download failed all files")
        self.playListInit()
        self.playListPlay() #play old files
        return

      #print self.url
      #print self.last_url
      #print set(self.url) != set(self.last_url)

      old_feed_path = "/opt/Refresh/Videos/CMS/%s" % self.instant_fname
      new_feed_path = "/opt/Refresh/Videos/CMS/tmp/%s" % self.instant_fname
      same_checksums = False
      if os.path.isfile(old_feed_path) and os.path.isfile(new_feed_path):
          old_checksum = os.popen("md5sum \"%s\"" % old_feed_path).read().split(" ")[0]
          new_checksum = os.popen("md5sum \"%s\"" % new_feed_path).read().split(" ")[0]
          same_checksums = old_checksum == new_checksum
          if not same_checksums:
              print("Playlist checksums changed: %s != %s" % (old_checksum, new_checksum))

      if self.download_count or (set(self.url) != set(self.last_url)) or not same_checksums:
        
        Logger.info("VideoPlayer", "Changing playlist")

        self.last_url = self.url

        # tell Video player to stop
        self.playListInit()

        # update playlist in database
        refreshDatabase.deleteAllPlaylists()
        refreshDatabase.makeNewPlaylist(self.files, self.url, self.valid_start, self.valid_end, self.source)      

        
        # delete old CMS files
        #os.system("rm /opt/Refresh/Videos/CMS/* 2>/dev/null")
        dirFiles = os.listdir(video_path)
        for f in dirFiles:
          if f in self.noDelete:
            pass
          else:
            if os.path.isfile(video_path + f):
               os.remove(video_path + f)
        
        
        # move files from tmp
        os.system("mv  /opt/Refresh/Videos/CMS/tmp/* /opt/Refresh/Videos/CMS")

        # restart playlist
        self.playListPlay()
      else:
        print "delete files tmp"
        os.system("rm /opt/Refresh/Videos/CMS/tmp/* 2>/dev/null")
        if self.playActive == False:
          self.playListPlay()

  ########################################################
  def onVideoDownloadInstantComplete(self):
      # Logger.debug("defVideoPlayer", sys._getframe().f_code.co_name);
      fname = "/opt/Refresh/Videos/CMS/tmp/" + self.instant_fname

      #if we just downloaded a xml or .rss file go recursive
      if fname.find(".xml") != -1:
        self.xmlToPlaylist(fname)     #cornelius file
      elif fname.find(".rss") != -1 :
        self.mrssToPlaylist(fname)    #revel file
      else:
    
        #stop video in case it has the same filename
        self.gstreamerStop()
        
        # delete old CMS files
        os.system("rm /opt/Refresh/Videos/CMS/* 2>/dev/null")

        # move file to Videos Directory
        os.system("mv "+  fname + " /opt/Refresh/Videos/CMS")

        # delete current playlist and add this file to it
        refreshDatabase.deleteAllPlaylists()
        try:
          refreshDatabase.makeInstantPlaylist(fname[fname.rfind('/',0, len(fname))+1:len(fname)], self.instant_url)
          refreshDatabase.insertSystemMessage("VideoPlayer", "Message",  "New instant file playlist [" + fname + "]")
        except:
          refreshDatabase.insertSystemMessage("VideoPlayer", "Error",  "Bad instant playlist file[ " + fname + "]")
          Logger.warning("VideoPlayer", 'bad filename ' + fname)
          traceback.print_exc(file=sys.stdout)
        self.playListInit()
        self.playListPlay()

GObject.type_register(UDPWorker)
GObject.signal_new("signal_broadcast_set_video_path", 		UDPWorker, GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE, (GObject.TYPE_STRING, ))
GObject.signal_new("signal_broadcast_set_mode", 		UDPWorker, GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE, (GObject.TYPE_STRING, ))
GObject.signal_new("signal_broadcast_start", 			UDPWorker, GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE, ())
GObject.signal_new("signal_broadcast_stop", 			UDPWorker, GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE, ())
GObject.signal_new("signal_broadcast_initialize_video_player", 	UDPWorker, GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE, ())
GObject.signal_new("signal_broadcast_show_ip_address", 		UDPWorker, GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE, ())

GObject.type_register(DBusWorker)
GObject.signal_new("signal_dbus_show_ui", 			DBusWorker, GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE, ())
GObject.signal_new("signal_dbus_hide_ui", 			DBusWorker, GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE, ())
GObject.signal_new("signal_dbus_exit_ui", 			DBusWorker, GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE, ())
GObject.signal_new("signal_dbus_init_video_player", 		DBusWorker, GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE, ())
GObject.signal_new("signal_dbus_set_playlist",   		DBusWorker, GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE, (GObject.TYPE_STRING, ) )
GObject.signal_new("signal_dbus_set_mrss",   		DBusWorker, GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE, (GObject.TYPE_STRING, ) )
GObject.signal_new("signal_show_service_ui", 			Overlay, GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE, ())
GObject.signal_new("signal_dbus_lock_db", 			DBusWorker, GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE, ())
GObject.signal_new("signal_dbus_unlock_db", 			DBusWorker, GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE, ())

GObject.type_register(Downloader)
GObject.signal_new("signal_download_complete", 			Downloader, GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE, ())
GObject.signal_new("signal_download_file_complete", 		Downloader, GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE,(GObject.TYPE_STRING, ))
GObject.signal_new("signal_download_started", 		Downloader, GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE,())


# START THE MAIN WINDOW AND GTK THREAD
start=Main()

if len(sys.argv) > 1:
  if sys.argv[1] == "hide":
    start.onSignalDbusHideUI(None)
GObject.threads_init()
Gtk.main()

